% Reads all excel files available and creates the structure structForecasts
% Also to be run when the predefined list of unique contributors has changed ID 

    clear
    clc;
    close all

    addpath(genpath([cd '\']));

    sOutputDirectory     = 'CE_Mat_files\';
    
%%  Define variables
    
    bCreate_structForecasts     = 1;
            
%% Define the countries for which the survey is being conducted
    vStringCountries_General        = fdefineNamesCountries_newformat;
    
%%  Define the variables that are being surveyed and the units in which they are reported 
    structVariables                 = fdefineVariables_newformat;

%%  Define the names of contributors
    sFileTitle_MasterContributors   = 'List_Contributors.xlsx';
    [~,Header_Contributors]         = xlsread(sFileTitle_MasterContributors, 'Master');
    % Name of the institution is found on the second column
    sListNamesContributors          = Header_Contributors(2:end,2);
    % Master name of the institution is found on the third column
    sListMasterNamesContributors    = Header_Contributors(2:end,3);
    
%% Load structure with excel data
    
    % For the moment this is a copy - work in progress
    load([sOutputDirectory 'structExcelData.mat']);
    
%%  
    
    if bCreate_structForecasts
    
        structForecasts = repmat(struct([]),length(vStringFileTitle),1);
    
        
        for iFile = 1:length(vStringFileTitle)
            
            iFile
            
            nSheets         = length(structExcelData(iFile).Country);
            sFileTitle      = vStringFileTitle{iFile}
                            
            % Read the data from each sheet
            for iSheet = 1:nSheets
                
                sCountry    = structExcelData(iFile).Country(iSheet).CountryName;
                Data        = structExcelData(iFile).Country(iSheet).Data;
                Header      = structExcelData(iFile).Country(iSheet).Header;
                
                if size(Data,1)>15
                    
                    % Delete THE blank column, if any
                    [Header, Data]      = fDeleteBlankColumn(Header, Data);

                    % Check that indeed the name of the country that appears in the first cell of each sheet corresponds to the name of the sheet
                    if ~strcmp(Header(1,1), sCountry)
                        sCountry 
                    end
                    
                    %% The date of the survey
                    % According to the user guide, the data is always located in the A4 cell 
                    sDate               = Header(4,1);
                    try 
                        dateSurvey      = datenum(sDate);
                    catch
                        dateSurvey      = datenum(sDate,'dd.mm.yyyy');
                    end
%                    
%                     try
%                     % Convert the string to a datetime object
%                     dateTimeObj = datetime(sDate, 'InputFormat', 'auto');
%                     % Convert the datetime object to a numeric serial date
%                     dateSurvey= datenum(dateTimeObj);
%                     catch ME
%                     % If conversion fails, display an error and set output to NaN
%                     warning('Failed to parse the date string: %s', char(sDate));
%                     dateSurvey = NaN;
%                     end
                    % dateSurvey      = datenum(sDate);
                    %% Check
                    % If there is a forecast available for the for past year. The
                    % only month where this happens is Jan 1990
                    
                    year_DateSurvey             = year(dateSurvey);
                
                    dIndexLineYears_Data        = find(Data(:,1)== year_DateSurvey);
                    
                    if numel(dIndexLineYears_Data)==0 
                        sFileTitle
                        dIndexLineYears_Data    = find(Data(:,2)== year_DateSurvey);
                    end
                
                 %% 2. CONTRIBUTORS - Identify the contributors for the country
                    % Assumption: According to the user guide, the list of contributors starts always on row 26
                    % It ends when there are no more contributors (the line is blank)
                    vStringNamesContributors    = {};
                    bContributors               = true;
                    dStartLine_ListContributors = 26; 
                    index_Contributor           = 0;
                    
                    while bContributors&&(dStartLine_ListContributors+index_Contributor<=size(Header,1))
                       if ~strcmp(Header(dStartLine_ListContributors+index_Contributor,1), '') 
                            if ~strcmp(deblank(Header(dStartLine_ListContributors+index_Contributor,1)),'')
                                vStringNamesContributors = [vStringNamesContributors; Header(dStartLine_ListContributors+index_Contributor,1)];
                            end
                            index_Contributor = index_Contributor + 1;
                       else
                           bContributors = 0;
                       end  
                    end          
                    
                    % Delete the useless spaces from the names
                    vStringNamesContributors    = strtrim(vStringNamesContributors);
                    
                    %% Delete contributors that provide no data for none of the variables 
                    
                    % Forecasts data start from the end of the Data - length(vStringNamesContributors) to the back
                    data_all                                                        = Data(end-length(vStringNamesContributors)+1:end,:);
                    index_deleteContributors_noForecasts                            = find(sum(isnan(data_all),2)==size(data_all,2));
                    vStringNamesContributors(index_deleteContributors_noForecasts)  = [];
                    
                    %% Check whether there is a new contributor not previously
                    % defined in the Master List of contributors
                    if length(sListNamesContributors(ismember(sListNamesContributors,vStringNamesContributors)))<length(vStringNamesContributors)
                         if sum(~ismember(vStringNamesContributors, sListNamesContributors)) ==0
                             fprintf('There is a name repeated in the list of the country')
                             sCountry
                             fprintf('from file')
                             sFileTitle
                             return
                         else 
                             fprintf('New contributor(s)')
                             vStringNamesContributors(~ismember(vStringNamesContributors, sListNamesContributors))   
                             fprintf('for country')
                             sCountry
                             fprintf('in file for ')
                             sFileTitle
                             return
                         end
                    end
                    
                    %%  Assigning names of contributors to IDs of contributors (according to the predefined list)
                    
                    % Find the order of the original unsorted list of names in the sorted list matched with the predefined names of institutions
                    indexNames_SortedList       = cellfun(@(InstitutionName) find(strcmp(sListNamesContributors(ismember(sListNamesContributors,vStringNamesContributors)),InstitutionName)), vStringNamesContributors);

                    vStringMasterNames_sorted   = sListMasterNamesContributors(ismember(sListNamesContributors,vStringNamesContributors));
                    vStringMasterNames_unsorted = cell(length(vStringMasterNames_sorted),1);
                    
                    for iNames = 1:length(vStringMasterNames_unsorted)
                        vStringMasterNames_unsorted(iNames) = vStringMasterNames_sorted(indexNames_SortedList(iNames));
                    end

                    if length(vStringNamesContributors)~= length(vStringMasterNames_unsorted)
                        if sum(~ismember(vStringNamesContributors, sListNamesContributors))==0
                            fprintf('Please check the institution that appears at least twice in the list')
                        else
                            sFileTitle
                            sCountry
                            return
                        end
                    end
                    
                 %% 3. VARIABLE: Identify the variables for which the information is being reported
                    
                    % Assumption: this type of information starts in the third colum of the Header
                    jVariable = 0;
                    
                    bVariable_3Columns = 0;
                    
                    if mod(size(Data,2),2)
                        nColumns = size(Data,2)-1;
                    else
                        nColumns = size(Data,2);
                    end
                        
                    for jColumn_Data = 1:2:nColumns
                        
                        jColumn_Header  = jColumn_Data+2+bVariable_3Columns;
    
                        bFoundVariable      = 0; % initialise the boolean variable indicating whether the variable is found or not
                        bTypicalVariable    = 0;
                        bAtypicalVariable   = 0;
                        clear sNameVariable
                        
                        % Read the name of the forecasted variable 
                        % The name of the variable must be composed from either the 3rd line of the Header variable
                        % (corresponding column) and the 4th line of the variable (if this is not blank)
                        % The forecasted values for one variable spreads over two columns. The sole exception is the
                        % variable: Monetary Policy Expectations that spreads over three columns and this impacts
                        % everything else afterwards once encountered
                        if ~strcmp(Header(3,jColumn_Header),'') % if there is a simple variable that takes two columns
                            if (strcmp(Header(4, jColumn_Header),' '))||(strcmp(Header(4, jColumn_Header),'')) % if the name of the variable is found on one line
                                sNameVariable = strtrim(Header(3, jColumn_Header));
                            else % if the name of the variable takes two lines
                                sNameVariable = strcat(strtrim(Header(3,jColumn_Header)),{' '},strtrim(Header(4, jColumn_Header)));
                            end
                            bTypicalVariable    = 1;
                            
                        else % the sole exception is the Monetary Policy Expectations questions
                           bVariable_3Columns   = 1;
                           jColumn_Header       = jColumn_Header + bVariable_3Columns;
                           sNameVariable        = strcat(strtrim(Header(3,jColumn_Header)),{' '},strtrim(Header(4, jColumn_Header)), {' '},strtrim(Header(5, jColumn_Header)));
                           bAtypicalVariable    = 1;
                            
                        end
                            
                        if ~strcmp(sNameVariable, '')&&bTypicalVariable

                            % Read the unit of the variable
                            % Assumption: this information is located on the 3th line before the name of the country 
                            sUnitVariable = Header(5, jColumn_Header);

                            % Check 
                            % Check whether the variable read matches the defined ones
                            % 1. check - is the variable in the predefined list of names of variables?
                            % 2. check - is the unit of the variable in the predefined list of units for that specific variables?

                            
                            % First find the variable in the array of structures of variables
                            indexlist_Variable  = 1;
                            
                            while (~bFoundVariable)&&(indexlist_Variable<=length(structVariables))
                                if sum(strcmp(structVariables(indexlist_Variable).Names, sNameVariable))==1 % if the variable is found
                                    if sum(strcmp(structVariables(indexlist_Variable).Units, sUnitVariable))==1 % if the unit of measure corresponds
                                        bFoundVariable = 1;
                                    else
                                        % problems with the units of the variable
                                        sFileTitle
                                        iFile
                                        sCountry
                                        indexlist_Variable
                                        fprintf('Problems with the UNITS of the variable')
                                        sNameVariable
                                        sUnitVariable
                                        return
                                    end
                                else
                                   indexlist_Variable = indexlist_Variable+1; 
                                end
                            end

                            if ~bFoundVariable % problems with the name of the variable
                                sFileTitle
                                iFile
                                sCountry
                                fprintf('Problems with the NAME of the variable')
                                sNameVariable
                                return
                            end    

                        end
                        
                        if bFoundVariable 

                            %% Check whether there is actually any data forecasted

                            % Read the forecasted values
                            nLines_OriginalData = length(index_deleteContributors_noForecasts) + length(vStringNamesContributors);
                            
                            data1 = Data(end-nLines_OriginalData+1:end,jColumn_Data+bVariable_3Columns);
                            data2 = Data(end-nLines_OriginalData+1:end,jColumn_Data+bVariable_3Columns+1);
                            
                            data1(index_deleteContributors_noForecasts,:) = [];
                            data2(index_deleteContributors_noForecasts,:) = [];
                            
                            if (length(data1)~= length(vStringNamesContributors))||(length(data2)~= length(vStringNamesContributors))  
                                    fprintf('Diferente')
                                    sFileTitle
                            end
                            
                            if (sum(~isnan(data1))>0)||(sum(~isnan(data2))>0)

                                structForecasts(iFile).Forecast(iSheet).Country               = sCountry;
                                structForecasts(iFile).Forecast(iSheet).Names_Contributors    = vStringNamesContributors;
                                structForecasts(iFile).Forecast(iSheet).Contributors          = vStringMasterNames_unsorted;    
                                structForecasts(iFile).Forecast(iSheet).Date_Survey           = dateSurvey;
                                
                                % Now we are sure that there is data and that this is correct  
                                jVariable = jVariable + 1;

                                structForecasts(iFile).Forecast(iSheet).Variables(jVariable).Variable_category = char(structVariables(indexlist_Variable).Variable);
                                structForecasts(iFile).Forecast(iSheet).Variables(jVariable).Variable_name     = char(sNameVariable);
                                structForecasts(iFile).Forecast(iSheet).Variables(jVariable).Variable_units    = char(sUnitVariable);

                                structForecasts(iFile).Forecast(iSheet).Variables(jVariable).Nowcast.IndividualData   = data1;
                                structForecasts(iFile).Forecast(iSheet).Variables(jVariable).Forecast.IndividualData  = data2;
                                  
                                % Read the years
                                % 4. Nr of times the variable is forecasted
                                % For a classical variable there are always 2 forecasts available 
                                % This info is either read by Matlab as numbers - case in which it can be found
                                % in the Data variable, or as string - case in which it can be found in the Header variable
                                
                                if strcmp(Header(6, jColumn_Header),'') % in this case the info is a number (the year for which the forecast is done)
                                    vStringVariables    = {num2str(Data(1,jColumn_Data+bVariable_3Columns)); num2str(Data(1,jColumn_Data+bVariable_3Columns+1))}; 
                                else % in this case the info is either a string 
                                    vStringVariables    = [Header(6, jColumn_Header); Header(1, jColumn_Header+1)];
                                end
                                
                                structForecasts(iFile).Forecast(iSheet).Variables(jVariable).Nowcast.YearForecast     = vStringVariables(1);
                                structForecasts(iFile).Forecast(iSheet).Variables(jVariable).Forecast.YearForecast    = vStringVariables(2);

                                %% Compute also statistics

                                % Mean forecast
                                structForecasts(iFile).Forecast(iSheet).Variables(jVariable).Nowcast.Mean     = nanmean(data1);
                                structForecasts(iFile).Forecast(iSheet).Variables(jVariable).Forecast.Mean    = nanmean(data2);

                                % Highest forecast
                                structForecasts(iFile).Forecast(iSheet).Variables(jVariable).Nowcast.Max      = nanmax(data1);
                                structForecasts(iFile).Forecast(iSheet).Variables(jVariable).Forecast.Max     = nanmax(data2);

                                % Lowest forecast
                                structForecasts(iFile).Forecast(iSheet).Variables(jVariable).Nowcast.Min      = nanmin(data1);
                                structForecasts(iFile).Forecast(iSheet).Variables(jVariable).Forecast.Min     = nanmin(data2);

                                % Highest - Lowest forecast
                                structForecasts(iFile).Forecast(iSheet).Variables(jVariable).Nowcast.MaxMinDiff      = nanmax(data1) - nanmin(data1);
                                structForecasts(iFile).Forecast(iSheet).Variables(jVariable).Forecast.MaxMinDiff     = nanmax(data2) - nanmin(data2);

                                % Standard deviation
                                structForecasts(iFile).Forecast(iSheet).Variables(jVariable).Nowcast.StandardDeviation    = nanstd(data1);
                                structForecasts(iFile).Forecast(iSheet).Variables(jVariable).Forecast.StandardDeviation   = nanstd(data2);

                                % Median
                                structForecasts(iFile).Forecast(iSheet).Variables(jVariable).Nowcast.Median       = nanmedian(data1);
                                structForecasts(iFile).Forecast(iSheet).Variables(jVariable).Forecast.Median      = nanmedian(data2);
                                
                                % Skewness
                                structForecasts(iFile).Forecast(iSheet).Variables(jVariable).Nowcast.Skewness     = skewness(data1);
                                structForecasts(iFile).Forecast(iSheet).Variables(jVariable).Forecast.Skewness    = skewness(data2);
                                
                                % Kurtosis
                                structForecasts(iFile).Forecast(iSheet).Variables(jVariable).Nowcast.Kurtosis     = kurtosis(data1);
                                structForecasts(iFile).Forecast(iSheet).Variables(jVariable).Forecast.Kurtosis    = kurtosis(data2);
                                
                                % 25th percentile
                                structForecasts(iFile).Forecast(iSheet).Variables(jVariable).Nowcast.Percentile25       = prctile(data1(~isnan(data1)),25);
                                structForecasts(iFile).Forecast(iSheet).Variables(jVariable).Forecast.Percentile25      = prctile(data2(~isnan(data2)),25);
                                
                                % 75th percentile
                                structForecasts(iFile).Forecast(iSheet).Variables(jVariable).Nowcast.Percentile75       = prctile(data1(~isnan(data1)),75);
                                structForecasts(iFile).Forecast(iSheet).Variables(jVariable).Forecast.Percentile75      = prctile(data2(~isnan(data2)),75);
                               
                                % 10th percentile
                                structForecasts(iFile).Forecast(iSheet).Variables(jVariable).Nowcast.Percentile10       = prctile(data1(~isnan(data1)),10);
                                structForecasts(iFile).Forecast(iSheet).Variables(jVariable).Forecast.Percentile10      = prctile(data2(~isnan(data2)),10);
                                
                                % 90th percentile
                                structForecasts(iFile).Forecast(iSheet).Variables(jVariable).Nowcast.Percentile90       = prctile(data1(~isnan(data1)),90);
                                structForecasts(iFile).Forecast(iSheet).Variables(jVariable).Forecast.Percentile90      = prctile(data2(~isnan(data2)),90);
                               
                                % IQR - Dovern et. al measure - 75th percentile - 25th percentile
                                structForecasts(iFile).Forecast(iSheet).Variables(jVariable).Nowcast.IQR_7525       = ...
                                    structForecasts(iFile).Forecast(iSheet).Variables(jVariable).Nowcast.Percentile75 - structForecasts(iFile).Forecast(iSheet).Variables(jVariable).Nowcast.Percentile25;
                                structForecasts(iFile).Forecast(iSheet).Variables(jVariable).Forecast.IQR_7525      = ...
                                    structForecasts(iFile).Forecast(iSheet).Variables(jVariable).Forecast.Percentile75 - structForecasts(iFile).Forecast(iSheet).Variables(jVariable).Forecast.Percentile25;
                                
                                % IQR - Dovern et. al measure - 90th percentile - 10th percentile
                                structForecasts(iFile).Forecast(iSheet).Variables(jVariable).Nowcast.IQR_9010       = ...
                                    structForecasts(iFile).Forecast(iSheet).Variables(jVariable).Nowcast.Percentile90 - structForecasts(iFile).Forecast(iSheet).Variables(jVariable).Nowcast.Percentile10;
                                structForecasts(iFile).Forecast(iSheet).Variables(jVariable).Forecast.IQR_9010      = ...
                                    structForecasts(iFile).Forecast(iSheet).Variables(jVariable).Forecast.Percentile90 - structForecasts(iFile).Forecast(iSheet).Variables(jVariable).Forecast.Percentile10;
                                
                                
                            end
                            
                            
                        end
                        
                        
                    end
                
                end
            
            end
            
            % Delete the empty structures corresponding to countries for
            % which there was no contributor and implicitly no data
            index_delete = [];
            for iCountry = 1:length(structForecasts(iFile).Forecast)
               if isempty(structForecasts(iFile).Forecast(iCountry).Names_Contributors)
                    index_delete = [index_delete iCountry];
               end
            end
            structForecasts(iFile).Forecast(index_delete) =[];   
            
        end

    end
    
    
%%  Save the structure
    save([sOutputDirectory 'structForecasts_NewFormat_raw.mat'], 'structForecasts');
    