% This file creates a Matlab structure from the monthly Excel files in the
% old format (1999 - 2004)

    clear
    clc;
    close all

    addpath(genpath([cd '\']));

    sInputDirectory     = 'CE_Excel_Files\';
    sOutputDirectory    = 'CE_Mat_files\';

%%  Define variables
    
    % Read all excel files available and create the structure
    % structForecasts - also to be run when the predefined list of unique
    % contributors has changed ID 
    bCreate_structForecasts     = 1;
                
%% Define the countries for which the survey is being conducted
    [vStringCountries_General, vStringCountries_newformat]        = fdefineNamesCountries;
    
%%  Define the variables that are being surveyed and the units in which they are reported 
    structVariables                 = fdefineVariables_newformat;

%%  Define the names of contributors
    sFileTitle_MasterContributors   = 'List_Contributors.xlsx';
    [~,Header_Contributors]         = xlsread(sFileTitle_MasterContributors, 'Master');
    % Name of the institution is found on the second column
    sListNamesContributors          = Header_Contributors(2:end,2);
    % Master name of the institution is found on the third column
    sListMasterNamesContributors    = Header_Contributors(2:end,3);
    
%%  Names of the Excel files containing the data

    vYears                          = 1989:1:2004;
    vStringMonths_smallcaps         = {'jan', 'feb', 'mar', 'apr', 'may', 'jun','jul','aug','sep','oct','nov','dec'};
    vStringMonths_bigcaps           = {'JAN', 'FEB', 'MAR', 'APR', 'MAY', 'JUN','JUL','AUG','SEP','OCT','NOV','DEC'};
    vStringMonths_firstletterbig    = {'Jan', 'Feb', 'Mar', 'Apr', 'May', 'Jun','Jul','Aug','Sep','Oct','Nov','Dec'};

    vStringFileTitle                = fCreateNames_CFfiles_oldformat(vYears, vStringMonths_smallcaps, vStringMonths_bigcaps, vStringMonths_firstletterbig, sInputDirectory);
   
    structForecasts_OldFormat       = repmat(struct([]),length(vStringFileTitle),1);
    
%%  
    if bCreate_structForecasts
        
        for iFile = 1:length(vStringFileTitle)

            iFile

            %   Read data and info from file
                sFileTitle          = vStringFileTitle{iFile}
                [Data, Header]      = xlsread(sFileTitle);

            %%  Delete THE blank column, if any
                [Header, Data]      = fDeleteBlankColumn(Header, Data);

                %% Check
                % Data has always 30 columns and Header 32 columns. As
                % of Dec 2002 forecasts become available also for EuroZone and
                % Data has 41 columns and Header 43 columns
                if ~(size(Data,2)==30&&size(Header,2)==32)
                    if ~(size(Data,2)==41&&size(Header,2)==43)
                    iFile
                    sFileTitle
                    end
                end

            %% Find the date of the survey

                % This is the info in the cell one line before the date of the survey
                sTitleDate          = 'Survey Date:';
                indexColumnDate     = 1;
                % Find the position of the string 'Survey Date:' in the first column of Header
                indexDate           = find(strcmp(Header(:,indexColumnDate),sTitleDate));
                sDate               = Header(indexDate+1,indexColumnDate);
                try 
                    dateSurvey = datenum(sDate,'dd/mm/yyyy');
                catch
                    dateSurvey = datenum(sDate,'dd.mm.yyyy');
                end
                year_DateSurvey = year(dateSurvey);

    %             %% Check 
    %             % Post together the name of the file together with the year and
    %             % month of the survey date and see whether there are any discrepancies
    %                 month_DateSurvey = month(dateSurvey);
    %                 [month_DateSurvey year_DateSurvey]
    %                 sFileTitle
    %                 pause(1)
    %       
            %%  Identify the countries and the position of each countries in the Header and Data 

                % Assumption: name of countries are on the 1st column of the Header
                indexColumnsCountries_Header    = 1;
                % Find the index of lines in the Header where the info for countries can be found
                vIndexLinesCountries_Header     = find(ismember(Header(:,indexColumnsCountries_Header),vStringCountries_General)==1);
                nCountriesSurveyed              = length(vIndexLinesCountries_Header);


    %             %% Check
    %             % See as of when is data for EuroZone also available
    %             if nCountriesSurveyed~= 12
    %                 nCountriesSurveyed
    %                 sFileTitle
    %             end

                vIndexLinesYears_Data           = find(Data(:,1)== year_DateSurvey);

                %% Check
                % If there is a forecast available for the past year. The
                % only month where this happens is Jan 1990
                if numel(vIndexLinesYears_Data)==0 
                    sFileTitle
                    vIndexLinesYears_Data       = find(Data(:,2)== year_DateSurvey);
                end


            %%  Read variables and data
                for iCountry = 1:nCountriesSurveyed

    %                 iCountry

                    %% 1. COUNTRY: Identify the name country for which the variables are available 
                    sCountry = Header(vIndexLinesCountries_Header(iCountry),indexColumnsCountries_Header);

                    %% 2. CONTRIBUTORS. Identify the contributors for the country: 
                    % Assumption: the list of contributors starts one line after the
                    % name of the country and ends there are no more contributors (the line is blank)
                    vStringNamesContributors    = {};
                    bContributors               = true;
                    index = 1;
                    while bContributors
                       if ~strcmp(Header(vIndexLinesCountries_Header(iCountry)+index,indexColumnsCountries_Header), '') 
                            if ~strcmp(deblank(Header(vIndexLinesCountries_Header(iCountry)+index,indexColumnsCountries_Header)),'')
                                vStringNamesContributors = [vStringNamesContributors; deblank(Header(vIndexLinesCountries_Header(iCountry)+index,indexColumnsCountries_Header))];
                            end
                            index = index + 1;
                       else
                           bContributors = 0;
                       end  
                    end

                    %% Delete contributors that provide no data for none of the variables 
                    
                    nLinesDiff_HeaderData = 23;

                    if strcmp(Header(vIndexLinesYears_Data(iCountry)+nLinesDiff_HeaderData), sCountry)
                        data_all = Data(vIndexLinesYears_Data(iCountry)+3:vIndexLinesYears_Data(iCountry)+length(vStringNamesContributors)+2,:);
                    else % for the moment the sole exception is the EuroZone. The way to treat for the moment this exception is to consider directly the vIndexLinesYears_Data(iCountry+1)
                        data_all = Data(vIndexLinesYears_Data(iCountry+1)+3:vIndexLinesYears_Data(iCountry+1)+length(vStringNamesContributors)+2,:);
                    end
                            
                    index_deleteContributors_noForecasts = find(sum(isnan(data_all),2)==size(data_all,2));
                    vStringNamesContributors(index_deleteContributors_noForecasts) = [];
                    
                    %% Other checks
                    
                    % Check whether there is a new contributor not previously
                    % defined in the Master List of contributors
                    if length(sListNamesContributors(ismember(sListNamesContributors,vStringNamesContributors)))<length(vStringNamesContributors)
                         if sum(~ismember(vStringNamesContributors, sListNamesContributors)) ==0
                             fprintf('There is a name repeated in the list of the country')
                             sCountry
                             fprintf('from file')
                             sFileTitle
                             pause(2)
    %                          return
                         else 
                             fprintf('New contributor(s)')
                             vStringNamesContributors(~ismember(vStringNamesContributors, sListNamesContributors))   
                             fprintf('for country')
                             sCountry
                             fprintf('in file for ')
                             sFileTitle
                             fprintf('Please introduce the contributor in the Master file')
                             sFileTitle_MasterContributors
                             pause(2)
                             return
                         end
                    end

                    %%  CONTRIBUTORS - Grouped
                    % Find the order of the original unsorted list of names in the sorted list matched with the predefined names of institutions
                    indexNames_SortedList       = cellfun(@(InstitutionName) find(strcmp(sListNamesContributors(ismember(sListNamesContributors,...
                                        vStringNamesContributors)),InstitutionName)), vStringNamesContributors);

                    vStringMasterNames_sorted   = sListMasterNamesContributors(ismember(sListNamesContributors,vStringNamesContributors));
                    vStringMasterNames_unsorted = cell(length(vStringMasterNames_sorted),1);
                    
                    for iNames = 1:length(vStringMasterNames_unsorted)
                        vStringMasterNames_unsorted(iNames) = vStringMasterNames_sorted(indexNames_SortedList(iNames));
                    end

                    if length(vStringNamesContributors)~= length(vStringMasterNames_unsorted)
                        if sum(~ismember(vStringNamesContributors, sListNamesContributors))==0
                            fprintf('Please check the institution that appears at least twice in the list')
                        else
                            sFileTitle
                            sCountry
                            return
                        end
                    end

                    %% 3. VARIABLE: Identify the variables for which the information is being reported
                    % Assumption: this type of information starts in the third colum of the Header

                    jVariable = 0;

                    for jColumn = 3:2:size(Header,2)-1

                        % Read the name of the forecasted variable 
                        % Assumption: this information on the 5th and the 4th lines before the name of the country 
                        if (strcmp(Header(vIndexLinesCountries_Header(iCountry)-4, jColumn),' '))||...
                                (strcmp(Header(vIndexLinesCountries_Header(iCountry)-4, jColumn),''))
                            sNameVariable = Header(vIndexLinesCountries_Header(iCountry)-5, jColumn);
                        else
                            if strcmp(Header(vIndexLinesCountries_Header(iCountry)-3, jColumn), 'Next FOMC Meeting') ||...
                                  strcmp(Header(vIndexLinesCountries_Header(iCountry)-3, jColumn), 'next 30 days?') ||...
                                  strcmp(Header(vIndexLinesCountries_Header(iCountry)-3, jColumn), 'Next 30 days?')
                              
                                sNameVariable = strcat(Header(vIndexLinesCountries_Header(iCountry)-5, jColumn),{' '},Header(vIndexLinesCountries_Header(iCountry)-4, jColumn), ...
                                    {' '},Header(vIndexLinesCountries_Header(iCountry)-3, jColumn));
                            else
                                sNameVariable = strcat(Header(vIndexLinesCountries_Header(iCountry)-5, jColumn),{' '},Header(vIndexLinesCountries_Header(iCountry)-4, jColumn));
                            end
                        end

                        if ~strcmp(sNameVariable, '')


                            % Read the unit of the variable
                            % Assumption: this information is located on the 3th line before the name of the country 
                            if strcmp(Header(vIndexLinesCountries_Header(iCountry)-3, jColumn), 'Next FOMC Meeting') ||...
                                  strcmp(Header(vIndexLinesCountries_Header(iCountry)-3, jColumn), 'next 30 days?') ||...
                                  strcmp(Header(vIndexLinesCountries_Header(iCountry)-3, jColumn), 'Next 30 days?')
                                
                                sUnitVariable = ''; % the  'Money Policy Evaluation' does not have units
                                
                            else
                                
                                sUnitVariable = Header(vIndexLinesCountries_Header(iCountry)-3, jColumn);
                            
                            end
                            
                            %% Check 
                            % Check whether the variable read matches the defined ones
                            % 1. check - is the variable in the predefined list of names of variables?
                            % 2. check - is the unit of the variable in the predefined list of units for that specific variables?

                            % First find the variable in the array of structures of variables
                            bFoundVariable = 0;
                            indexlist_Variable = 1;
                            while (~bFoundVariable)&&(indexlist_Variable<=length(structVariables))
                                if sum(strcmp(structVariables(indexlist_Variable).Names, sNameVariable))==1 % if the variable is found
                                    if sum(strcmp(structVariables(indexlist_Variable).Units, sUnitVariable))==1 % if the unit of measure corresponds
                                        bFoundVariable = 1;
                                    % The only exception so far discovered and allowed
                                    elseif strcmp(sNameVariable,'Three month Interest Rates')&&strcmp(sUnitVariable,'')
                                        sUnitVariable = '%';
                                        bFoundVariable = 1;
                                    elseif strcmp(sNameVariable,'Money Policy Evaluation Prob. of rate change in the next 30 days?')&&strcmp(sUnitVariable,'')
                                        bFoundVariable = 1;
                                    else
                                        % problems with the units of the variable
                                        sFileTitle
                                        iFile
                                        indexlist_Variable
                                        Header(vIndexLinesCountries_Header(iCountry),indexColumnsCountries_Header)
                                        fprintf('Problems with the UNITS of the variable')
                                        sNameVariable
                                        break
                                    end
                                else
                                   indexlist_Variable = indexlist_Variable+1; 
                                end
                            end

                            if ~bFoundVariable % problems with the name of the variable
                                sFileTitle
                                iFile
                                Header(vIndexLinesCountries_Header(iCountry),indexColumnsCountries_Header)
                                fprintf('Problems with the NAME of the variable')
                                sNameVariable
                                return
                            end    

                        end

                        if bFoundVariable 

                            %% Check whether there is actually any data forecasted

                            % Read the numbers
                            % Before reading, check again that the country
                            % matches with the string from two lines below
                            nLinesDiff_HeaderData = 23;

                            nLines_OriginalData = length(index_deleteContributors_noForecasts) + length(vStringNamesContributors);
                            
                            if strcmp(Header(vIndexLinesYears_Data(iCountry)+nLinesDiff_HeaderData), sCountry)
                                data1 = Data(vIndexLinesYears_Data(iCountry)+3:vIndexLinesYears_Data(iCountry)+nLines_OriginalData+2,jColumn-2);
                                data2 = Data(vIndexLinesYears_Data(iCountry)+3:vIndexLinesYears_Data(iCountry)+nLines_OriginalData+2,jColumn-1);
                            else % for the moment the sole exception is the EuroZone. The way to treat for the moment this exception is to consider directly the vIndexLinesYears_Data(iCountry+1)

                                data1 = Data(vIndexLinesYears_Data(iCountry+1)+3:vIndexLinesYears_Data(iCountry+1)+nLines_OriginalData+2,jColumn-2);
                                data2 = Data(vIndexLinesYears_Data(iCountry+1)+3:vIndexLinesYears_Data(iCountry+1)+nLines_OriginalData+2,jColumn-1);
                            end

                            data1(index_deleteContributors_noForecasts,:) = [];
                            data2(index_deleteContributors_noForecasts,:) = [];
                            
                            if (length(data1)~= length(vStringNamesContributors))||(length(data2)~= length(vStringNamesContributors))  
                                    fprintf('Diferente')
                                    sFileTitle
                            end
                            
                            % Check for NaN data
                            if (sum(~isnan(data1))>0)||(sum(~isnan(data2))>0)

                                % 1. COUNTRY: Identify the name country for which the variables are available 
                                    % Find the position of the country name under the old format in the list with countries names under the new format
                                index_Country = find(ismember(vStringCountries_General,sCountry)==1);
                                
                                structForecasts_OldFormat(iFile).Forecast(iCountry).Country               = vStringCountries_newformat(index_Country);
                                structForecasts_OldFormat(iFile).Forecast(iCountry).Names_Contributors    = vStringNamesContributors;
                                structForecasts_OldFormat(iFile).Forecast(iCountry).Contributors          = vStringMasterNames_unsorted;    
                                structForecasts_OldFormat(iFile).Forecast(iCountry).Date_Survey           = dateSurvey;
                                % Now we are sure that there is data and that this is correct (:)) 
                                jVariable = jVariable + 1;

                                structForecasts_OldFormat(iFile).Forecast(iCountry).Variables(jVariable).Variable_category = char(structVariables(indexlist_Variable).Variable);
                                structForecasts_OldFormat(iFile).Forecast(iCountry).Variables(jVariable).Variable_name     = char(sNameVariable);
                                structForecasts_OldFormat(iFile).Forecast(iCountry).Variables(jVariable).Variable_units    = char(sUnitVariable);

                                structForecasts_OldFormat(iFile).Forecast(iCountry).Variables(jVariable).Nowcast.IndividualData = data1;
                                structForecasts_OldFormat(iFile).Forecast(iCountry).Variables(jVariable).Forecast.IndividualData = data2;

                                % if everythink is fine, then read the years
                                % 4. Nr of times the variable is forecasted
                                % Assumption 1: there are always 2 forecasts available and
                                % Assumption 2: these are located 2 lines before the name of the country
                                % This info is either read by Matlab as numbers, or as string
                                if ~strcmp(Header(vIndexLinesCountries_Header(iCountry)-2, jColumn),'')
                                    % the info is either a string
                                    vStringVariables = [Header(vIndexLinesCountries_Header(iCountry)-2, jColumn); Header(vIndexLinesCountries_Header(iCountry)-2, jColumn+1)];
                                else % the info is a number (the year for which the forecast is done)
                                    if strcmp(Header(vIndexLinesYears_Data(iCountry)+nLinesDiff_HeaderData), sCountry)
                                        vStringVariables = {num2str(Data(vIndexLinesYears_Data(iCountry),jColumn-2)); num2str(Data(vIndexLinesYears_Data(iCountry),jColumn-1))}; 
                                    else
                                        vStringVariables = {num2str(Data(vIndexLinesYears_Data(iCountry+1),jColumn-2)); num2str(Data(vIndexLinesYears_Data(iCountry+1),jColumn-1))}; 
                                    end
                                end
                                structForecasts_OldFormat(iFile).Forecast(iCountry).Variables(jVariable).Nowcast.YearForecast = vStringVariables(1);
                                structForecasts_OldFormat(iFile).Forecast(iCountry).Variables(jVariable).Forecast.YearForecast = vStringVariables(2);

                                %% Compute also statistics

                                % Mean forecast
                                structForecasts_OldFormat(iFile).Forecast(iCountry).Variables(jVariable).Nowcast.Mean = nanmean(data1);
                                structForecasts_OldFormat(iFile).Forecast(iCountry).Variables(jVariable).Forecast.Mean = nanmean(data2);

                                % Highest forecast
                                structForecasts_OldFormat(iFile).Forecast(iCountry).Variables(jVariable).Nowcast.Max = nanmax(data1);
                                structForecasts_OldFormat(iFile).Forecast(iCountry).Variables(jVariable).Forecast.Max = nanmax(data2);

                                % Lowest forecast
                                structForecasts_OldFormat(iFile).Forecast(iCountry).Variables(jVariable).Nowcast.Min = nanmin(data1);
                                structForecasts_OldFormat(iFile).Forecast(iCountry).Variables(jVariable).Forecast.Min = nanmin(data2);

                                % Standard deviation
                                structForecasts_OldFormat(iFile).Forecast(iCountry).Variables(jVariable).Nowcast.StandardDeviation = nanstd(data1);
                                structForecasts_OldFormat(iFile).Forecast(iCountry).Variables(jVariable).Forecast.StandardDeviation = nanstd(data2);

                                % Median
                                structForecasts_OldFormat(iFile).Forecast(iCountry).Variables(jVariable).Nowcast.Median = nanmedian(data1);
                                structForecasts_OldFormat(iFile).Forecast(iCountry).Variables(jVariable).Forecast.Median = nanmedian(data2);
                                
                                % Skewness
                                structForecasts_OldFormat(iFile).Forecast(iCountry).Variables(jVariable).Nowcast.Skewness = skewness(data1);
                                structForecasts_OldFormat(iFile).Forecast(iCountry).Variables(jVariable).Forecast.Skewness = skewness(data2);
                                
                                % Kurtosis
                                structForecasts_OldFormat(iFile).Forecast(iCountry).Variables(jVariable).Nowcast.Kurtosis = kurtosis(data1);
                                structForecasts_OldFormat(iFile).Forecast(iCountry).Variables(jVariable).Forecast.Kurtosis = kurtosis(data2);
                                
                            end

                        end

                    end

                end  

                % Delete the empty structures corresponding to countries for
                % which there was no contributor and implicitly no data
                index_delete = [];
                for iCountry = 1:length(structForecasts_OldFormat(iFile).Forecast)
                   if isempty(structForecasts_OldFormat(iFile).Forecast(iCountry).Names_Contributors)
                        index_delete = [index_delete iCountry];
                   end
                end
                structForecasts_OldFormat(iFile).Forecast(index_delete) =[];   

        end

        %  Save the structure
        save([sOutputDirectory 'structForecasts_OldFormat.mat'], 'structForecasts_OldFormat');
        
    end
    