% This code appends quarterly files to the monthly structure file

    clear
    clc;
    close all

    addpath('Functions');
   
    sInputDirectory     = 'CE_Mat_files\';
    
    
%%  Define the variables that are being surveyed and the units in which they are reported 
    structVariables         = fdefineVariables_newformat;

%% Load structure with excel data
    % This loads also the name of the Excel files thart contain data
    load([sInputDirectory 'structExcelData_QuarterlyForecasts.mat']);
    nCountries = length(structExcelData_QuarterlyForecasts);
    
%%  
    
    for iCountry = 1:nCountries
        
        % Identify to which country does the information from the structure with Excel data corresponds
        sFileTitle          = vStringFileTitle{iCountry};
        sCountry            = sFileTitle(1:end-5) % this is the part of the name of the Excel file that contains the info about the name of the country
                                            % (Here:changed to -5 instead of -4 because Consensus changed from .xls files to .xlsx files!)                                          
                                            
        if strcmp(sCountry,'United Kingdom')
            sCountry = 'UK';
        elseif strcmp(sCountry,'United States')
            sCountry = 'USA';
        end
            
        % Load the country structure in accordance
        load([sInputDirectory sCountry '_raw.mat']);
               
        % Create a vector with all the dates for which forecasts have been provided for that country
        vAllDates_monthly_CESurveys      = transpose(cell2mat({structCountry.Date_Survey}));
        
        for iSheet = 1:length(structExcelData_QuarterlyForecasts(iCountry).Info) % for each sheet available in the file with quarterly values
            
            Header          = structExcelData_QuarterlyForecasts(iCountry).Info(iSheet).Header;
            Data            = structExcelData_QuarterlyForecasts(iCountry).Info(iSheet).Data;
            Info_Variable   = structExcelData_QuarterlyForecasts(iCountry).Info(iSheet).Variable_Name;
            sInfoVariable   = char(Info_Variable);
            
            % Find whether the info if the forecast is Y-o-Y or Q-o-Q
            % annualized is stored in the name of the variable and extract
            % the information 
                
            if ~isempty(strfind(char(Info_Variable),'(Y'))
                
                index_Paranthesis   = strfind(char(Info_Variable),'(Y');
                sType_Forecast      = sInfoVariable(index_Paranthesis+1:end-1);
                sInfoVariable(index_Paranthesis:end) = [];
                
            elseif ~isempty(strfind(char(Info_Variable),'(Q'))
                
                index_Paranthesis   = strfind(char(Info_Variable),'(Q');
                sType_Forecast      = sInfoVariable(index_Paranthesis+1:end-1);
                sInfoVariable(index_Paranthesis:end) = [];
                
            end
            
            % Extract whether the VARIABLE and info from sheet is the mean, high, low,
            % st.dev or no. of forecasters type
            cFields     = {'Consensus', 'High', 'Low', 'Std Dev', 'No. Forecasts'};
            bFoundField = 0;
            k           = 1;
            
            while bFoundField==0 && k<=5
            
                index_start     = strfind(sInfoVariable, cFields{k});
                if ~isempty(index_start)
                    sNameVariable       = sInfoVariable(1:index_start-1);
                    sField              = sInfoVariable(index_start:end);
                    
                    bFoundField = 1;
                else
                    k = k+1;
                end
            end
            
            if (k>5) && (bFoundField==0) % if no field is found  
                fprintf('Error')
            end
            
            if sNameVariable(end)==' '
                sNameVariable(end)= [];
            end
            if sField(end)==' '
                sField(end)= [];
            end
            
            % Search for the variable in the list of all variables
            % available
            bFoundVariable      = 0; % initialise the boolean variable indicating whether the variable is found or not
            % First find the variable in the array of structures of variables
            indexlist_Variable  = 1;
            while (~bFoundVariable)&&(indexlist_Variable<=length(structVariables))
                if sum(strcmp(structVariables(indexlist_Variable).Names, sNameVariable))==1 % if the variable is found
                    bFoundVariable = 1;
                else
                   indexlist_Variable = indexlist_Variable+1; 
                end
            end 

             if (indexlist_Variable>length(structVariables)) && (bFoundVariable==0) % if no field is found  
                sNameVariable
                fprintf('Error name variable')
                return
            end
             
            % Find the dates for which the variable and the fields are
            % available - the Header and Data have different sizes formats.
            % What matters is that Data is available for the end of the
            % Header dates. Hence, the relevant info of Header needs to
            % be extracted from bottom to top based on size of Data
            nMonths = size(Data,1);
            % Extract the date of the survey from the Header variable
            
            %If the date format of the last survey is different: 
            if cellfun('length', Header(end,1)) > cellfun('length', Header(14,1))                 
                 x = datenum(Header(end,1), 'mmm dd,yyyy'); 
                 x = datestr(x, 'dd/mm/yyyy');
                Header(end,1) = {x};
            end
            
            index = find(cellfun('length', Header(end-nMonths+1:end,1)) > repmat(cellfun('length', Header(13,1)), nMonths, 1));
            
            if ~isempty(index)  
                x = datenum(Header(length(Header(:,1)) - nMonths+index,1), 'mmm dd,yyyy'); 
                x = datestr(x, 'dd/mm/yyyy');
                Header(length(Header(:,1)) - nMonths+index,1) = {x};
            end 
            
%             try 
%                 dateSurvey      = datenum(sDate);
%             catch
%                 dateSurvey      = datenum(sDate,'dd.mm.yyyy');
%             end
            
%             try
%                 vDatesSurvey_Quarterly = datenum(Header(end-nMonths+1:end,1), 'dd/mm/yyyy');
%             catch
%                 vDatesSurvey_Quarterly = datenum(Header(end-nMonths+1:end,1), 'mmm dd,yyyy');                     
%             end
             try
                 vDatesSurvey_Quarterly = datenum(Header(end-nMonths+1:end,1));
             catch
                vDatesSurvey_Quarterly = datenum(Header(end-nMonths+1:end,1), 'dd.mm.yyyy');                     
             end
            
            % Determining the horizons requires a more thorough analysis
            % A. determine the last horizon based on the last date for
            % which the survey is available. 
            dQuarters_LastObs = length(Data(end,~isnan(Data(end,:))));
            dLastHorizon = datestr(addtodate(vDatesSurvey_Quarterly(end),(dQuarters_LastObs-1)*3,'month'),'mmmyyyy');
            cAllHorizons = Header(11,2:end);
            for iHorizon = 1:length(cAllHorizons)
                sHorizon = char(cAllHorizons(iHorizon));
                if length(sHorizon)>10
                    sHorizon(11:end) = [];
                end
                
                try
                    cAllHorizons(iHorizon) = cellstr(datestr(datenum(sHorizon,'dd.mm.yyyy'), 'mmmyyyy'));
                catch
                    cAllHorizons(iHorizon) = cellstr(datestr(datenum(sHorizon,'dd/mm/yyyy'), 'mmmyyyy'));
                end
            end
            
            % Find this month in the list of horizons available from Header
            iIndexEnd = find(strcmp(cAllHorizons, dLastHorizon));
            % Delete as of this point onward forecast horizons without data
            Header(:,iIndexEnd+2:end) = [];
            % Determine the starting index 
            indexStartHorizons = size(Header,2) - 1 - size(Data,2)+1; % deduct one for the first column, add 1 for the difference 
            %indexStartHorizons = size(Header,2) - size(Data,2)+1   
            
             for iDateSurvey = 1:nMonths
                 
                if ~find(vAllDates_monthly_CESurveys==vDatesSurvey_Quarterly(iDateSurvey)) % if the corresponding monthly values don't exist
                    sCountry
                    fprintf('Error with survey date')
                    return

                else % if the monthly data is already read in

                    if ~(sum(isnan(Data(iDateSurvey,:))) == length(Data(iDateSurvey,:))) % if for that date any kind of data is available

                        % The match will be done for the moment for variable category
                        index_date      = find(vAllDates_monthly_CESurveys==vDatesSurvey_Quarterly(iDateSurvey));

                        if ~isempty(index_date)
                            
                            % Find the index of the quarterly variable in the list of variables for which the forecasts are provided on a monthly basis
                            indexCategoryVariable_currentStructure = find(strcmp({structCountry(index_date).Variables.Variable_category}, structVariables(indexlist_Variable).Variable));

                             if ~isempty(indexCategoryVariable_currentStructure) % if and only if there were also monthly forecasts for that variable

                                % First determine the data that should be stored in the structure
                                dInfoForecasts = Data(iDateSurvey, ~isnan(Data(iDateSurvey,:)));

                                % Second, determine the horizons for which the forecast is made
                                indices_Horizon     = logical([zeros(1, indexStartHorizons) ~isnan(Data(iDateSurvey,:))]);
                                cHorizons           = Header(11,indices_Horizon);
                                nHorizons           = length(cHorizons);

                                % Store each horizon individually
                                for iHorizon = 1:nHorizons

                                    sHorizon = char(cHorizons(iHorizon));
                                    if length(sHorizon)>10
                                        sHorizon(11:end) = [];
                                    end
                                    try
                                        diffMonths          = months(vDatesSurvey_Quarterly(iDateSurvey),datenum(sHorizon, 'dd.mm.yyyy'))+1;
                                    catch
                                         diffMonths          = months(vDatesSurvey_Quarterly(iDateSurvey),datenum(sHorizon, 'dd/mm/yyyy'))+1;
                                    end
                                    %Changed the date format above

                                    if diffMonths>0
                                        % Store the horizon in quarters 
                                        quarter = floor(diffMonths/3);


                                        if ~exist('sType_Forecast') && ~strcmp(structVariables(indexlist_Variable).Variable, 'Three month Interest Rates') % if there is no indication whether the forecast is Y-o-Y or Q-o-Q, then, unless the forecast is for the interest rate, then it is Y-o-Y
                                            sType_Forecast = 'YoY';
                                        elseif strcmp(structVariables(indexlist_Variable).Variable, 'Three month Interest Rates')
                                            sType_Forecast = '';
                                        end

                                        if isempty(sType_Forecast)
                                            sField_FixedHorizon = ['FixedHorizon_Q' num2str(quarter)];
                                        else
                                            if strfind(sType_Forecast,' ')
                                                sField_FixedHorizon = ['FixedHorizon_Q' num2str(quarter) '_'  strrep(sType_Forecast,' ','')];
                                            elseif strfind(sType_Forecast,'-')
                                                sField_FixedHorizon = ['FixedHorizon_Q' num2str(quarter) '_' strrep(sType_Forecast,'-','')];
                                            else
                                                sField_FixedHorizon = ['FixedHorizon_Q' num2str(quarter) '_' sType_Forecast];
                                            end
                                        end


                                        % Store also the type of the forecast
                                        if ~isempty(sType_Forecast)
                                            structCountry(index_date).Variables(indexCategoryVariable_currentStructure).(sField_FixedHorizon).TypeForecast = sType_Forecast;  
                                        end

                                        try
                                            structCountry(index_date).Variables(indexCategoryVariable_currentStructure).(sField_FixedHorizon).Horizon_End = datestr(datenum(sHorizon, 'dd.mm.yyyy'), 'mmmyyyy');
                                        catch
                                            structCountry(index_date).Variables(indexCategoryVariable_currentStructure).(sField_FixedHorizon).Horizon_End = datestr(datenum(sHorizon, 'dd/mm/yyyy'), 'mmmyyyy');
                                        end
                                        structCountry(index_date).Variables(indexCategoryVariable_currentStructure).(sField_FixedHorizon).NrMonths = diffMonths;
                                        if strcmp(sField, 'Consensus')
                                            structCountry(index_date).Variables(indexCategoryVariable_currentStructure).(sField_FixedHorizon).Mean = dInfoForecasts(iHorizon);
                                        elseif strcmp(sField, 'High')
                                            structCountry(index_date).Variables(indexCategoryVariable_currentStructure).(sField_FixedHorizon).Max = dInfoForecasts(iHorizon);
                                        elseif strcmp(sField, 'Low')
                                            structCountry(index_date).Variables(indexCategoryVariable_currentStructure).(sField_FixedHorizon).Min = dInfoForecasts(iHorizon);
                                        elseif strcmp(sField, 'Std Dev')
                                             structCountry(index_date).Variables(indexCategoryVariable_currentStructure).(sField_FixedHorizon).StandardDeviation = dInfoForecasts(iHorizon);
                                        elseif  strcmp(sField, 'No. Forecasts')
                                             structCountry(index_date).Variables(indexCategoryVariable_currentStructure).(sField_FixedHorizon).NoForecasts = dInfoForecasts(iHorizon);   
                                        end

                                    end

                                end

                             end
                        end
                    end

                end
            
            
            end
                 
        clear sType_Forecast
        end
        
        save([sInputDirectory sCountry '_raw_Quarterly.mat'], 'structCountry');
             
    
    end
            
    
    

    
    