
function  structVariables = fdefineVariables_newformat

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% In this function there are defined the names of variables asked in the Consensus Forecasts questionnaire
%     Order of countries is: 
%     {1 - 'U.S.A.'; ...
%      2 - 'JAPAN';...
%      3 - 'GERMANY';...
%      4 - 'FRANCE';...
%      5 - 'UNITED KINGDOM';...
%      6 - 'ITALY';...
%      7 - 'CANADA';...
%      8 - 'NETHERLANDS';...
%      9 - 'NORWAY';...
%      10 - 'SPAIN';...
%      11 - 'SWEDEN';...
%      12 - 'SWITZERLAND';...
%      13 - 'Euro Zone'};

%   Generally, we will store the following information: 
%      vector = {'Gross Domestic Product'; 'Personal Consumption';'Investment';'Corporate Profits';'Industrial Production';'Consumer Prices';...
%         'Producer Prices';'Wages'; 'Car Sales'; 'Housing Starts'; 'Unemployment Rate'; 'Current Account';'Budget Deficit (Fiscal Years)';...
%         'Three month Interest Rates';'10 Year Govt Bond Yield'; 'Merchandise Exports'; 'Change in Inventories'; 'Gen. Gov Budget Balance';...
%         'Money Policy Evaluation'; 'Government Consumption'; 'Exports'; 'Imports'; 'Money Supply M3'};
%     nVariables = length(vector);
    % Check the length of this vector to initialise the structure of variables
%   OBS: Sometimes the variable surveyed for one country changes with time 
    
    indVariable         = 1;
%     structVariables     = repmat(struct('Variable', [],'Names', [], 'Units',[], 'Additional_Info', ''),1,nVariables);
                        
    %% Variable 1: 'Gross Domestic Product'
    structVariables(indVariable).Variable   = {'Gross Domestic Product'};
    structVariables(indVariable).Names      = {'Gross Domestic Product','Gross National Product', 'Gross Domestic Product (Mainland)', 'GDP'};
    structVariables(indVariable).Units      = {'real, % change', '% change from previous calendar year'};
    
    indVariable                             = plus(indVariable,1);
   
    %% Variable 2: 'Private Consumption'
    structVariables(indVariable).Variable   = {'Private Consumption'};
    structVariables(indVariable).Names      = {'Personal Consumption','Private Consumption', 'Personal Expenditure','Household Consumption', 'Consumer Expenditure', 'Consumer Spending', 'Consumption'};
    structVariables(indVariable).Units      = {'real, % change', '% change from previous calendar year'};
    
    indVariable                             = plus(indVariable,1);
    
    %% Variable 3: 'Investment'
    structVariables(indVariable).Variable   = {'Investment'};
    structVariables(indVariable).Names      = {'Business Investment', 'Gross Fixed Investment', 'Machinery & Equip. Investment', 'Investment', 'Machinery & Equipment Investment'};
    structVariables(indVariable).Units      = {'real, % change', '% change from previous calendar year'};

    indVariable                             = plus(indVariable,1);
    
    %% Variable 4: 'Corporate Profits'
    structVariables(indVariable).Variable   = {'Corporate Profits'};
    structVariables(indVariable).Names      = structVariables(indVariable).Variable;
    structVariables(indVariable).Units      = {'nominal, % change'};
    
    indVariable                             = plus(indVariable,1);
    
    %% Variable 5: 'Industrial Production'
    structVariables(indVariable).Variable   = {'Production'}';
    structVariables(indVariable).Names      = {'Industrial Production', 'Manufacturing Production', 'Mining & Manuf. Production', 'Mining & Manufacturing Production', 'Production'};
    structVariables(indVariable).Units      = {'real, % change', '% change', '% change from previous calendar year'};
    
    indVariable                             = plus(indVariable,1);
        
    %% Variable 6: 'Consumer Prices'
    structVariables(indVariable).Variable   = {'Consumer Prices'};
    structVariables(indVariable).Names      = {'Consumer Prices', 'Consumer Prices Index', 'Consumer Prices Index (HICP)', 'Consumer Price Inflation', 'Consumer Prices (CPI)', 'CPI'};
    structVariables(indVariable).Units      = {'% change', '% change from previous calendar year'};
    
    indVariable                             = plus(indVariable,1);
    
    %% Variable 6bis: 'Consumer Prices'
    structVariables(indVariable).Variable   = {'Retail Prices'};
    structVariables(indVariable).Names      = {'Consumer Prices (RPIX)', 'Retail Prices (RPIX)', 'Retail Prices (Headline Rate)', 'Retail Price Inflation (RPI-X, Underlying Rate)', 'Retail Prices (RPI)', 'RPI'};
    structVariables(indVariable).Units      = {'% change', '% change from previous calendar year'};
    
    indVariable                             = plus(indVariable,1);

    %% Variable 7: 'Producer Prices'
    structVariables(indVariable).Variable   = {'Producer Prices'};
    structVariables(indVariable).Names      = {'Producer Prices','Wholesale Prices'};
    structVariables(indVariable).Units      = {'% change'};
   
    indVariable                             = plus(indVariable,1);
    
    %% Variable 8: 'Wages'
    structVariables(indVariable).Variable   = {'Wages'};
    structVariables(indVariable).Names      = {'Employment Costs', 'Wages', 'Average Hourly Earnings'};
    structVariables(indVariable).Units      = {'% change'};
    
    indVariable                             = plus(indVariable,1);
    
    %% Variable 9: 'Car Sales'
    structVariables(indVariable).Variable   = {'Car Sales'};
    structVariables(indVariable).Names      = {'Car Sales', 'New Car Registrations', 'Car Sales Registrations'};
    structVariables(indVariable).Units      = {'million units'};
   
    indVariable                             = plus(indVariable,1);
    
    %% Variable 10: 'Housing Starts'
    structVariables(indVariable).Variable   = {'Housing Starts'};
    structVariables(indVariable).Names      = structVariables(indVariable).Variable;
    structVariables(indVariable).Units      = {'million units', 'thousand units'};
   
    indVariable                             = plus(indVariable,1);
    
    %% Variable 11: 'Unemployment Rate'
    structVariables(indVariable).Variable   = {'Unemployment Rate'};
    structVariables(indVariable).Names      = structVariables(indVariable).Variable;
    structVariables(indVariable).Units      = {'% of labour force'};
   
    indVariable                             = plus(indVariable,1);
   
    %% Variable 12: 'Current Account'
    structVariables(indVariable).Variable   = {'Current Account'};
    structVariables(indVariable).Names      = {'Current Account', 'Current Account Balance'};
    structVariables(indVariable).Units      = {'US$ bn', 'US$bn', ' bn', 'UK bn', 'C$ bn', 'Nkr Bn', 'local currency, bn', 'local currency, tn', ' tn', 'Skr bn','Nkr bn',...
                                        'SwFr bn','Yen tn','euros, bn', 'Euros bn', 'Euro bn'};
   
    indVariable                             = plus(indVariable,1);
  
    %% Variable 13: 'Budget Deficit (Fiscal Years)'
    structVariables(indVariable).Variable   = {'Budget Deficit'};
    structVariables(indVariable).Names      = {'Budget Deficit', 'Budget Deficit (Fiscal Years)','PSNCR (Fiscal Years)', 'PSNB ex (Fiscal Years)'};
    structVariables(indVariable).Units      = {'local currency, bn', 'local currency, tn', ' bn', 'euros, bn'};
   
    indVariable                             = plus(indVariable,1);
    
    %% Variable 14: 'Three month Interest Rates'
    structVariables(indVariable).Variable   = {'Three month Interest Rates'};
    structVariables(indVariable).Names      = {'Three month Interest Rates', '3-month Int. Rates (Ger/Fra/It/Nld/Spa)', '3 Month Interest Rate', '3M IR', '3 Month Treasury Bill Rate',...
        '3 month Yen (TIBOR) Interest Rate',  '3 month Interbank Interest Rate', '3 month Treasury Bill Interest Rate', '3 month Interbank Rate', '3 month Euro Swiss Franc Deposit Rate', ...
         '3-month Euribor Rate'};
    structVariables(indVariable).Units      = {'%'};
   
    indVariable                             = plus(indVariable,1);

    %% Variable 15: '10 Year Govt Bond Yield'
    structVariables(indVariable).Variable   = {'10 Year Govt Bond Yield'};
    structVariables(indVariable).Names      = {'10 Year Govt Bond Yield', '10 Year Ger. Govt Bond Yield', '10 Year Treasury Bond Yield', '10 Year German Govt Bond Yield', '10 Year French Govt Bond Yield', ...
                                            '10 Year Gilt Yield', '10 Year Italian Govt Bond Yield', '10 Year Government Bond Yield', '10 Year Dutch Govt Bond Yield', '10 Year Spanish Govt Bond Yield', ...
                                            }; 
    structVariables(indVariable).Units      = {'%'};

    indVariable                             = plus(indVariable,1);

    %% Variable 16: 'Merchandise Exports'
    structVariables(indVariable).Variable   = {'Merchandise Exports'};
    structVariables(indVariable).Names      = structVariables(indVariable).Variable;
    structVariables(indVariable).Units      = {'local currency, bn'};
   
    indVariable                             = plus(indVariable,1);

    %% Variable 17: 'Change in Inventories'
    structVariables(indVariable).Variable   = {'Change in Inventories'};
    structVariables(indVariable).Names      = structVariables(indVariable).Variable;
    structVariables(indVariable).Units      = {'Euros bn'};
   
    indVariable                             = plus(indVariable,1);
 
    %% Variable 18: 'Gen. Gov Budget Balance'
    structVariables(indVariable).Variable   = {'Budget Balance'};
    structVariables(indVariable).Names      = {'Budget Balance', 'Gen. Gov Budget Balance', 'Budget Balance (Fiscal Years)', 'PSNB (Fiscal Years)'};
    structVariables(indVariable).Units      = {'local currency, bn', 'Skr bn', 'SwFr bn', 'US$ bn', 'Yen tn', ' bn','C$ bn', 'Nkr bn', 'Euros bn', 'euros, bn', 'local currency, tn'};
   
    indVariable                             = plus(indVariable,1);
    
    %% Variable 19: 'Money Policy Evaluation'
    structVariables(indVariable).Variable   = {'Money Policy Evaluation'};
    structVariables(indVariable).Names      = {'Money Policy Evaluation Prob. of rate change in the next 30 days?', 'Money Policy Evaluation Prob. of Rate Change in the Next 30 days?',...
        'Money Policy Evaluation Prob. of Rate Change at Next FOMC Meeting','Monetary Policy Evaluation Prob. of Rate Change at/before'};
    structVariables(indVariable).Units      = [];
    
    indVariable                             = plus(indVariable,1);
    
    %% Variable 20: 'Government Consumption'
    structVariables(indVariable).Variable   = {'Government Consumption'};
    structVariables(indVariable).Names      = structVariables(indVariable).Variable;
    structVariables(indVariable).Units      = {'real, % change', '% change'};
   
    indVariable                             = plus(indVariable,1);
    
    %% Variable 21: 'Exports'
    structVariables(indVariable).Variable   = {'Exports'};
    structVariables(indVariable).Names      = structVariables(indVariable).Variable;
    structVariables(indVariable).Units      = {'% change'};
   
    indVariable                             = plus(indVariable,1);
    
    %% Variable 22: 'Imports'
    structVariables(indVariable).Variable   = {'Imports'};
    structVariables(indVariable).Names      = structVariables(indVariable).Variable;
    structVariables(indVariable).Units      = {'% change'};
   
    indVariable                             = plus(indVariable,1);    
    
    %% Variable 23: 'Money Supply M3'
    structVariables(indVariable).Variable   = {'Money Supply M3'};
    structVariables(indVariable).Names      = structVariables(indVariable).Variable;
    structVariables(indVariable).Units      = {'% change', 'end priod, % change'};
    
  
    
   