
% Code to read in the realised levels of the CPI for euro area countries
% and the euro area HICP and compute autocorrelation coeficients

    clear
    clc;
    close all

    addpath(genpath([cd '\']));
    
    sInputDirectory     = 'Input\';
    sOutputDirectory    = 'Input\';
    
    sFileTitle          = 'CPI_realisedLevels_NatInstStat - update2020.xlsx';

%%  Read the data from the file

    [Data, Header]      = xlsread([sInputDirectory sFileTitle], 'CPI level');
    
    % Construct the vector of dates
    vDates_years        = Data(:,1);
    vDates_months       = Data(:,2);
    % Create a vectot that for the moment contains the first day of the month
    vDates_startofmonth              = datenum([[vDates_years; vDates_years(end)], [vDates_months; vDates_months(end)+1], ones(length(vDates_years)+1,1)]);
    
    % Convert these start of months dates into end of months dates
    vDates_endofmonth       = vDates_startofmonth - 1;
    vDates_endofmonth(1)  = [];
    
    nCPIlevels  = size(Data,2)-2;
    
    for iIndex = 1: nCPIlevels
        
        vCPI                                = Data(:,2 + iIndex);
        
        structCPI(iIndex).CPIrealised       = vCPI(~isnan(vCPI));
        structCPI(iIndex).Dates_CPIrealised = vDates_endofmonth(~isnan(vCPI));
        
        vMonthlyGrowthRates = structCPI(iIndex).CPIrealised(2:end)./ structCPI(iIndex).CPIrealised(1:end-1) -1;
        structCPI(iIndex).vMonthlyGrowthRates = [0; vMonthlyGrowthRates];
        
        % Add also the name of the country for which the forecast is done
        if strcmp(Header(2, 2 + iIndex), 'FRCPI')
            structCPI(iIndex).Country = 'France';
        elseif strcmp(Header(2, 2 + iIndex), 'GECPI')
            structCPI(iIndex).Country = 'Germany';
        elseif strcmp(Header(2, 2 + iIndex), 'ITCPI')
            structCPI(iIndex).Country = 'Italy';
        elseif strcmp(Header(2, 2 + iIndex), 'ESCPI')
            structCPI(iIndex).Country = 'Spain';
        elseif strcmp(Header(2, 2 + iIndex), 'NLCPI')
            structCPI(iIndex).Country = 'Netherlands';
        elseif strcmp(Header(2, 2 + iIndex), 'EUHCPI')
            structCPI(iIndex).Country = 'Euro Zone';  
        elseif strcmp(Header(2, 2 + iIndex), 'USCPI')
            structCPI(iIndex).Country = 'USA';  
        elseif strcmp(Header(2, 2 + iIndex), 'JPCPI')
            structCPI(iIndex).Country = 'Japan';  
        elseif strcmp(Header(2, 2 + iIndex), 'CHCPI')
            structCPI(iIndex).Country = 'Switzerland';  
        elseif strcmp(Header(2, 2 + iIndex), 'SECPI')
            structCPI(iIndex).Country = 'Sweden';  
        elseif strcmp(Header(2, 2 + iIndex), 'NOCPI')
            structCPI(iIndex).Country = 'Norway';  
        elseif strcmp(Header(2, 2 + iIndex), 'UKRPI')
            structCPI(iIndex).Country = 'UK';  
        elseif strcmp(Header(2, 2 + iIndex), 'CACPI')
            structCPI(iIndex).Country = 'Canada';  
            
        end
        
    end

%%  Compute the annual inflation rate (CPI level / CPI level 12 months ago) on a monthly basis

    for iIndex = 1: nCPIlevels
        
        vCPI    = structCPI(iIndex).CPIrealised;
        
        vInflationRate_annual = NaN(length(vCPI)-12,1);
        for iDate = 13:length(vCPI)
            vInflationRate_annual(iDate-12) = (vCPI(iDate)/vCPI(iDate-12)-1)*100;
        end
        structCPI(iIndex).InflationRate_annual          = vInflationRate_annual;
        structCPI(iIndex).dDates_InflationRate_annual    = structCPI(iIndex).Dates_CPIrealised(13:end);
        structCPI(iIndex).sDates_InflationRate_annual    = datestr(structCPI(iIndex).dDates_InflationRate_annual,'mmmyyyy');
 
        % By construction, countries are assumed to have had zero VAT changes
        structCPI(iIndex).nVATChanges         = 0;
        
    end


%%  Read the information about the VAT changes
    
    [Data, Header]      = xlsread([sInputDirectory 'VAT changes_Consensus Countries.xlsx'], 'Info');
    cCountries_VAT      = unique(Header(2:end,1));
    
    for iCountry    = 1: length(cCountries_VAT)
        
        % Find the position in the structures of all countries to store the
        % information about the VAT adjustments
        indexVAT        = find(ismember({structCPI.Country},cCountries_VAT(iCountry)));
       
        index_changes       = find(ismember(Header(2:end,1),cCountries_VAT(iCountry)));
        structCPI(indexVAT).sizeVATChanges    = Data(index_changes,3);
        
        try 
            dates_VATchanges      = datenum(Header(index_changes+1,2));
        catch
            dates_VATchanges      = datenum(Header(index_changes+1,2),'dd.mm.yyyy');
        end
        
        %dates_VATchanges   = datenum(Header(index_changes+1,2),'dd/mm/yyyy');
        
        % Prepare the vector of dummies to be included in the regression
        % for the autocorrelation coefficient
        structCPI(indexVAT).vDummiesVATChanges  = zeros(length(structCPI(indexVAT).InflationRate_annual),length(dates_VATchanges));
        structCPI(indexVAT).nVATChanges         = length(dates_VATchanges);
        
        for iVATchanges = 1:length(dates_VATchanges)
            
            indexStartPeriodVATchanges     = find(strcmp(cellstr(structCPI(indexVAT).sDates_InflationRate_annual), ...
                datestr(dates_VATchanges(iVATchanges), 'mmmyyyy')));
            if ~isempty(indexStartPeriodVATchanges)
                structCPI(indexVAT).vDummiesVATChanges(indexStartPeriodVATchanges:indexStartPeriodVATchanges+11, iVATchanges) = ones(12,1);
            end
   
        end
        
    end
    
%% Finally, estimate the autocorrelation coefficient

    for iCountry = 1:nCPIlevels
    
        y = structCPI(iCountry).InflationRate_annual;
        T = length(y);
               
        nParam = 2 + structCPI(iCountry).nVATChanges;
        % The ordering is constant, t-1, dummies for VAT changes, if any
        if ~isempty(structCPI(iCountry).vDummiesVATChanges)
            % Construct the handle function
            myfunc  = @(beta)(y(2:end) - beta(1)*ones(T-1,1) - beta(2) * (y(1:end-1) - ...
                structCPI(iCountry).vDummiesVATChanges(1:end-1,:) * beta(3:nParam)) - structCPI(iCountry).vDummiesVATChanges(2:end,:) * beta(3:nParam));
            
            beta_0 = [0.1;0.9;ones(structCPI(iCountry).nVATChanges,1)];
            
%             test = y(2:end) - beta_0(1)*ones(T-1,1) - beta_0(2) * (y(1:end-1) - ...
%                 structCPI(iCountry).vDummiesVATChanges(1:end-1,:) * beta_0(3:nParam)) - structCPI(iCountry).vDummiesVATChanges(2:end,:) * beta_0(3:nParam);
            
            % Perform the nonlinear estimation
            [beta, resnorm,residual,exitflag,output,lambda,jacobian]    = lsqnonlin(myfunc, beta_0);
            
            % Construct the standard errors and intervals
            df              = T-1-2; % degrees of freedom in the problem
            stdressiduals   = sqrt(sum(residual.^2)/df); % standard deviation of the residuals
            stderrors       = full(sqrt(diag(stdressiduals^2*inv(jacobian'*jacobian)))); % Parameter standard errors
            upper_bound     = beta + 2*stderrors; % upper lower bound for the confidence interval
            lower_bound     = beta - 2*stderrors; % lower bound for the confidence interval
            
            structCPI(iCountry).beta_constant           = beta(1);
            structCPI(iCountry).beta_VATchanges         = beta(3:nParam);
            structCPI(iCountry).stderrors               = stderrors;
            structCPI(iCountry).beta_autocorr           = beta(2);
            structCPI(iCountry).beta_autocorr_monthly   = find_m_o_m_rho_based_on_y_o_y_rho(structCPI(iCountry).beta_autocorr);
            structCPI(iCountry).interval_beta_autocorr  = [lower_bound(2) upper_bound(2)];
            structCPI(iCountry).interval_beta_autocorr_monthly  = [find_m_o_m_rho_based_on_y_o_y_rho(lower_bound(2))...
                find_m_o_m_rho_based_on_y_o_y_rho(upper_bound(2))];
            
        
        else
            % Construct the handle function
            myfunc  = @(beta)(y(2:end) - [ones(T-1,1) y(1:end-1)]*beta);
            % Perform the nonlinear estimation
            [beta, resnorm,residual,exitflag,output,lambda,jacobian]    = lsqnonlin(myfunc, [0;0.9]);
           
            % Construct the standard errors and intervals
            df              = T-1-2; % degrees of freedom in the problem
            stdressiduals   = sqrt(sum(residual.^2)/df); % standard deviation of the residuals
            stderrors       = full(sqrt(diag(stdressiduals^2*inv(jacobian'*jacobian)))); % Parameter standard errors
            upper_bound     = beta + 2*stderrors; % upper lower bound for the confidence interval
            lower_bound     = beta - 2*stderrors; % lower bound for the confidence interval
            
            structCPI(iCountry).beta_constant   = beta(1);
            structCPI(iCountry).stderrors       = stderrors;
            structCPI(iCountry).beta_autocorr   = beta(2);
            structCPI(iCountry).beta_autocorr_monthly   = find_m_o_m_rho_based_on_y_o_y_rho(structCPI(iCountry).beta_autocorr);
            structCPI(iCountry).interval_beta_autocorr  = [lower_bound(2) upper_bound(2)];
            structCPI(iCountry).interval_beta_autocorr_monthly  = [find_m_o_m_rho_based_on_y_o_y_rho(lower_bound(2))...
                find_m_o_m_rho_based_on_y_o_y_rho(upper_bound(2))];
            
        end
       
        structCPI(iCountry).Country
        structCPI(iCountry).beta_autocorr_monthly 
        
    end
               
    
%%  Save the structure
    save([sOutputDirectory 'AutocorrCoef_CPI_2020.mat'], 'structCPI');
    
