% Code that estimates the autocorrelation coefficient of the QoQ GDP growth 
% based on the OECD time series and determines the MoM autocorrelation 
% coefficient of the process using this estimate. 


    clear
    clc;
    close all

    addpath(genpath([cd '\']));
    
    sInputDirectory     = 'Input\';
    
%% Input parameters

    vCountries         = fdefineNamesCountries_newformat;
    nCountries         = length(vCountries);
    
%%  Load the structure that contains GDP data - QoQ growth, seasonally adjusted
   
    [Data, Header]      = xlsread('GDP_QoQ_SeasonallyAdjusted_update2019.xlsx');
    
    [T, nCountries_GDP] = size(Data);
    structGDP           = repmat(struct([]), nCountries_GDP, 1);
    
    for iCountry = 1:nCountries_GDP
        
        structGDP(iCountry).Country             = char(Header(1,iCountry+1));
        structGDP(iCountry).GDP_QoQ_SeasAdj     = Data(:,iCountry);
        
    end
    
%%  Estimate the autoregression coefficient

    AutocorrCoef_GDP           = repmat(struct([]), nCountries_GDP, 1);
   
    for iCountry = 1: nCountries
        
        sCountry_of_Interest                = vCountries{iCountry};

        AutocorrCoef_GDP(iCountry).Country  = sCountry_of_Interest;
        
        %% Actual GDP realizations
        
        % Find the position of the current country in the list of countries
        % with realised GDP
        indexstructRealizedGDP          = find(ismember({structGDP.Country},sCountry_of_Interest));
       
        % Collect actual GDP and the dates when they are
        % reported
        vGDP_QoQ_SeasAdj     = structGDP(indexstructRealizedGDP).GDP_QoQ_SeasAdj;
        
        % Perform simple OLS on a constant and previous observation
        y = vGDP_QoQ_SeasAdj(2:end);
        x = [ones(length(y),1) vGDP_QoQ_SeasAdj(1:end-1)];
        
        [BETA, BINT]  = regress(y, x);
        
        AutocorrCoef_GDP(iCountry).GDP_QoQ_SeasAdj          = vGDP_QoQ_SeasAdj;
        AutocorrCoef_GDP(iCountry).CoeffEstimate_QoQ        = BETA(2);
        AutocorrCoef_GDP(iCountry).ConfidenceBounds         = BINT(2,:);
        
        AutocorrCoef_GDP(iCountry).CoeffEstimate_Monthly       = find_monthly_m_o_m_rho_based_on_quarterly_q_o_q_rho(BETA(2));
        AutocorrCoef_GDP(iCountry).ConfidenceBounds_Monthly(1) = find_monthly_m_o_m_rho_based_on_quarterly_q_o_q_rho(BINT(2,1));
        AutocorrCoef_GDP(iCountry).ConfidenceBounds_Monthly(2) = find_monthly_m_o_m_rho_based_on_quarterly_q_o_q_rho(BINT(2,2));
        
        sCountry_of_Interest
        AutocorrCoef_GDP(iCountry).CoeffEstimate_Monthly  
      
    end
    
    save([sInputDirectory 'AutocorrCoef_GDP_2019.mat'], 'AutocorrCoef_GDP');
    
    