
    clear
    clc;
    close all

    addpath(genpath([cd '\']));

    sInputDirectory         = '.\..\Files_ConsensusEconomics\CE_Mat_files\';
    sOutputDirectory        = 'Output_Mat_files\';
    sOutputDirectory_Excel  = 'Output_Excel\';
    
%%  Input coordinates

    % Specify the variable of interest
    sVariable_of_Interest   = 'Consumer Prices'; %Consumer Prices
  
    % Specify the field of interest
    sField_TermStructure    = 'Mean';
    
    % Specify the countries for which the graph should be produced
    vStringCountries_General = fdefineNamesCountries_newformat;
    nCountries               = length(vStringCountries_General);
    
%%  Time sample
    
    % Full desirable time series availability
    sDateStartSample_full   = 'Oct-1989'; 
    sDateEndSample_full     = 'Apr-2019';
    
    % Specify also the format of these dates
    sFormat                 = 'mmm-yyyy';
 
%% Info about VAT adjustments
    
    cCountries_VAT      = {'Japan','Germany', 'France','UK', 'Netherlands', 'Spain'};
    
    for iCountry = 1:length(cCountries_VAT)
        structCountriesVAT(iCountry).Country = cCountries_VAT(iCountry);
    end
    
    iCountry = 1; %Japan
    structCountriesVAT(iCountry).StartDates =  {'01-Jan-1996','01-Jan-2013'};
    structCountriesVAT(iCountry).EndDates =  {'31-Dec-1998','31-Dec-2015'};
    
    iCountry = 2; %Germany
    structCountriesVAT(iCountry).StartDates =  {'01-Jan-2006'};
    structCountriesVAT(iCountry).EndDates =  {'31-Dec-2007'};
    
    iCountry = 3; %France
    structCountriesVAT(iCountry).StartDates =  {'01-Jan-1994'};
    structCountriesVAT(iCountry).EndDates =  {'31-Dec-1996'};
   
    iCountry = 4; %UK
    structCountriesVAT(iCountry).StartDates =  {'01-Jan-1990','01-Jan-2008'};
    structCountriesVAT(iCountry).EndDates =  {'31-Dec-1992','31-Dec-2012'};
   
    iCountry = 5; %Netherlands
    structCountriesVAT(iCountry).StartDates =  {'01-Jan-2011'};
    structCountriesVAT(iCountry).EndDates =  {'31-Dec-2013'};
  
    iCountry = 6; %Spain
    structCountriesVAT(iCountry).StartDates =  {'01-Jan-1991','01-Jan-2009'};
    structCountriesVAT(iCountry).EndDates =  {'31-Dec-1993','31-Dec-2013'};
 
%%
    % Specify the quarterly horizon for the graphs - monthly values
    sFields_FixedEvents     = {'Nowcast', 'Forecast'};
    nFields                 = length(sFields_FixedEvents);
    sField_FixedHorizon     = {'FixedHorizon_Q4_YoY'};
    
%%  Some input parameters for the grid search

    stepsize        = 0.01;
    rho_vec         = transpose(-(1-stepsize):stepsize:(1-stepsize));
    h_subtract_vec  = 1:3;
    index_rho0      = find(rho_vec==0);
            
    vUnionDates     = [];
    
%% Plot the data for each country
   
    for iCountry = 1: nCountries
        
         bParticularities_VAT = 0; % for each country the assumption is that initially there are no VAT particularities
       
       % Load the structure for each country
        sCountry_of_Interest    = vStringCountries_General{iCountry}
        
        load([sInputDirectory sCountry_of_Interest  '_raw_quarterly.mat']);
       
        % only for UK replace Consumer Prices with Retail Prices
        % only for UK replace Consumer Prices with Retail Prices
        if strcmp(sCountry_of_Interest, 'UK')
            sVariable_of_Interest_country   = 'Retail Prices'; %Consumer Prices
        else
            sVariable_of_Interest_country = sVariable_of_Interest;

        end
        
        vDates = cell2mat({structCountry.Date_Survey}');
        
        % Find whether the country features VAT changes particularities
        if find(ismember(cCountries_VAT, sCountry_of_Interest))
            bParticularities_VAT    = 1;
            indexCountry            = find(ismember(cCountries_VAT,sCountry_of_Interest),1);
            vStartExcludingDates     = structCountriesVAT(indexCountry).StartDates; 
            vEndExcludingDates      = structCountriesVAT(indexCountry).EndDates;
            nPeriodsExclusion       = length(vStartExcludingDates); 
        end
        
        % Find the position of the starting date dates in the list of dates available for that country
        iIndexStart_forecasts    = find(strcmp(cellstr(datestr(vDates, sFormat)), sDateStartSample_full));
        if isempty(iIndexStart_forecasts)
            iIndexStart_forecasts = 1;
        end
        % Find the position of the end date dates in the list of dates available for that country
        iIndexEnd_forecasts      = find(strcmp(cellstr(datestr(vDates, sFormat)), sDateEndSample_full));
        if isempty(iIndexEnd_forecasts)
            iIndexEnd_forecasts = length(vDates);
        end
        
        vIndicesDates           = iIndexStart_forecasts:iIndexEnd_forecasts; 
        if bParticularities_VAT
            for iPeriod = 1:nPeriodsExclusion
                vIndexStart_temp    = find(vDates> datenum(char(vStartExcludingDates(iPeriod))),1);
                vIndexEnd_temp      = find(vDates> datenum(char(vEndExcludingDates(iPeriod))),1)-1;
                vMonths_tobeExcluded = vIndexStart_temp: vIndexEnd_temp;
                [~,ind] = ismember(vMonths_tobeExcluded, vIndicesDates);
                
                vIndicesDates(ind) = [];
            end
            
            nMonthsForecasts    = length(vIndicesDates);
            vDates_subsample    = vDates(vIndicesDates);
        else
            clear iIndexEnd_temp iIndexStart_temp
            % Total number of months
            nMonthsForecasts        = iIndexEnd_forecasts-iIndexStart_forecasts+1;
            vDates_subsample        = vDates(iIndexStart_forecasts:iIndexEnd_forecasts);
            vIndicesDates           = [iIndexStart_forecasts:iIndexEnd_forecasts]'; 
        end
        
        %%  For each country identify the fixed-event forecasts and the fixed-horizon forecasts
        mForecasts_FixedEvents          = NaN(nMonthsForecasts,nFields);
        vForecasts_FixedHorizons        = NaN(nMonthsForecasts,1);
        
        for iDate = 1:nMonthsForecasts

            % First, find the position of the variable of interest in the structure of each month dhorizons back
            indexVariable       = find(strcmp({structCountry(vIndicesDates(iDate)).Variables.Variable_category},sVariable_of_Interest_country));
            if ~isempty(indexVariable)
                 
                for iField = 1:nFields
                 
                    sField_individually     = char(sFields_FixedEvents{iField});
                    mForecasts_FixedEvents(iDate,iField)  = structCountry(vIndicesDates(iDate)).Variables(indexVariable).(sField_individually).(sField_TermStructure);                 
                   
                end
                
                sField_FixedHorizon = char(sField_FixedHorizon);
                 if isfield(structCountry(vIndicesDates(iDate)).Variables(indexVariable), sField_FixedHorizon) && ...
                            (~isempty(structCountry(vIndicesDates(iDate)).Variables(indexVariable).(sField_FixedHorizon)))

                    if isfield(structCountry(vIndicesDates(iDate)).Variables(indexVariable).(sField_FixedHorizon), sField_TermStructure) &&...
                            ~isempty(structCountry(vIndicesDates(iDate)).Variables(indexVariable).(sField_FixedHorizon).(sField_TermStructure))
                        vForecasts_FixedHorizons(iDate, 1) =  structCountry(vIndicesDates(iDate)).Variables(indexVariable).(sField_FixedHorizon).(sField_TermStructure);
                    end

                 end

             end
         
        end
        
       % Identify the dates where the fixed-horizon forecast is available
        indexNoData = sum(isnan(vForecasts_FixedHorizons),2)==size(vForecasts_FixedHorizons,2);
        
        vForecasts_FixedHorizons    = vForecasts_FixedHorizons(~indexNoData ,:); 
        mForecasts_FixedEvents      = mForecasts_FixedEvents(~indexNoData ,:); 
        vDates                      = vDates_subsample(~indexNoData ,:); 
        nDates                      = length(vDates);
        vMonths                     = month(vDates);
        vQuarters                   = NaN(nDates,1);
        for iDate = 1:nDates
            if mod(vMonths(iDate),3)==0
                vQuarters(iDate)    = floor(vMonths(iDate)/3);
            else
                vQuarters(iDate)    = floor(vMonths(iDate)/3)+1;
            end
        end
        vHorizons                   = 3*vQuarters + 12;
        
        % Store already some results
        structCPI_expost(iCountry).Country              = sCountry_of_Interest;
        structCPI_expost(iCountry).Dates                = vDates;
        structCPI_expost(iCountry).FixedHorizons_true   = vForecasts_FixedHorizons;
        structCPI_expost(iCountry).FixedEvents          = mForecasts_FixedEvents;  
       
        
        mInformation_SQAE_quarters  = zeros(length(rho_vec), length(h_subtract_vec), 4);
        mInformation_SQAE           = zeros(length(rho_vec), length(h_subtract_vec));
        
        
        for h_subtract = h_subtract_vec
            
            for indexRho = 1:length(rho_vec)    
                
                rho     = rho_vec(indexRho);
                for iDate = 1:nDates
                    
                    h       = vMonths(iDate) - h_subtract; % assumption about the previous realization known
                    q       = vHorizons(iDate);
                    omega_Forecast   = optweights_hybnew_medfreq_target(12,rho,h,q,4,4);
                    
                    squared_error    = (vForecasts_FixedHorizons(iDate) - ...
                        ((1-omega_Forecast) * mForecasts_FixedEvents(iDate,1) + omega_Forecast*mForecasts_FixedEvents(iDate,2)))^2;
                 	mInformation_SQAE_quarters(indexRho, h_subtract, vQuarters(iDate)) = mInformation_SQAE_quarters(indexRho, h_subtract,  vQuarters(iDate)) + ...
                        squared_error;
                    mInformation_SQAE(indexRho, h_subtract) = mInformation_SQAE(indexRho, h_subtract) + squared_error;
                    
                end
                
            end
            
            % Statistics for all horizons
            % find the minimizing rho
            [minimum, index]                            = min(mInformation_SQAE(:, h_subtract));
            structCPI_expost(iCountry).min_rho_all(h_subtract)      = rho_vec(index);
            structCPI_expost(iCountry).SQAE_minrho_all(h_subtract)  = minimum;
            % find, for comparison, the value corresponding to zero
            structCPI_expost(iCountry).SQAE_rho0_all(h_subtract)    = mInformation_SQAE(index_rho0, h_subtract);
            
            % Improvements for individual quarters
            structCPI_expost(iCountry).Ratio_SQAE_Optimvs0_Q1(h_subtract)     = mInformation_SQAE_quarters(index, h_subtract,1) / mInformation_SQAE_quarters(index_rho0, h_subtract,1);
            structCPI_expost(iCountry).Ratio_SQAE_Optimvs0_Q2(h_subtract)     = mInformation_SQAE_quarters(index, h_subtract,2) / mInformation_SQAE_quarters(index_rho0, h_subtract,2);
            structCPI_expost(iCountry).Ratio_SQAE_Optimvs0_Q3(h_subtract)     = mInformation_SQAE_quarters(index, h_subtract,3) / mInformation_SQAE_quarters(index_rho0, h_subtract,3);
            structCPI_expost(iCountry).Ratio_SQAE_Optimvs0_Q4(h_subtract)     = mInformation_SQAE_quarters(index, h_subtract,4) / mInformation_SQAE_quarters(index_rho0, h_subtract,4);
            
              
        end
    
        plot(rho_vec, mInformation_SQAE)
        title(sCountry_of_Interest)
        
        
        % Store Results
        structCPI_expost(iCountry).mInformation_SQAE            = mInformation_SQAE; 
        structCPI_expost(iCountry).mInformation_SQAE_quarters   = mInformation_SQAE_quarters;
        
        % Construct the union of vector of dates across all countries 
        vUnionDates = union(vUnionDates, vDates);
        
    end
    
%% Perform the search grid across all countries
    
    nDates      = length(vUnionDates);
    vMonths     = month(vUnionDates);
    vQuarters   = NaN(nDates,1);
    for iDate = 1:nDates
        if mod(vMonths(iDate),3)==0
            vQuarters(iDate)    = floor(vMonths(iDate)/3);
        else
            vQuarters(iDate)    = floor(vMonths(iDate)/3)+1;
        end
    end
    vHorizons   = 3*vQuarters + 12;
       
    mInformation_SQAE_allcountries              = zeros(length(rho_vec), length(h_subtract_vec));
    mInformation_SQAE_allcountries_quarters     = zeros(length(rho_vec), length(h_subtract_vec), 4);
       
    structCPI_expost(nCountries+1).Country              = 'All_countries';
    structCPI_expost(nCountries+1).Dates                = vUnionDates;
       
        
    for h_subtract = h_subtract_vec;

        for indexRho = 1:length(rho_vec)    

            rho     = rho_vec(indexRho);
            
            for iDate = 1:nDates

                h       = vMonths(iDate) - h_subtract; % assumption about the previous realization known
                q       = vHorizons(iDate);
                omega_Forecast   = optweights_hybnew_medfreq_target(12,rho,h,q,4,4);
                
                for iCountry = 1:nCountries
                    % Find the current date in the vector of dates
                    % available for each country
                    
                    index_Date = find(structCPI_expost(iCountry).Dates  == vUnionDates(iDate));
                    if ~isempty(index_Date)
                        
                        squared_error   =  (structCPI_expost(iCountry).FixedHorizons_true(index_Date) - ((1-omega_Forecast) * structCPI_expost(iCountry).FixedEvents(index_Date,1) + ...
                            omega_Forecast*structCPI_expost(iCountry).FixedEvents(index_Date,2)))^2;
                        mInformation_SQAE_allcountries(indexRho, h_subtract) = mInformation_SQAE_allcountries(indexRho, h_subtract) + squared_error;
                        mInformation_SQAE_allcountries_quarters(indexRho, h_subtract, vQuarters(iDate)) = ...
                            mInformation_SQAE_allcountries_quarters(indexRho, h_subtract, vQuarters(iDate)) + squared_error;
                        
                    end
                end
            end

        end
        
    % find the minimizing rho
    [minimum, index]            = min(mInformation_SQAE_allcountries(:, h_subtract));
    structCPI_expost(nCountries+1).min_rho_all(h_subtract)      = rho_vec(index);
    structCPI_expost(nCountries+1).SQAE_minrho_all(h_subtract)  = minimum;
    % find, for comparison, the value corresponding to zero
    structCPI_expost(nCountries+1).SQAE_rho0_all(h_subtract)    = mInformation_SQAE_allcountries(index_rho0, h_subtract);
      
    structCPI_expost(nCountries+1).Ratio_SQAE_Optimvs0_Q1(h_subtract)     = mInformation_SQAE_allcountries_quarters(index, h_subtract,1) / mInformation_SQAE_allcountries_quarters(index_rho0, h_subtract,1);
    structCPI_expost(nCountries+1).Ratio_SQAE_Optimvs0_Q2(h_subtract)     = mInformation_SQAE_allcountries_quarters(index, h_subtract,2) / mInformation_SQAE_allcountries_quarters(index_rho0, h_subtract,2);
    structCPI_expost(nCountries+1).Ratio_SQAE_Optimvs0_Q3(h_subtract)     = mInformation_SQAE_allcountries_quarters(index, h_subtract,3) / mInformation_SQAE_allcountries_quarters(index_rho0, h_subtract,3);
    structCPI_expost(nCountries+1).Ratio_SQAE_Optimvs0_Q4(h_subtract)     = mInformation_SQAE_allcountries_quarters(index, h_subtract,4) / mInformation_SQAE_allcountries_quarters(index_rho0, h_subtract,4);
            
            
    end
    
    structCPI_expost(nCountries+1).mInformation_SQAE           = mInformation_SQAE_allcountries;
    structCPI_expost(nCountries+1).mInformation_SQAE_quarters  = mInformation_SQAE_allcountries_quarters;
    
    plot(rho_vec, mInformation_SQAE_allcountries)
   
%%  Save the results
    save([sOutputDirectory 'GridSearch_rho_' sVariable_of_Interest '_' sField_TermStructure, '_', sDateStartSample_full, '_', ...
        sDateEndSample_full], 'structCPI_expost');
  
%% Write it to Excel

     sExcelFile   = ['Results_GridSearch_Infeasiblerho_' sVariable_of_Interest '_' sField_TermStructure, '_', sDateStartSample_full, '_', sDateEndSample_full, '.xlsx'];
     
     for iInfo = 1:length(structCPI_expost)
         
        cExcelOutput            = cell(length(structCPI_expost(iInfo).mInformation_SQAE)+10,length(h_subtract_vec)+1);
        % First line - minimizing rho
        cExcelOutput(1,1)       = {'Minimizing rho'};
        cExcelOutput(1,2:end)   = num2cell(structCPI_expost(iInfo).min_rho_all);
        % Second line - minimized squared approximation error
        cExcelOutput(2,1)       = {'Min. squared appr. error'};
        cExcelOutput(2,2:end)   = num2cell(structCPI_expost(iInfo).SQAE_minrho_all );
        % Third line - squared approximation error for rho = 0
        cExcelOutput(3,1)       = {'Squared appr. error for rho=0'};
        cExcelOutput(3,2:end)   = num2cell(structCPI_expost(iInfo).SQAE_rho0_all);
        % Fourth line - ratios for all quarters
        cExcelOutput(4,1)       = {'Ratio of SQAE for all quarters'};
        cExcelOutput(4,2:end)   = num2cell(structCPI_expost(iInfo).SQAE_minrho_all / structCPI_expost(iInfo).SQAE_rho0_all);
        
        % Fourth, fifth, sixth and seventh line are for ratios for
        % individual quarters
        cExcelOutput(5,1)       = {'Ratio of SQAE for Q1'};
        cExcelOutput(5,2:end)   = num2cell(structCPI_expost(iInfo).Ratio_SQAE_Optimvs0_Q1);
        cExcelOutput(6,1)       = {'Ratio of SQAE for Q2'};
        cExcelOutput(6,2:end)   = num2cell(structCPI_expost(iInfo).Ratio_SQAE_Optimvs0_Q2);
        cExcelOutput(7,1)       = {'Ratio of SQAE for Q3'};
        cExcelOutput(7,2:end)   = num2cell(structCPI_expost(iInfo).Ratio_SQAE_Optimvs0_Q3);
        cExcelOutput(8,1)       = {'Ratio of SQAE for Q4'};
        cExcelOutput(8,2:end)   = num2cell(structCPI_expost(iInfo).Ratio_SQAE_Optimvs0_Q4);
        
        % Next we have a blank line
        
        % Then, in the columns, the first column is the date of the
        % quarterly forecasts
        cExcelOutput(10,1)       = {'rho'};
        cExcelOutput(11:end,1)   = num2cell(rho_vec);
        
        cExcelOutput(10,2)       = {'SQAE - publication lag -1'};
        cExcelOutput(10,3)       = {'SQAE - publication lag -2'};
        cExcelOutput(10,4)       = {'SQAE - publication lag -3'};
        
        cExcelOutput(11:end,2)   = num2cell(structCPI_expost(iInfo).mInformation_SQAE(:,1));
        cExcelOutput(11:end,3)   = num2cell(structCPI_expost(iInfo).mInformation_SQAE(:,2));
        cExcelOutput(11:end,4)   = num2cell(structCPI_expost(iInfo).mInformation_SQAE(:,3));
        
        % Write one sheet per country
        xlswrite([sOutputDirectory_Excel sExcelFile],cExcelOutput,structCPI_expost(iInfo).Country);

     end


