
    clear
    clc;
    close all

    addpath(genpath([cd '\']));

    sInputCE         = '.\..\Files_ConsensusEconomics\CE_Mat_files\';
    sInput          = 'Input\';
    sOutputDirectory_Mat    = 'Output_Mat_files\';
    sOutputDirectory_Excel  = 'Output_Excel\';
    
%%  Input coordinates

    % Specify the variable of interest
    sVariable_of_Interest   = 'Gross Domestic Product';
  
    % Specify the field of interest
    sField_TermStructure    = 'Mean';
    
    % Specify the countries for which the graph should be produced
    vStringCountries_General = fdefineNamesCountries_newformat;
    vAbbreCountries          = fdefineAbbreviationsCountries_newformat;
    nCountries               = length(vStringCountries_General);

%%  Specify which rho to use for computing the approximation
    
    bRho0                   = 1;
    bRho_EmpiricalExpost    = 0;
    bRho_Infeasible         = 0;
    
%%  Time sample
    
    % Full desirable time series availability
    sDateStartSample_full   = 'Oct-1989'; 
    sDateEndSample_full     = 'Mar-2019';
    
    % Specify also the format of these dates
    sFormat                 = 'mmm-yyyy';
      
%%
    dhorizonQuarter = 4;
    
    % Specify the quarterly horizon for the graphs - monthly values
    sFields             = ['FixedHorizon_Q' num2str(dhorizonQuarter) '_YoY'];
    sField_ApprOptim    = ['FixedHorizon_Q' num2str(dhorizonQuarter) '_YoY_Optim'];
    sField_Adhoc        = ['FixedHorizon_Q' num2str(dhorizonQuarter) '_YoY_Adhoc'];

%% Store the data for each country
    
    for iCountry = 1: nCountries
        
         %% FORECASTS DATA
       % Load the structure for each country
        sCountry_of_Interest    = vStringCountries_General{iCountry}
        
        load([sInputCE sCountry_of_Interest  '_raw_quarterly.mat']);
       
        vDates = cell2mat({structCountry.Date_Survey}');
        
        % Find the position of the starting date dates in the list of dates available for that country
        iIndexStart_forecasts    = find(strcmp(cellstr(datestr(vDates, sFormat)), sDateStartSample_full));
        if isempty(iIndexStart_forecasts)
            iIndexStart_forecasts = 1;
        end
        
        % Find the position of the end date dates in the list of dates available for that country
        iIndexEnd_forecasts      = find(strcmp(cellstr(datestr(vDates, sFormat)), sDateEndSample_full));
        
        if isempty(iIndexEnd_forecasts)
            iIndexEnd_forecasts = length(vDates);
        end
        
        
        vIndicesDates       = iIndexStart_forecasts:iIndexEnd_forecasts; 
        nMonthsForecasts    = length(vIndicesDates);
        vDates_subsample    = vDates(vIndicesDates);
        
        
%         % Total number of months
%         nMonthsForecasts        = iIndexEnd_forecasts-iIndexStart_forecasts+1;
%         
%         vDates_subsample        = vDates(iIndexStart_forecasts:iIndexEnd_forecasts);
        
        vForecasts              = NaN(nMonthsForecasts, 1);
        vForecasts_ApprOptim    = NaN(nMonthsForecasts, 1);
        vForecasts_Adhoc        = NaN(nMonthsForecasts, 1);
        vQuarters               = NaN(nMonthsForecasts,1);
        vFixedEvent_Nowcast     = NaN(nMonthsForecasts,1);
        vFixedEvent_Forecast    = NaN(nMonthsForecasts,1);
        vDisagreement_Nowcast   = NaN(nMonthsForecasts,1);
        vDisagreement_Forecast  = NaN(nMonthsForecasts,1);
        
        
        % Use of the function in the monthly version
        if bRho0 % if we compute approximations with rho set to 0
            rho = 0;
        elseif bRho_EmpiricalExpost
            load([sInput 'AutocorrCoef_GDP_2019.mat']);
            % Find the position of the current country in the list of countries 
            indexstructGDP    = find(ismember({AutocorrCoef_GDP.Country},sCountry_of_Interest));
            rho               = AutocorrCoef_GDP(indexstructGDP).CoeffEstimate_Monthly 
        elseif bRho_Infeasible
            load([sOutputDirectory_Mat 'GridSearch_rho_' sVariable_of_Interest '_' sField_TermStructure, '_', sDateStartSample_full, '_', ...
        sDateEndSample_full]);
            % Find the position of the current country in the list of countries 
            indexstructGDP    = find(ismember({structGDP_expost.Country},sCountry_of_Interest));
            rho               = structGDP_expost(indexstructGDP).min_rho_all(1);  
        end
                            
        for iDate = 1:nMonthsForecasts

            % First, find the position of the variable of interest in the structure of each month dhorizons back
            indexVariable       = find(strcmp({structCountry(vIndicesDates(iDate)).Variables.Variable_category},sVariable_of_Interest));

            if ~isempty(indexVariable)

                sField_individually = char(sFields);
                if isfield(structCountry(vIndicesDates(iDate)).Variables(indexVariable), sField_individually) && ...
                        (~isempty(structCountry(vIndicesDates(iDate)).Variables(indexVariable).(sField_individually)))
                        
                    if isfield(structCountry(vIndicesDates(iDate)).Variables(indexVariable).(sField_individually), sField_TermStructure) &&...
                            ~isempty(structCountry(vIndicesDates(iDate)).Variables(indexVariable).(sField_individually).(sField_TermStructure))
                            % Finally store the true fixed-event forecast 
                            vForecasts(iDate) =  structCountry(vIndicesDates(iDate)).Variables(indexVariable).(sField_individually).(sField_TermStructure);

                            %% OPTIMAL & AD-HOC APPROXIMATION
                            % Compute also the optimal approximations for the fixed-horizon forecast
                             dMonth      = month(structCountry(vIndicesDates(iDate)).Date_Survey);
                             dYear       = year(structCountry(vIndicesDates(iDate)).Date_Survey);
                             if mod(dMonth,3)==0
                                dquarter    = floor(dMonth/3);
                             else
                                dquarter    = floor(dMonth/3)+1;
                             end
                             vQuarters(iDate)         = dquarter;
                             
%                              % USE of MALTE's FUNCTION - in the Quartely version
%                              weight_Nowcast_Optim     = 1 - optweights_hybnew_medfreq_target(4, 0, dquarter-1, dquarter+3, 4, 4);
%                              weight_Nowcast_Adhoc     = (4-dquarter+1)/4;

                            % Use of the function in the monthly version
                            % Assuming the last month of the previous quarter is known
%                             weight_Nowcast_Optim     = 1 - optweights_hybnew_medfreq_target(12, rho, (dquarter-1)*3+1, dquarter*3+dhorizonQuarter*3, 4, 4);
                            % Assuming the previous month is known
                            weight_Nowcast_Optim     = 1 - optweights_hybnew_medfreq_target(12, rho, dMonth-1, dquarter*3+dhorizonQuarter*3, 4, 4);
                            
                            if dhorizonQuarter== 4
                                weight_Nowcast_Adhoc     = (4-dquarter)/dhorizonQuarter;
                            elseif dhorizonQuarter== 3
                                  weight_Nowcast_Adhoc     = (4-dquarter+1)/(dhorizonQuarter+1);
                            end

                             
                           if strcmp(structCountry(vIndicesDates(iDate)).Variables(indexVariable).Nowcast.YearForecast,num2str(dYear))&&...
                                    strcmp(structCountry(vIndicesDates(iDate)).Variables(indexVariable).Forecast.YearForecast,num2str(dYear+1))
                                    
                                %% OPTIMAL APPROXIMATION 
%                                 dValue_Optim = weight_Nowcast_Optim * structCountry(iIndexStart_forecasts+iDate-1).Variables(indexVariable).Nowcast.IndividualData + ...
%                                         (1- weight_Nowcast_Optim)* structCountry(iIndexStart_forecasts+iDate-1).Variables(indexVariable).Forecast.IndividualData;
% 
%                                 structCountry(iIndexStart_forecasts+iDate-1).Variables(indexVariable).(sField_ApprOptim).IndividualData  = dValue_Optim; 
%                                 
%                                 % Compute also statistics
%                                 % Mean forecast
%                                 structCountry(iIndexStart_forecasts+iDate-1).Variables(indexVariable).(sField_ApprOptim).Mean            = nanmean(dValue_Optim);
                                structCountry(vIndicesDates(iDate)).Variables(indexVariable).(sField_ApprOptim).Mean = weight_Nowcast_Optim * ...
                                    structCountry(vIndicesDates(iDate)).Variables(indexVariable).Nowcast.Mean + (1- weight_Nowcast_Optim)* structCountry(vIndicesDates(iDate)).Variables(indexVariable).Forecast.Mean;

%                                 % Highest forecast
%                                 structCountry(iIndexStart_forecasts+iDate-1).Variables(indexVariable).(sField_ApprOptim).Max             = nanmax(dValue_Optim);
%                                 % Lowest forecast
%                                 structCountry(iIndexStart_forecasts+iDate-1).Variables(indexVariable).(sField_ApprOptim).Min             = nanmin(dValue_Optim);
%                                 % Standard deviation
%                                 structCountry(iIndexStart_forecasts+iDate-1).Variables(indexVariable).(sField_ApprOptim).StandardDeviation = nanstd(dValue_Optim); 
%                                 
                                vForecasts_ApprOptim(iDate) = structCountry(vIndicesDates(iDate)).Variables(indexVariable).(sField_ApprOptim).(sField_TermStructure);
                                
%                                 vFixedEvent_Nowcast(iDate,:) = nanmean(structCountry(iIndexStart_forecasts+iDate-1).Variables(indexVariable).Nowcast.IndividualData);
                                vFixedEvent_Nowcast(iDate,:) = structCountry(vIndicesDates(iDate)).Variables(indexVariable).Nowcast.Mean;

%                                 vFixedEvent_Forecast(iDate,:) = nanmean(structCountry(iIndexStart_forecasts+iDate-1).Variables(indexVariable).Forecast.IndividualData);
                                vFixedEvent_Forecast(iDate,:) = structCountry(vIndicesDates(iDate)).Variables(indexVariable).Forecast.Mean;

                                
                                %% AD-HOC approximation
%                                 dValue_Adhoc = weight_Nowcast_Adhoc * structCountry(iIndexStart_forecasts+iDate-1).Variables(indexVariable).Nowcast.IndividualData + ...
%                                         (1- weight_Nowcast_Adhoc)* structCountry(iIndexStart_forecasts+iDate-1).Variables(indexVariable).Forecast.IndividualData;
%                                     
%                                 structCountry(iIndexStart_forecasts+iDate-1).Variables(indexVariable).(sField_Adhoc).IndividualData  = dValue_Adhoc; 
% 
%                                 % Compute also statistics
%                                 % Mean forecast
%                                 structCountry(iIndexStart_forecasts+iDate-1).Variables(indexVariable).(sField_Adhoc).Mean            = nanmean(dValue_Adhoc);
                                
                                structCountry(vIndicesDates(iDate)).Variables(indexVariable).(sField_Adhoc).Mean =  weight_Nowcast_Adhoc * ...
                                    structCountry(vIndicesDates(iDate)).Variables(indexVariable).Nowcast.Mean + (1- weight_Nowcast_Adhoc)* ...
                                        structCountry(vIndicesDates(iDate)).Variables(indexVariable).Forecast.Mean;
                      
%                                 % Highest forecast
%                                 structCountry(iIndexStart_forecasts+iDate-1).Variables(indexVariable).(sField_Adhoc).Max             = nanmax(dValue_Adhoc);
%                                 % Lowest forecast
%                                 structCountry(iIndexStart_forecasts+iDate-1).Variables(indexVariable).(sField_Adhoc).Min             = nanmin(dValue_Adhoc);
%                                 % Standard deviation
%                                 structCountry(iIndexStart_forecasts+iDate-1).Variables(indexVariable).(sField_Adhoc).StandardDeviation = nanstd(dValue_Adhoc); 

                                vForecasts_Adhoc(iDate) = structCountry(vIndicesDates(iDate)).Variables(indexVariable).(sField_Adhoc).(sField_TermStructure);
                                
                                vDisagreement_Nowcast(iDate) = structCountry(vIndicesDates(iDate)).Variables(indexVariable).Nowcast.StandardDeviation;
                                vDisagreement_Forecast(iDate) = structCountry(vIndicesDates(iDate)).Variables(indexVariable).Forecast.StandardDeviation;
                           
                               end
                            
                            
                        end
                end
                
            end

        end
        
       %% Store the values and evaluate the approximation
        indexNoData             = sum(isnan(vForecasts),2)==size(vForecasts,2);
        vQuarters               = vQuarters(~indexNoData ,:);
        
        Info(iCountry).Forecast = vForecasts(~indexNoData ,:);
        Info(iCountry).Dates    = vDates_subsample(~indexNoData);
        Info(iCountry).Country  = sCountry_of_Interest;
        Info(iCountry).AbbrCountry  = vAbbreCountries{iCountry};
        
        
        indexNoData                         = sum(isnan(vFixedEvent_Nowcast),2)==size(vFixedEvent_Nowcast,2);
        Info(iCountry).FixedEvent_Nowcast   = vFixedEvent_Nowcast(~indexNoData ,:);
        indexNoData                         = sum(isnan(vFixedEvent_Forecast),2)==size(vFixedEvent_Forecast,2);
        Info(iCountry).FixedEvent_Forecast   = vFixedEvent_Forecast(~indexNoData ,:);
        
        indexNoData                         = sum(isnan(vForecasts_ApprOptim),2)==size(vForecasts_ApprOptim,2);
        Info(iCountry).Forecast_ApprOptim   = vForecasts_ApprOptim(~indexNoData ,:);
        
        
        indexNoData                         = sum(isnan(vForecasts_Adhoc),2)==size(vForecasts_Adhoc,2);
        Info(iCountry).Forecast_Adhoc       = vForecasts_Adhoc(~indexNoData ,:);
        
        % Disagreement
        indexNoData                         = sum(isnan(vDisagreement_Nowcast),2)==size(vDisagreement_Nowcast,2);
        Info(iCountry).Disagreement_Nowcast = vDisagreement_Nowcast(~indexNoData ,:);
        indexNoData                         = sum(isnan(vDisagreement_Forecast),2)==size(vDisagreement_Forecast,2);
        Info(iCountry).Disagreement_Forecast = vDisagreement_Forecast(~indexNoData ,:);
        
        
        Info(iCountry).vSE_ApprOptim_allperiods      = (Info(iCountry).Forecast_ApprOptim - Info(iCountry).Forecast).^2;
        Info(iCountry).vSE_Adhoc_allperiods          = (Info(iCountry).Forecast_Adhoc - Info(iCountry).Forecast).^2;
       
        % MSE for all periods
        Info(iCountry).MSE_ApprOptim_allperiods  = mean((Info(iCountry).Forecast_ApprOptim - Info(iCountry).Forecast).^2);
        Info(iCountry).MSE_Adhoc_allperiods      = mean((Info(iCountry).Forecast_Adhoc - Info(iCountry).Forecast).^2);
        Info(iCountry).phi_MSE_overall           = Info(iCountry).MSE_ApprOptim_allperiods/Info(iCountry).MSE_Adhoc_allperiods;
        
        % MSE for individual quarters
        index_Q1 = vQuarters==1;
        Info(iCountry).MSE_ApprOptim_Q1       = mean((Info(iCountry).Forecast_ApprOptim(index_Q1) - Info(iCountry).Forecast(index_Q1)).^2);
        Info(iCountry).MSE_Adhoc_Q1           = mean((Info(iCountry).Forecast_Adhoc(index_Q1)  - Info(iCountry).Forecast(index_Q1) ).^2);
        Info(iCountry).phi_MSE_Q1             = Info(iCountry).MSE_ApprOptim_Q1/Info(iCountry).MSE_Adhoc_Q1;
        correlation                           = corrcoef(Info(iCountry).Forecast_ApprOptim(index_Q1),Info(iCountry).Forecast(index_Q1));
        Info(iCountry).corrcoef_ApprOptim_Q1  = correlation(1,2);
        correlation                           = corrcoef(Info(iCountry).Forecast_Adhoc(index_Q1),Info(iCountry).Forecast(index_Q1));
        Info(iCountry).corrcoef_Adhoc_Q1      = correlation(1,2);
        correlation_NowcastForecast           = corrcoef(Info(iCountry).FixedEvent_Nowcast(index_Q1),Info(iCountry).FixedEvent_Forecast(index_Q1));
        Info(iCountry).corrcoef_NowcastForecast_Q1  = correlation_NowcastForecast(1,2);
        correlation_NowcastForecast           = corrcoef(Info(iCountry).FixedEvent_Nowcast(index_Q1),Info(iCountry).FixedEvent_Forecast(index_Q1));
        Info(iCountry).corrcoef_NowcastForecast_Q1  = correlation_NowcastForecast(1,2);
         correlation_DisagNowcastForecast      = corrcoef(Info(iCountry).Disagreement_Nowcast(index_Q1),...
            Info(iCountry).Disagreement_Forecast(index_Q1));
        Info(iCountry).DisagNowcastForecast_Q1  = correlation_DisagNowcastForecast(1,2);
        
        
        index_Q2 = vQuarters==2;
        Info(iCountry).MSE_ApprOptim_Q2       = mean((Info(iCountry).Forecast_ApprOptim(index_Q2) - Info(iCountry).Forecast(index_Q2)).^2);
        Info(iCountry).MSE_Adhoc_Q2           = mean((Info(iCountry).Forecast_Adhoc(index_Q2)  - Info(iCountry).Forecast(index_Q2) ).^2);
        Info(iCountry).phi_MSE_Q2             = Info(iCountry).MSE_ApprOptim_Q2/Info(iCountry).MSE_Adhoc_Q2;
        correlation                           = corrcoef(Info(iCountry).Forecast_ApprOptim(index_Q2),Info(iCountry).Forecast(index_Q2));
        Info(iCountry).corrcoef_ApprOptim_Q2  = correlation(1,2);
        correlation                           = corrcoef(Info(iCountry).Forecast_Adhoc(index_Q2),Info(iCountry).Forecast(index_Q2));
        Info(iCountry).corrcoef_Adhoc_Q2      = correlation(1,2);
       correlation_NowcastForecast           = corrcoef(Info(iCountry).FixedEvent_Nowcast(index_Q2),Info(iCountry).FixedEvent_Forecast(index_Q2));
        Info(iCountry).corrcoef_NowcastForecast_Q2  = correlation_NowcastForecast(1,2);
        correlation_DisagNowcastForecast      = corrcoef(Info(iCountry).Disagreement_Nowcast(index_Q2),...
            Info(iCountry).Disagreement_Forecast(index_Q2));
        Info(iCountry).DisagNowcastForecast_Q2  = correlation_DisagNowcastForecast(1,2);
        
        
        index_Q3 = vQuarters==3;
        Info(iCountry).MSE_ApprOptim_Q3       = mean((Info(iCountry).Forecast_ApprOptim(index_Q3) - Info(iCountry).Forecast(index_Q3)).^2);
        Info(iCountry).MSE_Adhoc_Q3           = mean((Info(iCountry).Forecast_Adhoc(index_Q3)  - Info(iCountry).Forecast(index_Q3) ).^2);
        Info(iCountry).phi_MSE_Q3             = Info(iCountry).MSE_ApprOptim_Q3/Info(iCountry).MSE_Adhoc_Q3;
        correlation                           = corrcoef(Info(iCountry).Forecast_ApprOptim(index_Q3),Info(iCountry).Forecast(index_Q3));
        Info(iCountry).corrcoef_ApprOptim_Q3  = correlation(1,2);
        correlation                           = corrcoef(Info(iCountry).Forecast_Adhoc(index_Q3),Info(iCountry).Forecast(index_Q3));
        Info(iCountry).corrcoef_Adhoc_Q3      = correlation(1,2);
       correlation_NowcastForecast           = corrcoef(Info(iCountry).FixedEvent_Nowcast(index_Q3),Info(iCountry).FixedEvent_Forecast(index_Q3));
        Info(iCountry).corrcoef_NowcastForecast_Q3  = correlation_NowcastForecast(1,2);
        correlation_DisagNowcastForecast      = corrcoef(Info(iCountry).Disagreement_Nowcast(index_Q3),...
            Info(iCountry).Disagreement_Forecast(index_Q3));
        Info(iCountry).DisagNowcastForecast_Q3  = correlation_DisagNowcastForecast(1,2);
        
        
        index_Q4 = vQuarters==4;
        Info(iCountry).MSE_ApprOptim_Q4       = mean((Info(iCountry).Forecast_ApprOptim(index_Q4) - Info(iCountry).Forecast(index_Q4)).^2);
        Info(iCountry).MSE_Adhoc_Q4           = mean((Info(iCountry).Forecast_Adhoc(index_Q4)  - Info(iCountry).Forecast(index_Q4) ).^2);
        Info(iCountry).phi_MSE_Q4             = Info(iCountry).MSE_ApprOptim_Q4/Info(iCountry).MSE_Adhoc_Q4;
        correlation                           = corrcoef(Info(iCountry).Forecast_ApprOptim(index_Q4),Info(iCountry).Forecast(index_Q4));
        Info(iCountry).corrcoef_ApprOptim_Q4  = correlation(1,2);
        correlation                           = corrcoef(Info(iCountry).Forecast_Adhoc(index_Q4),Info(iCountry).Forecast(index_Q4));
        Info(iCountry).corrcoef_Adhoc_Q4      = correlation(1,2);
         correlation_NowcastForecast           = corrcoef(Info(iCountry).FixedEvent_Nowcast(index_Q4),Info(iCountry).FixedEvent_Forecast(index_Q4));
        Info(iCountry).corrcoef_NowcastForecast_Q4  = correlation_NowcastForecast(1,2);
        correlation_DisagNowcastForecast      = corrcoef(Info(iCountry).Disagreement_Nowcast(index_Q4),...
            Info(iCountry).Disagreement_Forecast(index_Q4));
        Info(iCountry).DisagNowcastForecast_Q4  = correlation_DisagNowcastForecast(1,2);
        
        Info(iCountry).psi_allquarters      = [Info(iCountry).phi_MSE_Q1; Info(iCountry).phi_MSE_Q2; Info(iCountry).phi_MSE_Q3; Info(iCountry).phi_MSE_Q4]; 
        Info(iCountry).corrcoef_allq_Optim  = [Info(iCountry).corrcoef_ApprOptim_Q1; Info(iCountry).corrcoef_ApprOptim_Q2; Info(iCountry).corrcoef_ApprOptim_Q3; Info(iCountry).corrcoef_ApprOptim_Q4]; 
        Info(iCountry).corrcoef_allq_Adhoc  = [Info(iCountry).corrcoef_Adhoc_Q1; Info(iCountry).corrcoef_Adhoc_Q2; Info(iCountry).corrcoef_Adhoc_Q3; Info(iCountry).corrcoef_Adhoc_Q4]; 
            
    end
    
    
%% Save the info

    if bRho0
        save([sOutputDirectory_Mat 'Info_' sVariable_of_Interest '_' sField_TermStructure '_Q' num2str(dhorizonQuarter) '_Rho0_' sDateStartSample_full '_'  sDateEndSample_full '.mat'],'Info');
    elseif bRho_EmpiricalExpost
        save([sOutputDirectory_Mat 'Info_' sVariable_of_Interest '_' sField_TermStructure '_Q' num2str(dhorizonQuarter) '_RhoEmpiricalExPost_' sDateStartSample_full '_'  sDateEndSample_full '.mat'],'Info');
    elseif bRho_Infeasible
        save([sOutputDirectory_Mat 'Info_' sVariable_of_Interest '_' sField_TermStructure '_Q' num2str(dhorizonQuarter) '_RhoInfeasible_'  sDateStartSample_full '_'  sDateEndSample_full '.mat'],'Info');
    end
    
%% Write it in Excel

    sExcelFile                 = ['Quarterly_' sVariable_of_Interest '_' sField_TermStructure '_MonthlyFreq_Q' num2str(dhorizonQuarter) '_' sDateEndSample_full '.xlsx'];
 
    for iInfo = 1:length(Info)
         
        cExcelOutput            = cell(length(Info(iInfo).Dates)+6,8); % 1 line for the header and 5 more for the MSE
        
        % MSE - all periods
        indexLine = 1;
        cExcelOutput(indexLine,2)       = {'MSE'};
        cExcelOutput(indexLine,3)       = num2cell(Info(iInfo).MSE_ApprOptim_allperiods);
        cExcelOutput(indexLine,4)       = num2cell(Info(iInfo).MSE_Adhoc_allperiods);
        
        cExcelOutput(2,2)       = {'Ratio MSE - Q1'}; cExcelOutput(2,3) = num2cell(Info(iInfo).phi_MSE_Q1);
        cExcelOutput(3,2)       = {'Ratio MSE - Q2'}; cExcelOutput(3,3) = num2cell(Info(iInfo).phi_MSE_Q2);
        cExcelOutput(4,2)       = {'Ratio MSE - Q3'}; cExcelOutput(4,3) = num2cell(Info(iInfo).phi_MSE_Q3);
        cExcelOutput(5,2)       = {'Ratio MSE - Q4'}; cExcelOutput(5,3) = num2cell(Info(iInfo).phi_MSE_Q4);
        
         % Correlation of disagreement for forecast and nowcast
        cExcelOutput(2,4)       = {'Corr. coeff disag. nowcast and forecast - Q1'}; cExcelOutput(2,5) = num2cell(Info(iInfo).DisagNowcastForecast_Q1);
        cExcelOutput(3,4)       = {'Corr. coeff disag. nowcast and forecast - Q2'}; cExcelOutput(3,5) = num2cell(Info(iInfo).DisagNowcastForecast_Q2);
        cExcelOutput(4,4)       = {'Corr. coeff disag. nowcast and forecast - Q3'}; cExcelOutput(4,5) = num2cell(Info(iInfo).DisagNowcastForecast_Q3);
        cExcelOutput(5,4)       = {'Corr. coeff disag. nowcast and forecast - Q4'}; cExcelOutput(5,5) = num2cell(Info(iInfo).DisagNowcastForecast_Q4);
        
        
        indexLine = 6;
        % Header
        cExcelOutput(indexLine,1)       = {'Survey date'};
        cExcelOutput(indexLine,2)       = {sFields};
        cExcelOutput(indexLine,3)       = {sField_ApprOptim};
        cExcelOutput(indexLine,4)       = {sField_Adhoc};
        cExcelOutput(indexLine,5)       = {'Mean Nowcast'};
        cExcelOutput(indexLine,6)       = {'Mean Forecast'};
        cExcelOutput(indexLine,7)       = {'Disag Nowcast'};
        cExcelOutput(indexLine,8)       = {'Disag Forecast'};
      
        
        cExcelOutput(indexLine+1:end,1)   = num2cell(Info(iInfo).Dates-693960);
        cExcelOutput(indexLine+1:end,2)   = num2cell(Info(iInfo).Forecast);
        cExcelOutput(indexLine+1:end,3)   = num2cell(Info(iInfo).Forecast_ApprOptim);
        cExcelOutput(indexLine+1:end,4)   = num2cell(Info(iInfo).Forecast_Adhoc);
        cExcelOutput(indexLine+1:end,5)   = num2cell(Info(iInfo).FixedEvent_Nowcast);
        cExcelOutput(indexLine+1:end,6)   = num2cell(Info(iInfo).FixedEvent_Forecast);
        cExcelOutput(indexLine+1:end,7)   = num2cell(Info(iInfo).Disagreement_Nowcast);
        cExcelOutput(indexLine+1:end,8)   = num2cell(Info(iInfo).Disagreement_Forecast);
        
        
        if bRho0
            xlswrite([sOutputDirectory_Excel sExcelFile],cExcelOutput,[Info(iInfo).AbbrCountry '_Rho0']);
        elseif bRho_EmpiricalExpost
            xlswrite([sOutputDirectory_Excel sExcelFile],cExcelOutput,[Info(iInfo).AbbrCountry '_RhoEmpirical']);
        elseif bRho_Infeasible
            xlswrite([sOutputDirectory_Excel sExcelFile],cExcelOutput,[Info(iInfo).AbbrCountry '_RhoExPost']);
        end
        
     end


