
    clear
    clc;
    close all

    addpath(genpath([cd '\']));

    sInputDirectory         = '.\..\Files_ConsensusEconomics\CE_Mat_files\';
    sOutputDirectory_Mat    = 'Output_Mat_files\';
    sOutputDirectory_Excel  = 'Output_Excel\';
    

%%  Input coordinates

    % Specify the variable of interest
    sVariable_of_Interest   = 'Consumer Prices';
  
    % Specify the field of interest
    sField_TermStructure    = 'StandardDeviation';
    
    % Specify the countries for which the graph should be produced
    vStringCountries_General = fdefineNamesCountries_newformat;
    vAbbreCountries          = fdefineAbbreviationsCountries_newformat;
    nCountries               = length(vStringCountries_General);
    
%%  Time sample
    
    % Full desirable time series availability
    sDateStartSample_full   = 'Oct-1989'; 
    sDateEndSample_full     = 'Apr-2019';
    
    % Specify also the format of these dates
    sFormat                 = 'mmm-yyyy';
   
%%  Specify which rho to use for computing the approximation
    
    bRho0               = 1;

%% Particularities:
   
% Japan:            1996 to 1998, 2013 to 2015 
% Germany:       2006 to 2007
% France:           1994 to 1996
% UK:                   1990 to 1992, 2008 to 2012
% Netherlands: 2011 to 2013
% Spain:              1991 to 1993, 2009 to 2013
    
    cCountries_VAT      = {'Japan','Germany', 'France','UK', 'Netherlands', 'Spain'};
    
    for iCountry = 1:length(cCountries_VAT)
        structCountriesVAT(iCountry).Country = cCountries_VAT(iCountry);
    end
    
    iCountry = 1;
    structCountriesVAT(iCountry).StartDates =  {'01-Jan-1996','01-Jan-2013'};
    structCountriesVAT(iCountry).EndDates =  {'31-Dec-1998','31-Dec-2015'};
    
    iCountry = 2;
    structCountriesVAT(iCountry).StartDates =  {'01-Jan-2006'};
    structCountriesVAT(iCountry).EndDates =  {'31-Dec-2007'};
    
    iCountry = 3;
    structCountriesVAT(iCountry).StartDates =  {'01-Jan-1994'};
    structCountriesVAT(iCountry).EndDates =  {'31-Dec-1996'};
   
    iCountry = 4;
    structCountriesVAT(iCountry).StartDates =  {'01-Jan-1990','01-Jan-2008'};
    structCountriesVAT(iCountry).EndDates =  {'31-Dec-1992','31-Dec-2012'};
   
    iCountry = 5;
    structCountriesVAT(iCountry).StartDates =  {'01-Jan-2011'};
    structCountriesVAT(iCountry).EndDates =  {'31-Dec-2013'};
  
    iCountry = 6;
    structCountriesVAT(iCountry).StartDates =  {'01-Jan-1991','01-Jan-2009'};
    structCountriesVAT(iCountry).EndDates =  {'31-Dec-1993','31-Dec-2013'};

%%
    dhorizonQuarter      = 4;
   
    % Specify the quarterly horizon for the graphs - monthly values
    sFields             = ['FixedHorizon_Q' num2str(dhorizonQuarter) '_YoY'];
    sField_ApprOptim    = ['FixedHorizon_Q' num2str(dhorizonQuarter) '_YoY_Optim'];
    sField_Adhoc        = ['FixedHorizon_Q' num2str(dhorizonQuarter) '_YoY_Adhoc'];

%% Store the data for each country
    
    for iCountry = 1: nCountries
        
        bParticularities_VAT = 0; % for each country the assumption is that initially there are no VAT particularities
        
       % FORECASTS DATA
       % Load the structure for each country
        sCountry_of_Interest    = vStringCountries_General{iCountry}
        
        load([sInputDirectory sCountry_of_Interest '_raw_quarterly.mat']);
       
        % only for UK replace Consumer Prices with Retail Prices
         if strcmp(sCountry_of_Interest, 'UK')
            sVariable_of_Interest_country   = 'Retail Prices'; %Consumer Prices
        else
            sVariable_of_Interest_country = sVariable_of_Interest;
        end
        
        vDates = cell2mat({structCountry.Date_Survey}');
        
        % Find whether the country features VAT changes particularities
        if find(ismember(cCountries_VAT,sCountry_of_Interest))
            bParticularities_VAT    = 1;
            indexCountry            = find(ismember(cCountries_VAT,sCountry_of_Interest),1);
            vStartExludingDates = structCountriesVAT(indexCountry).StartDates; 
            vEndExcludingDates  = structCountriesVAT(indexCountry).EndDates;
            nPeriodsExclusion   = length(vStartExludingDates); 
        end
        
        % Find the position of the starting date dates in the list of dates available for that country
        iIndexStart_forecasts    = find(strcmp(cellstr(datestr(vDates, sFormat)), sDateStartSample_full));
        if isempty(iIndexStart_forecasts)
            iIndexStart_forecasts = 1;
        end
        
        % Find the position of the end date dates in the list of dates available for that country
        iIndexEnd_forecasts      = find(strcmp(cellstr(datestr(vDates, sFormat)), sDateEndSample_full));
        if isempty(iIndexEnd_forecasts)
            iIndexEnd_forecasts = length(vDates);
        end
        
        vIndicesDates           = iIndexStart_forecasts:iIndexEnd_forecasts; 
        
        % Find the position of the starting dates in the list of dates available for that country
        if bParticularities_VAT
            for iPeriod = 1:nPeriodsExclusion
                vIndexEnd_temp(iPeriod)      = find(vDates> datenum(char(vEndExcludingDates(iPeriod))),1)-1;
                vIndexStart_temp(iPeriod)    = find(vDates> datenum(char(vStartExludingDates(iPeriod))),1);
                vMonths_tobeExcluded = vIndexStart_temp(iPeriod): vIndexEnd_temp(iPeriod);
                [~,ind] = ismember(vMonths_tobeExcluded, vIndicesDates);
                vIndicesDates(ind) = [];
            end
            
            nMonthsForecasts    = length(vIndicesDates);
            vDates_subsample    = vDates(vIndicesDates);
        else
            clear vIndexEnd_temp vIndexStart_temp
            % Total number of months
            nMonthsForecasts        = iIndexEnd_forecasts-iIndexStart_forecasts+1;
            vDates_subsample        = vDates(iIndexStart_forecasts:iIndexEnd_forecasts);
            vIndicesDates           = [iIndexStart_forecasts:iIndexEnd_forecasts]'; 
        end
        
        
        vForecasts              = NaN(nMonthsForecasts, 1);
        vForecasts_ApprOptim    = NaN(nMonthsForecasts, 1);
        vForecasts_Adhoc        = NaN(nMonthsForecasts, 1);
        vQuarters               = NaN(nMonthsForecasts,1);
        vFixedEvent_StDev_Nowcast     = NaN(nMonthsForecasts,1);
        vFixedEvent_StDev_Forecast    = NaN(nMonthsForecasts,1);
       
        
         % Use of the function in the monthly version
        if bRho0 % if we compute approximations with rho set to 0
            rho = 0;
        end
        
        
        for iDate = 1:nMonthsForecasts

            % First, find the position of the variable of interest in the structure of each month dhorizons back
            indexVariable       = find(strcmp({structCountry(vIndicesDates(iDate)).Variables.Variable_category},sVariable_of_Interest_country));
            
            if ~isempty(indexVariable)
                
                sField_individually = char(sFields);
                if isfield(structCountry(vIndicesDates(iDate)).Variables(indexVariable), sField_individually) && ...
                            (~isempty(structCountry(vIndicesDates(iDate)).Variables(indexVariable).(sField_individually)))
                        
                    if isfield(structCountry(vIndicesDates(iDate)).Variables(indexVariable).(sField_individually), sField_TermStructure) &&...
                                ~isempty(structCountry(vIndicesDates(iDate)).Variables(indexVariable).(sField_individually).(sField_TermStructure))
                            
                            % Store the true fixed-event forecast 
                            vForecasts(iDate) =  structCountry(vIndicesDates(iDate)).Variables(indexVariable).(sField_individually).(sField_TermStructure);
                            
                            %% OPTIMAL & AD-HOC APPROXIMATION
                            % Compute also the optimal approximations for the fixed-horizon forecast
                             dMonth      = month(structCountry(vIndicesDates(iDate)).Date_Survey);
                             dYear       = year(structCountry(vIndicesDates(iDate)).Date_Survey);
                             if mod(dMonth,3)==0
                                dquarter    = floor(dMonth/3);
                             else
                                dquarter    = floor(dMonth/3)+1;
                             end
                             vQuarters(iDate)         = dquarter;
                             
                            % Use of the function in the monthly version
                             weight_Nowcast_Optim     = 1 - optweights_hybnew_medfreq_target(12, rho, dMonth-1, dquarter*3+dhorizonQuarter*3, 4, 4);

                             if dhorizonQuarter==4
                                weight_Nowcast_Adhoc    = (4-dquarter)/dhorizonQuarter;
                             elseif dhorizonQuarter==3
                                weight_Nowcast_Adhoc    = (4-dquarter+1)/(dhorizonQuarter+1);
                             end
                            
                            if strcmp(structCountry(vIndicesDates(iDate)).Variables(indexVariable).Nowcast.YearForecast,num2str(dYear))&&...
                                    strcmp(structCountry(vIndicesDates(iDate)).Variables(indexVariable).Forecast.YearForecast,num2str(dYear+1))
                                    
                                %% OPTIMAL APPROXIMATION 
                                dValue_Optim = weight_Nowcast_Optim * structCountry(vIndicesDates(iDate)).Variables(indexVariable).Nowcast.IndividualData + ...
                                        (1- weight_Nowcast_Optim)* structCountry(vIndicesDates(iDate)).Variables(indexVariable).Forecast.IndividualData;

                                structCountry(vIndicesDates(iDate)).Variables(indexVariable).(sField_ApprOptim).IndividualData  = dValue_Optim; 

                                % Compute also statistics
                                % Mean forecast
                                structCountry(vIndicesDates(iDate)).Variables(indexVariable).(sField_ApprOptim).Mean            = nanmean(dValue_Optim);
                                % Highest forecast
                                structCountry(vIndicesDates(iDate)).Variables(indexVariable).(sField_ApprOptim).Max             = nanmax(dValue_Optim);
                                % Lowest forecast
                                structCountry(vIndicesDates(iDate)).Variables(indexVariable).(sField_ApprOptim).Min             = nanmin(dValue_Optim);
                                % Standard deviation
                                structCountry(vIndicesDates(iDate)).Variables(indexVariable).(sField_ApprOptim).StandardDeviation = nanstd(dValue_Optim); 

                                vForecasts_ApprOptim(iDate) = structCountry(vIndicesDates(iDate)).Variables(indexVariable).(sField_ApprOptim).(sField_TermStructure);
                                
                                vFixedEvent_StDev_Nowcast(iDate,:) = nanmean(structCountry(vIndicesDates(iDate)).Variables(indexVariable).Nowcast.StandardDeviation);
                                vFixedEvent_StDev_Forecast(iDate,:) = nanmean(structCountry(vIndicesDates(iDate)).Variables(indexVariable).Forecast.StandardDeviation);
                            
                            
                                %% AD-HOC approximation
                                dValue_Adhoc = weight_Nowcast_Adhoc * structCountry(vIndicesDates(iDate)).Variables(indexVariable).Nowcast.IndividualData + ...
                                        (1- weight_Nowcast_Adhoc)* structCountry(vIndicesDates(iDate)).Variables(indexVariable).Forecast.IndividualData;
                                    
                                structCountry(vIndicesDates(iDate)).Variables(indexVariable).(sField_Adhoc).IndividualData  = dValue_Adhoc; 

                                % Compute also statistics
                                % Mean forecast
                                structCountry(vIndicesDates(iDate)).Variables(indexVariable).(sField_Adhoc).Mean            = nanmean(dValue_Adhoc);
                                % Highest forecast
                                structCountry(vIndicesDates(iDate)).Variables(indexVariable).(sField_Adhoc).Max             = nanmax(dValue_Adhoc);
                                % Lowest forecast
                                structCountry(vIndicesDates(iDate)).Variables(indexVariable).(sField_Adhoc).Min             = nanmin(dValue_Adhoc);
                                % Standard deviation
                                structCountry(vIndicesDates(iDate)).Variables(indexVariable).(sField_Adhoc).StandardDeviation = nanstd(dValue_Adhoc); 
                                
                                vForecasts_Adhoc(iDate) = structCountry(vIndicesDates(iDate)).Variables(indexVariable).(sField_Adhoc).(sField_TermStructure);
                                
                            end
                            
                    end
                        
                end 
            end

        end
        
       %% Store the values and evaluate the approximation
        indexNoData             = sum(isnan(vForecasts),2)==size(vForecasts,2);
        vQuarters               = vQuarters(~indexNoData ,:);
        vForecasts              = vForecasts(~indexNoData ,:);
       
        Info(iCountry).Forecast = vForecasts;
        Info(iCountry).Dates    = vDates_subsample(~indexNoData);
        Info(iCountry).Country  = sCountry_of_Interest;
        Info(iCountry).AbbrCountry  = vAbbreCountries{iCountry};
        
        indexNoData                 = sum(isnan(vForecasts_ApprOptim),2)==size(vForecasts_ApprOptim,2);
        vForecasts_ApprOptim        = vForecasts_ApprOptim(~indexNoData ,:);
        
        Info(iCountry).Forecast_ApprOptim   = vForecasts_ApprOptim;
        
        % Seasonal adjustment of time series of disagreement for the
        % ad-hoc approximation
        indexNoData                         = sum(isnan(vForecasts_Adhoc),2)==size(vForecasts_Adhoc,2);
        vForecasts_Adhoc                    = vForecasts_Adhoc(~indexNoData ,:);
        
        indexNoData                         = sum(isnan(vFixedEvent_StDev_Nowcast),2)==size(vFixedEvent_StDev_Nowcast,2);
        Info(iCountry).FixedEvent_StDev_Nowcast   = vFixedEvent_StDev_Nowcast(~indexNoData ,:);
        indexNoData                         = sum(isnan(vFixedEvent_StDev_Forecast),2)==size(vFixedEvent_StDev_Forecast,2);
        Info(iCountry).FixedEvent_StDev_Forecast   = vFixedEvent_StDev_Forecast(~indexNoData ,:);
        
        
        Info(iCountry).Forecast_Adhoc       = vForecasts_Adhoc;
        
        Info(iCountry).vSE_ApprOptim_allperiods       = (Info(iCountry).Forecast_ApprOptim - Info(iCountry).Forecast).^2;
        Info(iCountry).vSE_Adhoc_allperiods           = (Info(iCountry).Forecast_Adhoc - Info(iCountry).Forecast).^2;
       
        % Mean for all periods
        Info(iCountry).MSE_ApprOptim_allperiods       = mean((Info(iCountry).Forecast_ApprOptim - Info(iCountry).Forecast).^2);
        Info(iCountry).MSE_Adhoc_allperiods           = mean((Info(iCountry).Forecast_Adhoc - Info(iCountry).Forecast).^2);
        Info(iCountry).Bias_Adhoc_allperiods          = mean(Info(iCountry).Forecast_Adhoc - Info(iCountry).Forecast);
        Info(iCountry).Bias_ApprOptim_allperiods      = mean(Info(iCountry).Forecast_ApprOptim - Info(iCountry).Forecast);
                
        % MSE for individual quarters
        index_Q1 = vQuarters==1;
        Info(iCountry).MSE_ApprOptim_Q1       = mean((Info(iCountry).Forecast_ApprOptim(index_Q1) - Info(iCountry).Forecast(index_Q1)).^2);
        Info(iCountry).MSE_Adhoc_Q1           = mean((Info(iCountry).Forecast_Adhoc(index_Q1)  - Info(iCountry).Forecast(index_Q1) ).^2);
        Info(iCountry).phi_MSE_Q1             = Info(iCountry).MSE_ApprOptim_Q1/Info(iCountry).MSE_Adhoc_Q1;
        Info(iCountry).Bias_ApprOptim_Q1       = mean(Info(iCountry).Forecast_ApprOptim(index_Q1) - Info(iCountry).Forecast(index_Q1));
        Info(iCountry).Bias_Adhoc_Q1           = mean(Info(iCountry).Forecast_Adhoc(index_Q1)  - Info(iCountry).Forecast(index_Q1));
        correlation                           = corrcoef(Info(iCountry).Forecast_ApprOptim(index_Q1),Info(iCountry).Forecast(index_Q1));
        Info(iCountry).corrcoef_ApprOptim_Q1  = correlation(1,2);
        correlation                           = corrcoef(Info(iCountry).Forecast_Adhoc(index_Q1),Info(iCountry).Forecast(index_Q1));
        Info(iCountry).corrcoef_Adhoc_Q1      = correlation(1,2);
        correlation_NowcastForecast           = corrcoef(Info(iCountry).FixedEvent_StDev_Nowcast(index_Q1),Info(iCountry).FixedEvent_StDev_Forecast(index_Q1));
        Info(iCountry).corrcoef_NowcastForecast_Q1  = correlation_NowcastForecast(1,2);
        
      
        index_Q2 = vQuarters==2;
        Info(iCountry).MSE_ApprOptim_Q2       = mean((Info(iCountry).Forecast_ApprOptim(index_Q2) - Info(iCountry).Forecast(index_Q2)).^2);
        Info(iCountry).MSE_Adhoc_Q2           = mean((Info(iCountry).Forecast_Adhoc(index_Q2)  - Info(iCountry).Forecast(index_Q2) ).^2);
        Info(iCountry).phi_MSE_Q2             = Info(iCountry).MSE_ApprOptim_Q2/Info(iCountry).MSE_Adhoc_Q2;
        Info(iCountry).Bias_ApprOptim_Q2       = mean(Info(iCountry).Forecast_ApprOptim(index_Q2) - Info(iCountry).Forecast(index_Q2));
        Info(iCountry).Bias_Adhoc_Q2           = mean(Info(iCountry).Forecast_Adhoc(index_Q2)  - Info(iCountry).Forecast(index_Q2));
        correlation                           = corrcoef(Info(iCountry).Forecast_ApprOptim(index_Q2),Info(iCountry).Forecast(index_Q2));
        Info(iCountry).corrcoef_ApprOptim_Q2  = correlation(1,2);
        correlation                           = corrcoef(Info(iCountry).Forecast_Adhoc(index_Q2),Info(iCountry).Forecast(index_Q2));
        Info(iCountry).corrcoef_Adhoc_Q2      = correlation(1,2);
        correlation_NowcastForecast           = corrcoef(Info(iCountry).FixedEvent_StDev_Nowcast(index_Q2),Info(iCountry).FixedEvent_StDev_Forecast(index_Q2));
        Info(iCountry).corrcoef_NowcastForecast_Q2  = correlation_NowcastForecast(1,2);
       
        index_Q3 = vQuarters==3;
        Info(iCountry).MSE_ApprOptim_Q3       = mean((Info(iCountry).Forecast_ApprOptim(index_Q3) - Info(iCountry).Forecast(index_Q3)).^2);
        Info(iCountry).MSE_Adhoc_Q3           = mean((Info(iCountry).Forecast_Adhoc(index_Q3)  - Info(iCountry).Forecast(index_Q3) ).^2);
        Info(iCountry).phi_MSE_Q3             = Info(iCountry).MSE_ApprOptim_Q3/Info(iCountry).MSE_Adhoc_Q3;
        Info(iCountry).Bias_ApprOptim_Q3       = mean(Info(iCountry).Forecast_ApprOptim(index_Q3) - Info(iCountry).Forecast(index_Q3));
        Info(iCountry).Bias_Adhoc_Q3           = mean(Info(iCountry).Forecast_Adhoc(index_Q3)  - Info(iCountry).Forecast(index_Q3));
        correlation                           = corrcoef(Info(iCountry).Forecast_ApprOptim(index_Q3),Info(iCountry).Forecast(index_Q3));
        Info(iCountry).corrcoef_ApprOptim_Q3  = correlation(1,2);
        correlation                           = corrcoef(Info(iCountry).Forecast_Adhoc(index_Q3),Info(iCountry).Forecast(index_Q3));
        Info(iCountry).corrcoef_Adhoc_Q3      = correlation(1,2);
        correlation_NowcastForecast           = corrcoef(Info(iCountry).FixedEvent_StDev_Nowcast(index_Q3),Info(iCountry).FixedEvent_StDev_Forecast(index_Q3));
        Info(iCountry).corrcoef_NowcastForecast_Q3  = correlation_NowcastForecast(1,2);
       
        index_Q4 = vQuarters==4;
        Info(iCountry).MSE_ApprOptim_Q4       = mean((Info(iCountry).Forecast_ApprOptim(index_Q4) - Info(iCountry).Forecast(index_Q4)).^2);
        Info(iCountry).MSE_Adhoc_Q4           = mean((Info(iCountry).Forecast_Adhoc(index_Q4)  - Info(iCountry).Forecast(index_Q4) ).^2);
        Info(iCountry).phi_MSE_Q4             = Info(iCountry).MSE_ApprOptim_Q4/Info(iCountry).MSE_Adhoc_Q4;
        Info(iCountry).Bias_ApprOptim_Q4       = mean(Info(iCountry).Forecast_ApprOptim(index_Q4) - Info(iCountry).Forecast(index_Q4));
        Info(iCountry).Bias_Adhoc_Q4           = mean(Info(iCountry).Forecast_Adhoc(index_Q4)  - Info(iCountry).Forecast(index_Q4));
        correlation                           = corrcoef(Info(iCountry).Forecast_ApprOptim(index_Q4),Info(iCountry).Forecast(index_Q4));
        Info(iCountry).corrcoef_ApprOptim_Q4  = correlation(1,2);
        correlation                           = corrcoef(Info(iCountry).Forecast_Adhoc(index_Q4),Info(iCountry).Forecast(index_Q4));
        Info(iCountry).corrcoef_Adhoc_Q4      = correlation(1,2);
        correlation_NowcastForecast           = corrcoef(Info(iCountry).FixedEvent_StDev_Nowcast(index_Q4),Info(iCountry).FixedEvent_StDev_Forecast(index_Q4));
        Info(iCountry).corrcoef_NowcastForecast_Q4  = correlation_NowcastForecast(1,2);

        Info(iCountry).psi_allquarters      = [Info(iCountry).phi_MSE_Q1; Info(iCountry).phi_MSE_Q2; Info(iCountry).phi_MSE_Q3; Info(iCountry).phi_MSE_Q4]; 
        Info(iCountry).Bias_allq_Optim      = [Info(iCountry).Bias_ApprOptim_Q1; Info(iCountry).Bias_ApprOptim_Q2; Info(iCountry).Bias_ApprOptim_Q3; Info(iCountry).Bias_ApprOptim_Q4]; 
        Info(iCountry).Bias_allq_Adhoc      = [Info(iCountry).Bias_Adhoc_Q1; Info(iCountry).Bias_Adhoc_Q2; Info(iCountry).Bias_Adhoc_Q3; Info(iCountry).Bias_Adhoc_Q4]; 
        Info(iCountry).corrcoef_allq_Optim  = [Info(iCountry).corrcoef_ApprOptim_Q1; Info(iCountry).corrcoef_ApprOptim_Q2; Info(iCountry).corrcoef_ApprOptim_Q3; Info(iCountry).corrcoef_ApprOptim_Q4]; 
        Info(iCountry).corrcoef_allq_Adhoc  = [Info(iCountry).corrcoef_Adhoc_Q1; Info(iCountry).corrcoef_Adhoc_Q2; Info(iCountry).corrcoef_Adhoc_Q3; Info(iCountry).corrcoef_Adhoc_Q4]; 
       
        
    end
    
%% Save the info
    
    sRho = ['_rho_' num2str(rho)];
    
    save([sOutputDirectory_Mat 'Info_' sVariable_of_Interest '_' sField_TermStructure '_Q' num2str(dhorizonQuarter) sRho '_' sDateStartSample_full '_'  sDateEndSample_full  '.mat'],'Info');
    
%% Write it in Excel

    sExcelFile                 = ['Quarterly_' sVariable_of_Interest '_' sField_TermStructure '_Q' num2str(dhorizonQuarter) '_' sDateStartSample_full '_'  sDateEndSample_full '.xlsx'];
    
     for iInfo = 1:length(Info)
         
        cExcelOutput            = cell(length(Info(iInfo).Dates)+3,4); % 1 line for the header 1 for the MSE and 1 for mean disagreement
        
        % MSE - all periods
        indexLine = 1;
        cExcelOutput(indexLine,2)       = {'MSE'};
        cExcelOutput(indexLine,3)       = num2cell(Info(iInfo).MSE_ApprOptim_allperiods);
        cExcelOutput(indexLine,4)       = num2cell(Info(iInfo).MSE_Adhoc_allperiods);
        
        % Mean disagreement
        indexLine = 2;
        cExcelOutput(indexLine,1)       = {'Mean disagreement'};
        cExcelOutput(indexLine,2)       = num2cell(mean(Info(iInfo).Forecast));
        cExcelOutput(indexLine,3)       = num2cell(mean(Info(iInfo).Forecast_ApprOptim));
        cExcelOutput(indexLine,4)       = num2cell(mean(Info(iInfo).Forecast_Adhoc));
        
        % Header
        indexLine = 3;
        cExcelOutput(indexLine,1)       = {'Survey date'};
        cExcelOutput(indexLine,2)       = {sFields};
        cExcelOutput(indexLine,3)       = {sField_ApprOptim};
        cExcelOutput(indexLine,4)       = {sField_Adhoc};
        
        cExcelOutput(indexLine+1:end,1)   = num2cell(Info(iInfo).Dates-693960);
        cExcelOutput(indexLine+1:end,2)   = num2cell(Info(iInfo).Forecast);
        cExcelOutput(indexLine+1:end,3)   = num2cell(Info(iInfo).Forecast_ApprOptim);
        cExcelOutput(indexLine+1:end,4)   = num2cell(Info(iInfo).Forecast_Adhoc);
        
        if bRho0
            xlswrite([sOutputDirectory_Excel sExcelFile],cExcelOutput,[Info(iInfo).AbbrCountry '_Rho0']);
        end
        
     end


