%   Code averaging the IQR variable over time for all possible horizons

    clear
    clc;
    close all

    addpath(genpath([cd '\']));

    bUseInterpolatedData    = 1;

    sInputDirectory         = '.\..\Files_ConsensusEconomics\CE_Mat_files\';
    sOutputDirectory_Excel  = 'Output_Excel\';
    
    
%%  Input coordinates

    % Specify the variable of interest
    sVariable_of_Interest   = 'Consumer Prices';
%     sVariable_of_Interest   = 'Gross Domestic Product';
  
  
    % Specify the field of interest
    sField_TermStructure    = 'NoForecasts';
    
    % Specify the countries for which the graph should be produced
    vStringCountries_General = fdefineNamesCountries_newformat;
     nCountries              = length(vStringCountries_General);
    
%%  Time sample
    
    % Full desirable time series availability
    sDateStartSample_full   = 'Oct-1989'; 
    sDateEndSample_full     = 'Apr-2019';
    
    % Specify also the format of these dates
    sFormat                 = 'mmm-yyyy';
    
%%
    % Specify the quarterly horizon for the graphs - monthly values
    dhorizonQuarter     = 4;
    sFields             = ['FixedHorizon_Q' num2str(dhorizonQuarter) '_YoY'];
    
%% Plot the data for each country
       
   
    for iCountry = 1: nCountries
        
         %% FORECASTS DATA
       % Load the structure for each country
        sCountry_of_Interest    = vStringCountries_General{iCountry}
        
        load([sInputDirectory sCountry_of_Interest  '_raw_Quarterly.mat']);
       
         % only for UK replace Consumer Prices with Retail Prices, if
         % interested in Inflation. Otherwise leave this if commented out
        if strcmp(sCountry_of_Interest, 'UK')&& strcmp(sVariable_of_Interest, 'Consumer Prices')
             
            sVariable_of_Interest_country   = 'Retail Prices'; 
        else
            sVariable_of_Interest_country   = sVariable_of_Interest; 
        end
        
        vDates = cell2mat({structCountry.Date_Survey}');
        
        % Find the position of the starting date dates in the list of dates available for that country
        iIndexStart_forecasts    = find(strcmp(cellstr(datestr(vDates, sFormat)), sDateStartSample_full));
        if isempty(iIndexStart_forecasts)
            iIndexStart_forecasts = 1;
        end
        % Find the position of the end date dates in the list of dates available for that country
        iIndexEnd_forecasts      = find(strcmp(cellstr(datestr(vDates, sFormat)), sDateEndSample_full));
        if isempty(iIndexEnd_forecasts)
            iIndexEnd_forecasts = length(vDates);
        end
        
        % Total number of months
        nMonthsForecasts        = iIndexEnd_forecasts-iIndexStart_forecasts+1;
        
        vDates_subsample        = vDates(iIndexStart_forecasts:iIndexEnd_forecasts);
        
        %%  For each month identify the short-term forecasts and, where available, the long-term forecasts
        vNoForecasts          = NaN(nMonthsForecasts,1);
        
        for iDate = 1:nMonthsForecasts

            % First, find the position of the variable of interest in the structure of each month dhorizons back
            indexVariable       = find(strcmp({structCountry(iIndexStart_forecasts+iDate-1).Variables.Variable_category},sVariable_of_Interest_country));
            if ~isempty(indexVariable)
                sField_individually = char(sFields);
                
                indexField       = find(strcmp(fieldnames(structCountry(iIndexStart_forecasts+iDate-1).Variables(indexVariable)),sField_individually));
            
                if ~isempty(indexField)
                
                    indexInfo = find(strcmp(fieldnames(structCountry(iIndexStart_forecasts+iDate-1).Variables(indexVariable).(sField_individually)), sField_TermStructure));
                    if ~isempty(indexInfo) 
                        vNoForecasts(iDate, :) = structCountry(iIndexStart_forecasts+iDate-1).Variables(indexVariable).(sField_individually).(sField_TermStructure);
                    end
                end
            end
       
        end
        
       
        Info(iCountry).Country          = sCountry_of_Interest;
        Info(iCountry).Dates            = vDates_subsample();
        Info(iCountry).NoForecasts       = vNoForecasts;
        Info(iCountry).meanNoForecasts     = nanmean(vNoForecasts);        
        
        
    end
    
%% Write it in Excel

     sExcelFile                 = ['NrForecasters_' sVariable_of_Interest '_', sDateStartSample_full, '_', sDateEndSample_full '.xlsx'];
     
     for iInfo = 1:length(Info)
         
        cExcelOutput            = cell(length(Info(iInfo).Dates)+2,2);
        cExcelOutput(1,1)       = {sFields};
        cExcelOutput(1,2)       = num2cell(Info(iInfo).meanNoForecasts);
        cExcelOutput(2,2:end)   = {sFields};
        cExcelOutput(2,1)       = {'Survey date'};
        cExcelOutput(3:end,1)   = num2cell(Info(iInfo).Dates-693960);
        cExcelOutput(3:end,2:end) = num2cell(Info(iInfo).NoForecasts);
        
         xlswrite([sOutputDirectory_Excel sExcelFile],cExcelOutput,[Info(iInfo).Country '_' char(sFields)]);

     end


