clc;
close all

% example: what is the weight for fixed-event forecast of CURRENT year made in MARCH of the current year in order to approximate fixed-horizon forecast for the growth rate between Q1 current year and
% Q1 next  year? 
% Assuming that value in February is known
% use [w,~] = optweights_quarterly_target_monthly_process(m=12,rho=0,h=2,q=15,mm=4,nn=4)
% m = 12 means that we have a monthly DGP
% rho = 0 is the AR-coefficient in the monthly DGP
% h = 2 means that we know the first two values (Jan and Feb) from the
% current year, counting January of the current year as the first month.
% q = 15 means that the last 
% last month to be forecast is the 15th month, counting January of the current year as the first month.
% Thus the 15th month is the 3rd month of the next year (we are interested in the Q1 growth rate of the next year,  
% and the last value needed to compute it is the March value of the
% next year.)
% mm = 4 means that, concerning the fixed-horizon forecasts to be approximated, we are looking at QUARTERLY growth rates 
% nn = 4 means that, concerning the fixed-horizon forecasts to be
% approximated, we want to compare the future level of interest (next
% year Q1) to the level that was attained 4 quarters before.

% When we are interested in the inflation rate comparing Q1 of the
% next year to Q1 of the current based on the forecast in made in FEBRUARY of the current year, we only
% have to change the number of known months from h=2 to h=1 (assuming that inflation 
% in January is known)

% When we are interested in the inflation rate comparing Q2 of the
% next year to Q2 of the current based on the forecast in JUNE  of the current year, we
% have to change the number of known months h to h=5 (assuming that inflation 
% in May is known), and the number of the last month to be forecast to 
% q= 18 (corresponding to June of next year, i.e. the last month in Q2 of next
% year)

m = 12;
rho = 0.00;
hvec = 0:11;
qvec = [15 15 15  18 18 18  21 21 21  24 24 24];
mm = 4;
nn = 4;



w_opt = NaN(12,1);
expected_squared_approx_error_opt = NaN(12,1);

w_adhoc = NaN(12,1);
expected_squared_approx_error_adhoc = NaN(12,1);

for i = 1:m
    [w_opt(i) , expected_squared_approx_error_opt(i)] = optweights_quarterly_target_monthly_process(m,rho,hvec(i),qvec(i),mm,nn);
    
    % The ad hoc weights are calculated as
    w_adhoc(i) = 1 - (i)/12; %Might also be calculated as 1-(i-1)/12
    % because for the first forecast, 1 quarter from the next year and 3
    % quarters from the current year have to be forecast etc
    [~ , expected_squared_approx_error_adhoc(i)] = optweights_quarterly_target_monthly_process(m,rho,hvec(i),qvec(i),mm,nn,w_adhoc(i));
end

months = ['Jan'; 'Feb'; 'Mar'; 'Apr'; 'May'; 'Jun'; 'Jul'; 'Aug'; 'Sep'; 'Oct'; 'Nov'; 'Dec'];
for i = 1:m
  
    q = mod( qvec(i)/(m/mm) -1 , mm) + 1;
    
    quarters(i,1:2) = ['Q' num2str(q)];
    
    if (qvec(i) >= m+1)&&(qvec(i) <= 2*m);
        year_end(i,:) = 'next year   ';
    elseif (qvec(i) >= 1)&&(qvec(i) <= m);
        year_end(i,:) = 'current year';
    end
    
    if (qvec(i) - nn*(m/mm) >= 1)&&(qvec(i) - nn*(m/mm) <= m);
        year_start(i,:) = 'current year ';
    elseif (qvec(i) - nn*(m/mm) >= -m+1)&&(qvec(i) - nn*(m/mm) <= 0);
        year_start(i,:) = 'previous year';
    end             
end
    
%quarters = ['Q1'; 'Q1'; 'Q1'; 'Q2'; 'Q2'; 'Q2'; 'Q3'; 'Q3'; 'Q3'; 'Q4'; 'Q4'; 'Q4'];

weights_results = cell(12,3);
expected_squared_approx_error_results = cell(12,3);
%description = cell(12,3);
for i = 1:12; 
    weights_results{i,1} = months(i,:);
    weights_results{i,2} = w_opt(i,:);
    weights_results{i,3} = w_adhoc(i,:);
    
    expected_squared_approx_error_results{i,1} = months(i,:);
    expected_squared_approx_error_results{i,2} = expected_squared_approx_error_opt(i);
    expected_squared_approx_error_results{i,3} = expected_squared_approx_error_adhoc(i);
    
    %Target to be approximated: 
    part1 = 'month when forecast is made:  ' ;
    part2 = [ months(i,:)];
    part3 = [part2 ' current year ---> '] ;
    part4 = [part3 ['( level ' quarters(i,:) ' ' year_end(i,:) ' ) / '] ];
    description(i,:) = [part4 ['( level ' quarters(i,:) ' ' year_start(i,:) ' ) - 1.'] ];
    
    
end

disp(' ');
disp('Month when forecast is made --- > Year-on-year rate to be approximated');
disp(description);

disp(' ');
disp('Month when forecast is made; optimal weights; ad hoc weights   to be applied to the current-year forecast'); 
disp(weights_results);

disp('Month when forecast is made; expected squared approx errors of optimal weights; and of ad-hoc weights'); 
disp(expected_squared_approx_error_results);


% And now the figure that maybe could be reproduced with the empirical data
ratio_expected_squared_approx_errors = expected_squared_approx_error_opt ./ expected_squared_approx_error_adhoc;

figure(1);
hold on;
plot(cell2mat(weights_results(:,2)),':bs','LineWidth',2);
xlim([1 12]);
ylim([-0.1 1]);
legend('weight on current-year forecast','Location','NorthWest'); 
set(gca,'XTick',1:12);
set(gca,'XTickLabel',months);
hold off;


figure(2);
hold on;
plot(ratio_expected_squared_approx_errors,':bs','LineWidth',2);
xlim([1 12]);
ylim([0 1]);
legend('ratio of expected squared approx errors','Location','NorthWest'); 
set(gca,'XTick',1:12);
set(gca,'XTickLabel',months);
hold off;
