clc;
close all
clear all
expflag = 1;
% suppose we have annual and quarterly data. The first 2 quarters in the
% current year are known, we are interested in the y-o-y rate of Q3 in
% the next year (made in, say, the beginning of Q3 in the current year). 
% The mean of y is 0, y is iid., i.e. y_t = e_t
% Then all forecasts for q-o-q rates after the 2nd quarter of the current year
% are zero. The forecast for the y-o-y rate of Q3 in the next year thus
% equals zero. The weights for the current- and next-year forecast hence
% have to be chosen such that the expected squared sum of v_1*e_1 , v_2*e_2
% , ..., v_6*e_6 is minimized, where v_i are the weights linking annual and quarterly variables, i.e. 
% v_1 = 0.00, v_2 = 0.25, v_3 = 0.50, v_4 = 0.75, v_5 = 1.00, v_6 = 0.75 for the current year forecast 
% v_1 = 0.00, v_2 = 0.00, v_3 = 0.00, v_4 = 0.00, v_5 = 0.00, v_6 = 0.25 for the next year forecast 
% Since the expected e_t^2 is one, we simply
% need to minimize the term ( v_1.^2 + v_2.^2 + ... + v_6.^2 )
% If we use the weight w = 0 for the current year forecast, we only have
% V_6 = 0.25 from the next year forecast which is different from zero. The
% expected squared error is 0.25^2 = 0.625
% If we use the weight w = -0.01 for the current year forecast, we ...
% 
k = 4;
n_known = 6; % number of periods known in previous year (4) + in current year (e.g. 2)


c = [0:1/k:1 (1-1/k):-1/k:1/k zeros(1,k)]';
n = [zeros(1,k) 0:1/k:1 (1-1/k):-1/k:1/k ]';
f = [zeros(1,6) ones(1,k) zeros(1,2)]';
wvec = -0.15:0.0025:0.1;
ct = 0;
disp('weight on current-year forecast (left) and corresponding squared approx error (right)');
for w = wvec
    ct = ct + 1;
    weights_first_n_known_prel = f - (w*c + (1-w)*n); 
    weights_first_n_known = weights_first_n_known_prel(1:n_known); 
    %disp(weights_first_n_known);
    %disp(sum(abs(weights_first_n_known)));
    abserr(ct,1) = sum(abs(weights_first_n_known));
    sqerr(ct,1) = sum(weights_first_n_known.^2);
    disp([w sqerr(ct)]);
end
% figure;
% plot(wvec,[sqerr],'LineWidth',2);
% xlabel('$w$','Interpreter','latex');
% ylim([0 max(sqerr)]);
% figure;
% plot(wvec,[abserr]);

[wstar,~] = optweights_monthly_target_and_process(k,0,n_known-k,n_known,k);
w = 0;
weights_first_n_known_prel = f - (w*c + (1-w)*n); 
weights_first_n_known_w0 = weights_first_n_known_prel(1:n_known); 
sqerr_w0 = sum(weights_first_n_known_w0.^2);
w = wstar;
weights_first_n_known_prel = f - (w*c + (1-w)*n); 
weights_first_n_known_wstar = weights_first_n_known_prel(1:n_known); 
sqerr_wstar = sum(weights_first_n_known_wstar.^2);
plot_w = [weights_first_n_known_w0(n_known-k:n_known).^2 weights_first_n_known_wstar(n_known-k:n_known).^2];
figure;
plot_w2 = [plot_w ; sum(plot_w)];
bar(plot_w2);
xaxisproperties= get(gca, 'XAxis'); xaxisproperties.TickLabelInterpreter = 'latex';
xticklabels({'$\varepsilon_{-2}$','$\varepsilon_{-1}$','$\varepsilon_{0}$','$\varepsilon_{1}$','$\varepsilon_{2}$','total'});
legend('$w=0$','$w=-0.0588$','Location','NorthWest','Interpreter','latex');
if expflag == 1
    saveas(gcf,['./contributions_shocks_negative_weight.pdf']);
end



    