clc;
close all
clear all
%addpath('.\..\..\..\mfiles');
% file that calculates weights shown in Knppel & Vladu
% example is designed for the case of approximating 11-month-ahead forecasts of monthly
% year-on-year inflation rates, given forecasts for annual inflation in the
% current and the next year. In January 2016, one would want to approximate
% the year-on-year inflation rate in December 2016 (the growth rate that relates the
% price level in December 2016 to the level in December 2015), based on
% annual forecasts for 2016 and 2017. In February 2016, one would want to
% approximate the year-on-year inflation rate in January 20167 etc.


m = 12;
rhovec = [0 0.5 0.8 0.99];

deffigpos = [280   350   560   420];
%deffigpos = get(0,'defaultfigureposition');
nw = numel(rhovec);
w_vec = NaN(m,nw);    % vector of optimal weights
w_dfs_vec = NaN(m,1);% vector of ad-hoc weights
approx_error = NaN(m,nw);     % approximation errors (squared) of optimal weights
approx_error_dfs = NaN(m,nw); % approximation errors (squared) of ad-hoc weights
% legend entries for plots
a = cell(nw+1,1);
b = cell(nw,1);

for i = 1:m
    w_dfs_vec(i) = 1-(i-1)/m;     % ad-hoc weights, Dovern-et-al. definition, 
       
    irho = 0;
    for rho = rhovec
        irho = irho + 1;
            % with i = 1, we are in January
            % m = 12, because montly is high frequency and annual
            % the low frequency
            % h = i-1, because the last observed value is the one from
            % December, when the forecast is made in January
            % q = i+m-1, because in January, we want to forecast the
            % (monthly y-o-y)
            % rate in December
            % n = m = 12 , because we want to forecast the y-o-y rate
            
            % optimal weights        
            [w_vec(i,irho),approx_error(i,irho)] = optweights_monthly_target_and_process(m,rho,i-1,i+m-1,m);
            % approximation errors of ad-hoc weights          
            [~,approx_error_dfs(i,irho)] =         optweights_monthly_target_and_process(m,rho,i-1,i+m-1,m, w_dfs_vec(i));
  
            a{irho} = ['w^{\ast}, \rho = ' num2str(rho) ];
            b{irho} = ['\rho = ' num2str(rho) ];
                       
    end
end
a{irho+1} = 'w^{adhoc}';

disp('optimal and ad-hoc weights (last column) for current year forecasts, for different values of rho (first line)');
disp('Following 12 lines refer to January, February, ..., December');
disp('Weights displayed correspond to those in Online Appendix Table I');
disp(' ');
disp( [rhovec NaN]);
disp([w_vec w_dfs_vec]);


% 2 plots
set(0,'defaultfigureposition',deffigpos);
months = ['Jan' ; 'Feb'; 'Mar'; 'Apr'; 'May'; 'Jun';'Jul';'Aug';'Sep';'Oct';'Nov';'Dec'];

figure(1);
hold on;
plot(1:12,w_vec,'LineWidth',2.5);
plot(1:12,w_dfs_vec,':b','LineWidth',2.5);
xlim([1 12]);
hline(0,'k-');
legend(a); 
set(gca,'XTick',1:12);
set(gca,'XTickLabel',months);
%set(gca,'XTickLabel',{'-pi','-pi/2','0','pi/2','pi'})
title('optimal and ad-hoc weights');
hold off;

figure(2);
hold on;
%plot(1:12,approx_error,'LineWidth',2);
%plot(1:12,approx_error_dfs2,':','LineWidth',2);
plot(1:12,(approx_error./approx_error_dfs),'LineWidth',2.5);
xlim([1 12]);
hline(0,'k-');
legend(b,'Location','NorthWest'); 
set(gca,'XTick',1:12);
set(gca,'XTickLabel',['Jan' ; 'Feb'; 'Mar'; 'Apr'; 'May'; 'Jun';'Jul';'Aug';'Sep';'Oct';'Nov';'Dec']);
title('ratio of expected squared approximation errors, optimal vs ad-hoc');
hold off;

