%% Clear command window and memory
    clc
    clear
    close all

    sOutputGraphs       = 'Graphs\';

    dhorizonQuarter = 4; % one-year ahead fixed-horizon forecasts
   

%% Consensus forecasts (monthly high-frequency)
% Define x-axis data (starting at -11)
    x = -11:24;  % x-axis range starts at -11
    tau = 3; % for CE in March

    % Define x-axis data (starting at -11)
    x = -11:24;  % x-axis range starts at -11
    m = 12;
    mm = 4; % mm is the frequency of the medium-frequency target. In the case of a
    % quartely target, mm = 4;
    nn = 4; % nn is the number of periods between the first and the last medium-frequency
    % period for which the growth rate of the corresponding levels is to be
    % forecast

% nn is the number of periods between the first and the last medium-frequency
% period for which the growth rate of the corresponding levels is to be
% forecast
    
    for tau = [3 6 9 12] % 3 for CE in March, 6 for CE in June, 9 for CE in Sep, 12 for CE in Dec
    

         if mod(tau,3)==0
            dquarter    = floor(tau/3);
         else
            dquarter    = floor(tau/3)+1;
         end
         
        % q is the number of the last high-frequency period to be forecast (for the fixed-horizon forecast), counted
        % from the first high-frequency period in the current year (which has the
        % number 1)
        q = dquarter*3+dhorizonQuarter*3;
        
        % Patton-Timmermann weights
        omegak = NaN(2*m,1);
        for i = 1:(2*m)
            omegak(i) = 1 - abs(i-m)/m;
        end
    
    % What is the number of the last medium-frequency period to be forecast (Q1 current year = 1, Q2 current year = 2 etc)?
    % months per quarter
        mpq  = m/mm
        omegak2 = NaN(2*mpq,1);
        for i = 1:(2*mpq)
            omegak2(i) = 1 - abs(i-mpq)/mpq;
        end

        n = nn*mpq;
        Aq = flip([zeros(1,2*m-q) omegak2(1:mpq)' ones(1,n - mpq) omegak2(mpq+1:2*mpq)' zeros(1,m+q-n-mpq)]);

        % Classical and simplest example of Patton and Timmerman
        k = [1: 24];
        b_SPF = 12;
        vWeights = flip(1- abs(k-b_SPF)/b_SPF);
        
        y1_panel2_realised = [vWeights(1:12+tau-1) nan(1, 24-tau+1)];  % Green line in Panel 2, starts at x = 0
        y1_panel2_forecast = [nan(1,12+tau-1) vWeights(12+tau:end) nan(1,12)];  % Green line in Panel 2, starts at x = 0
        y2_panel2_realised = [nan(1, 12), vWeights(1:tau-1) nan(1,24-tau+1)];  % Red line in Panel 2, starts at x = 12
        y2_panel2_forecast = [nan(1, 12+tau-1), vWeights(tau:end)];  % Red line in Panel 2, starts at x = 12
        
        Aq_nonzero = Aq(12+tau-3+1:12+12+tau);
        
        y3_panel2_realised = [nan(1, 12+tau-3)  Aq_nonzero(1:mpq-1) nan(1, length(Aq_nonzero)-mpq+1) nan(1,36-24-tau)]; 
        y3_panel2_forecast = [nan(1, 12+tau-3)  nan(1,mpq-1) Aq_nonzero(mpq:end) nan(1,36-24-tau)]; 
        
%     [nan(1, 12+tau-1), ones(1,12), nan(1,12-tau+1)];  % Blue flat line with bars

        % Create the figure
        figure;
        f111 = plot(x, [vWeights nan(1,12)], 'g:', 'LineWidth', 1.5 );  % Green line (triangular), realised
        hold on        
        f222 = plot(x, [nan(1,12) vWeights], 'r:', 'LineWidth', 1.5 );  % Green line (triangular), realised
        hold on
        f333 = plot(x, [nan(1, 12+tau-3) Aq_nonzero nan(1,36-24-tau)], ':b', 'LineWidth', 1.5 );  % Green line (triangular), realised
        hold on
        f11 = plot(x, y1_panel2_realised, 'g-+', 'LineWidth', 1.5,  'LineStyle', 'none' );  % Green line (triangular), realised
        hold on
        f12 = plot(x, y1_panel2_forecast, 'g-o', 'LineWidth', 1.5, 'LineStyle', 'none' );  % Green line (triangular), forecast
        hold on
        f31 = plot(x, y3_panel2_realised, 'b-+', 'LineWidth', 3,  'LineStyle', 'none');  % Blue flat line without error barshold off, realised
        hold on
        f32 = plot(x, y3_panel2_forecast, 'b-o', 'LineWidth', 3,  'LineStyle', 'none');  % Blue flat line without error barshold off, forecast
        hold on
        f21 = plot(x, y2_panel2_realised, 'r-+', 'LineWidth', 1.5,  'LineStyle', 'none' );  % Red line, realised
        hold on
        f22 = plot(x, y2_panel2_forecast, 'r-o', 'LineWidth', 1.5,  'LineStyle', 'none' );  % Red line, realised
        hold on
        hold on
        xline(tau, '--k', 'LineWidth', 2);
        xlabel('month');
    %     ylabel('Panel 2');
        xlim([-11, 24]);  % Set x-axis limits starting from -11
        ylim([0 1.05]);    % Adjust y-axis limits for better visibility
        xticks([-11: 1: 23]) % Make visible the granulariy of the the x-axis 

        hhh=get(gca,'children');
        for i = 1:length(hhh)
            if strcmp(get(hhh(i),'type'),'line')        % check if it's a line
                set(hhh(i),'markersize',10);
            end
        end
        %fontsize(scale=1.25);


        switch tau
        case 3
            sFileTitle = 'Illustration_Consensus_March';
            title('Weights of observed and forecasted month-on-month growth rates in...');
            legend([f111, f222, f333], '... current-year annual growth rate', ...
            '... next-year annual growth rate', ...
            '... next year''s Q1 y-o-y growth rate','Location','west');
        case 6
            sFileTitle = 'Illustration_Consensus_June';
            title('Weights of observed and forecasted month-on-month growth rates in...');
            legend([f111, f222, f333], '... current-year annual growth rate', ...
            '... next-year annual growth rate', ...
            '... next year''s Q2 y-o-y growth rate','Location','west');

        case 9
            sFileTitle = 'Illustration_Consensus_September';
            title('Weights of observed and forecasted month-on-month growth rates in...');
            legend([f111, f222, f333], '... current-year annual growth rate', ...
            '... next-year annual growth rate', ...
            '... next year''s Q3 y-o-y growth rate','Location','west');

        case 12
            sFileTitle = 'Illustration_Consensus_December';
            title('Weights of observed and forecasted month-on-month growth rates in...');
            legend([f111, f222, f333], '... current-year annual growth rate', ...
            '... next-year annual growth rate', ...
            '... next year''s Q4 y-o-y growth rate','Location','west');

        end

        h = gcf;

        set(h, 'PaperUnits','centimeters');
        set(h,'PaperPosition', [1 1 28 19]);
        set(h,'PaperOrientation','landscape');
        % Save it in the pdf format for the paper
        print(gcf, '-dpdf', [sOutputGraphs sFileTitle]); 
    %         print(gcf, '-depsc', [sOutputGraphs sFileTitle]); 

    end
    
% end

%% SPF forecasts (quarterly high-frequency)

% Define x-axis data (starting at -3)
    x = -3:8;  % x-axis range starts at -3
    
for tau = [1 2 3 4] % 1 for US SPF in February, 2 for US SPF in May, 3 for US SPF in August, 4 for US SPF in November
    
    k = [1: 8];
    b_SPF = 4;
    vWeights = flip(1- abs(k-b_SPF)/b_SPF);
    
    y1_panel2_realised = [vWeights(1:4+tau-1) nan(1, 8-tau+1)];  % Green line in Panel 2, starts at x = 0
    y1_panel2_forecast = [nan(1,4+tau-1) vWeights(4+tau:end) nan(1,4)];  % Green line in Panel 2, starts at x = 0
    y2_panel2_realised = [nan(1, 4), vWeights(1:tau-1) nan(1,8-tau+1)];  % Red line in Panel 2, starts at x = 12
    y2_panel2_forecast = [nan(1, 4+tau-1), vWeights(tau:end)];  % Red line in Panel 2, starts at x = 12
    y3_panel2 = [nan(1, 4+tau), ones(1,4), nan(1,4-tau)];  % Blue flat line with bars

     % Create the figure
    figure;

    f111 = plot(x, [vWeights nan(1,4)], 'g:', 'LineWidth', 1.5 );  % Green line (triangular), realised
    hold on        
    f222 = plot(x, [nan(1,4) vWeights], 'r:', 'LineWidth', 1.5 );  % Green line (triangular), realised
    hold on
    f333 = plot(x, y3_panel2, 'b:', 'LineWidth', 1.5 );  % Green line (triangular), realised
    hold on

    f3=plot(x, y3_panel2, 'b-o', 'LineWidth', 3,  'LineStyle', 'none');  % Blue flat line without error barshold off;
    hold on
    f11=plot(x, y1_panel2_realised, 'g-+', 'LineWidth', 1.5,  'LineStyle', 'none');  % Green line (triangular), realised
    hold on
    f12=plot(x, y1_panel2_forecast, 'g-o', 'LineWidth', 1.5,  'LineStyle', 'none');  % Green line (triangular), forecast
    hold on
    f21=plot(x, y2_panel2_realised, 'r-+', 'LineWidth', 1.5,  'LineStyle', 'none');  % Red line, realised
    hold on
    f22=plot(x, y2_panel2_forecast, 'r-o', 'LineWidth', 1.5,  'LineStyle', 'none');  % Red line, realised
    hold on
    
    xline(tau, '--k', 'LineWidth', 2);
    xlabel('quarter');
%     ylabel('Panel 2');
    xlim([-3, 8]);  % Set x-axis limits starting from -11
    ylim([0 1.05]);    % Adjust y-axis limits for better visibility
    xticks([-3: 1: 8]) % Make visible the granulariy of the the x-axis 
    % legend([f11, f21, f3], '... quarterly growth rates in current year annual inflation rate', ...
    %     '... quarterly growth rates in next year annual inflation rate', ...
    %     '1-year ahead y-o-y inflation rate');

     hhh=get(gca,'children');
    for i = 1:length(hhh)
        if strcmp(get(hhh(i),'type'),'line')        % check if it's a line
            set(hhh(i),'markersize',15);
        end
    end
    %fontsize(scale=1.25);
    
    switch tau
    case 1
        sFileTitle = 'Illustration_SPF_Feb';
        title('Weights of observed and forecasted quarter-on-quarter growth rates in...');
        legend([f111, f222, f333], '... current-year annual growth rate', ...
            '... next-year annual growth rate', ...
            '... next year''s Q1 y-o-y growth rate','Location','west');
    case 2
        sFileTitle = 'Illustration_SPF_May';
        title('Weights of observed and forecasted quarter-on-quarter growth rates in...');
        legend([f111, f222, f333], '... current-year annual growth rate', ...
            '... next-year annual growth rate', ...
            '... next year''s Q2 y-o-y growth rate','Location','west');

    case 3
        sFileTitle = 'Illustration_SPF_Aug';
        title('Weights of observed and forecasted quarter-on-quarter growth rates in...');
        legend([f111, f222, f333], '... current-year annual growth rate', ...
            '... next-year annual growth rate', ...
            '... next year''s Q3 y-o-y growth rate','Location','west');

    case 4
        sFileTitle = 'Illustration_SPF_Nov';
        title('Weights of observed and forecasted quarter-on-quarter growth rates in...');
        legend([f111, f222, f333], '... current-year annual growth rate', ...
            '... next-year annual growth rate', ...
            '... next year''s Q4 y-o-y growth rate','Location','west');

    end
    
    h = gcf;
    
    set(h, 'PaperUnits','centimeters');
    set(h,'PaperPosition', [1 1 28 19]);
    set(h,'PaperOrientation','landscape');
    % Save it in the pdf format for the paper
    print(gcf, '-dpdf', [sOutputGraphs sFileTitle]); 
%         print(gcf, '-depsc', [sOutputGraphs sFileTitle]); 

    end
    
