clc;
close all
clear all
% example: what is the weight for fixed-event forecast of CURRENT year made in MARCH of the current year in order to approximate fixed-horizon forecast for the growth rate between February current year and
% February next year? 
% Assuming that value in February is known
% use [w,~] = optweights_monthly_target_and_process(m=12,rho=0,h=2,q=14,n=12)
% m = 12 means that we have a monthly DGP
% rho = 0 is the AR-coefficient in the monthly DGP
% h = 2 means that we know the first two values (Jan and Feb) from the
% current year, counting January of the current year as the first month.
% q = 14 means that the last 
% last month to be forecast is the 14th month, counting January of the current year as the first month.
% Thus the 14th month is the 2nd month of the next year, ie February next year
% n = 12 means that, concerning the fixed-horizon forecasts to be
% approximated, we want to compare the future level of interest (next
% year February) to the level that was attained 12 months before.

% When we are interested in the growth rate comparing February of the
% next year to February of the current based on the forecast in made in FEBRUARY of the current year, we only
% have to change the number of known months from h=2 to h=1 (assuming that inflation 
% in January is known)

% When we are interested in the growth rate comparing the level in APRIL of the
% next year to the level in APRIL of the current year based on the forecast in MARCH  of the current year, we
% only have to set the number of known months h to h=2 (assuming that the level 
% in February is known), and the number of the last month to be forecast to 
% q = 16 (corresponding to April of next year)

m = 12;
rho = 0.0;
hvec = 0:11;
qvec = 12:23;
n = 12;


w_opt = NaN(12,2);
expected_squared_approx_error_opt = NaN(12,1);

w_adhoc = NaN(12,1);
expected_squared_approx_error_adhoc = NaN(12,1);

w_opt_2years = NaN(12,1);
expected_squared_approx_error_opt_2years = NaN(12,1);

for i = 1:m
    [w_opt(i,1:2) , expected_squared_approx_error_opt(i)] = optweights_monthly_target_and_process_lastcurrentnextyear(m,rho,hvec(i),qvec(i),n);
    
    [w_opt_2years(i) , expected_squared_approx_error_opt_2years(i)] = optweights_monthly_target_and_process(m,rho,hvec(i),qvec(i),n);
    
    % The ad hoc weights are calculated as
    w_adhoc(i) = 1 - (i-1)/12; %Might also be calculated as 1-i/12
    % because for the first forecast, 1 quarter from the next year and 3
    % quarters from the current year have to be forecast etc
    % would give weight of zero to known growth rate from previous year, so
    % I can continue to use the result based on current and next year's
    % growth rates only
    [~ , expected_squared_approx_error_adhoc(i)] = optweights_monthly_target_and_process(m,rho,hvec(i),qvec(i),n,w_adhoc(i));
end

months = ['Jan'; 'Feb'; 'Mar'; 'Apr'; 'May'; 'Jun'; 'Jul'; 'Aug'; 'Sep'; 'Oct'; 'Nov'; 'Dec'];
for i = 1:m
  
    q = mod( qvec(i)  - 1 , m ) + 1;
    qs = mod( qvec(i)  - n - 1 , m ) + 1;
    
    target_month_end(i,1:3) = months(q,:);
    target_month_start(i,1:3) = months(qs,:);
    
    if (qvec(i) >= m+1)&&(qvec(i) <= 2*m);
        year_end(i,:) = 'next year   ';
    elseif (qvec(i) >= 1)&&(qvec(i) <= m);
        year_end(i,:) = 'current year';
    end
    
    if (qvec(i) -n >= m+1)&&(qvec(i) -n <= 2*m);
        year_start(i,:) = 'nextyear     ';         
    elseif (qvec(i) -n >= 1)&&(qvec(i) -n <= m);
        year_start(i,:) = 'current year ';
    elseif (qvec(i) -n >= -m+1)&&(qvec(i) -n <= 0);
        year_start(i,:) = 'previous year';
    end             
end
    
%quarters = ['Q1'; 'Q1'; 'Q1'; 'Q2'; 'Q2'; 'Q2'; 'Q3'; 'Q3'; 'Q3'; 'Q4'; 'Q4'; 'Q4'];

weights_results = cell(12,4);
expected_squared_approx_error_results = cell(12,3);
%description = cell(12,3);
for i = 1:12; 
    weights_results{i,1} = months(i,:);
    weights_results{i,2} = w_opt(i,1);
    weights_results{i,3} = w_opt(i,2);
    weights_results{i,4} = w_adhoc(i,:);
    
    expected_squared_approx_error_results{i,1} = months(i,:);
    expected_squared_approx_error_results{i,2} = expected_squared_approx_error_opt(i);
    expected_squared_approx_error_results{i,3} = expected_squared_approx_error_adhoc(i);
    
    %Target to be approximated: 
    part1 = 'month when forecast is made:  ' ;
    part2 = [ months(i,:)];
    part3 = [part2 ' current year ---> '] ;
    part4 = [part3 ['( level ' target_month_end(i,:) ' ' year_end(i,:) ' ) / '] ];
    description(i,:) = [part4 ['( level ' target_month_start(i,:) ' ' year_start(i,:) ' ) - 1.'] ];
    
    
end

disp(' ');
disp('Month when forecast is made --- > Year-on-year rate to be approximated');
disp(description);

disp(' ');
disp('Month when forecast is made; optimal weights for current- and previous year forecast; ad hoc weights for current-year forecast'); 
disp(weights_results);

disp('Month when forecast is made; expected squared approx errors of optimal weights; and of ad-hoc weights'); 
disp(expected_squared_approx_error_results);


%And now the figure that maybe could be reproduced with the empirical data
ratio_expected_squared_approx_errors   = expected_squared_approx_error_opt ./ expected_squared_approx_error_adhoc;
ratio_expected_squared_approx_errors2y = expected_squared_approx_error_opt_2years ./ expected_squared_approx_error_adhoc;

figure(1);
hold on;
plot(ratio_expected_squared_approx_errors,'-bs','LineWidth',2);
plot(ratio_expected_squared_approx_errors2y,'--b+','LineWidth',2);
xlim([1 12]);
ylim([0 1]);
title('ratio of expected squared approximation errors');
legend('2 forecasts, 1 observation','2 forecasts','Location','NorthWest'); 
set(gca,'XTick',1:12);
set(gca,'XTickLabel',months);
hold off;
print('expectedsquaredapproxerror_pastyeargrowthrateknown','-dpdf');

% figure(2);
% hold on;
% plot(1-w_opt(:,1)-w_opt(:,2),'-ks','LineWidth',2);
% plot(w_opt(:,1),'-bs','LineWidth',2);
% plot(w_opt(:,2),'-rs','LineWidth',2);
% plot(w_adhoc,'--b+','LineWidth',2);
% xlim([1 12]);
% ylim([-0.2 1.2]);
% legend({'next','current','previous','ad hoc'},'Location','West'); 
% set(gca,'XTick',1:12);
% set(gca,'XTickLabel',months);
% hline(0,'-k');
% hold off;

figure(2);
hold on;
plot(1-w_opt(:,1)-w_opt(:,2),'-ks','LineWidth',2);
plot(w_opt(:,1),'-bs','LineWidth',2);
plot(w_opt(:,2),'-rs','LineWidth',2);
plot(1-w_opt_2years,'--k+','LineWidth',2);
plot(w_opt_2years,'--b+','LineWidth',2);
xlim([1 12]);
ylim([-0.2 1.2]);
% legend({'next','current','previous','next(2y)','current(2y)'},'Location','West');
set(gca,'XTick',1:12);
set(gca,'XTickLabel',months);
h = legend({'next year; 2 forecasts, 1 observation','current year; 2 forecasts, 1 observation','previous year: 2 forecasts, 1 observation','next year; 2 forecasts','current year; 2 forecasts'},'Location','West');
h.AutoUpdate = 'off';
% legend({'next year; 2 forecasts, 1 observation','current year; 2 forecasts, 1 observation','previous year: 2 forecasts, 1 observation','next year; 2 forecasts','current year; 2 forecasts'},'Location','West'); 
% hold off;
yline(0,'-k');

print('weights_pastyeargrowthrateknown','-dpdf');


