function [theta] = ar_coefs_to_ma(rho,q,p)
% gives approximate MA-coefs, based on AR-coefs
% rho: AR coefs
% q: number of MA coefs. The higher, the better is
% the approximation
% p: total number of AR-coefficients, implying zero elements added to rho if p > rows(rho). The higher, the better is
% the approximation
% q < p !

% following 
% Efficient Estimation of Parameters in Moving-Average Models Author(s): J. DurbinReviewed work(s):Source: Biometrika, Vol. 46, No. 3/4 (Dec., 1959), pp. 306-316
% as described in
% A Simple Noniterative Estimator for Moving Average Models Author(s): John W. Galbraith and Victoria Zinde-WalshReviewed work(s):Source: Biometrika, Vol. 81, No. 1 (Mar., 1994), pp. 143-155

n = numel(rho);
m = p - n;
a = [rho ; zeros(m,1)];
Q = zeros(p,q);

for i = 1:q
    Q(i:p,i) = [ 1 ; -a(1:p-i)];
end

theta = lscov(Q,a);
        
