% Procedure based on Bai&Ng, JBES(2005), translated from GAUSS and adapted
% for regression analysis (not just estimation of means)

% test statistic for parameter b is b_hat/sqrt(hac)

function [hac,k] = nw_new_nossc(u,x,prewhite,k)
% k is number of lags to be used
% if k is not provided or equals -1, it is determined automatically (Andrews 1991)
% if prewhite == 0 and k == 0, vcv-matrix equals standard vcv-matrix known from OLS 

n = rows(u);
nreg = cols(x);
% rho = zeros(nreg,1);
sigma = zeros(nreg,1);
% d = zeros(nreg,nreg);
beta = zeros(nreg,nreg);

% do a VAR(1) prewhitening
if prewhite == 1;
    v = zeros(n-1,nreg);
    reg = (u(1:n-1)*ones(1,nreg)).*x(1:n-1,:);
    i = 1;
    while i<=nreg;
        beta(:,i) = lscov( reg , u(2:n).*x(2:n,i) );    
        v(:,i) = u(2:n).*x(2:n,i) - reg * beta(:,i);
        i = i + 1;
    end;
else
    v = (u*ones(1,nreg)).*x;
end;

rho = zeros(nreg,1);
i = 1;
while i<=nreg;
    rho(i) = lscov( v(1:rows(v)-1,i) , v(2:rows(v),i) );
    r = v(2:rows(v),i) - v(1:rows(v)-1,i) * rho(i);
    sigma(i) = r'*r / rows(v);
    i = i + 1;
end;

if (nargin < 4)||(k==-1);
    bot = 0; top = 0;
    i = 1;
    while i<=nreg;
        top = top + 4*(rho(i)^2)*sigma(i)^2 / (((1-rho(i))^6)*(1+rho(i))^2);
        bot = bot + sigma(i)^2 / ((1-rho(i))^4);
        i = i+1;
    end;   
    alpha = top/bot;   
    k = ceil(1.1447*(alpha*n)^(1/3));
end

if k > n/2; 
    k = n/2;
end;

%disp('truncation lag'); 
%disp(k);
vcv = v' * v  ;
%disp(vcv);

i = 1;
while i<=k;
    % x = i/k;
    w = 1 - i/(k+1);
      
    cov1 = v(i+1:n-prewhite,:)' * v(1:n-prewhite-i,:) ;     
    cov2 = v(1:n-prewhite-i,:)' * v(i+1:n-prewhite,:) ;
   
    vcv = vcv + w*(cov1 + cov2);
    i = i + 1;
end;


d = inv(eye(nreg)-beta');
    
hac = d*vcv*d';
    
qmat = x' * x;
    
hac = inv(qmat) * hac * inv(qmat) ;
    





