
clearvars -except data_download_time quarters_known vari cutoff cutoff_start expflag results_all weights_all sample_choice;


if vari == 1
    export_sheet = 'PGDP';
elseif vari == 2
    export_sheet = 'RGDP';
end 
all_data = xlsread(['.\DataOutput\observations_and_forecasts_' data_download_time '.xlsx'],export_sheet,'a2:z400');
date_vec = all_data(:,1);
forecasts_annual_growth = all_data(:,16:17);
forecasts_yoy           = all_data(:,10:14);
% annualized qoq 2nd vintage data would be given by
vintage2_qoq = all_data(:,2);

%plot([forecasts_annual_growth(64:end-cutoff,1:2) forecasts_yoy(64:end-cutoff,5)]);

for approach_type = 0:4 % FEASIBLE:   0: iid process, 1: AR-processes, recursive estimation, 2: AR-processes, full sample 
                        % INFEASIBLE (requires fh-forecasts) 3: AR(1)-process, optimal rho, 4: optimal weights without AR-process
        
    disp('approach_type');
    disp(approach_type);
    if (approach_type == 1)||(approach_type == 2)
        pvec = [1 2 4] ;
    elseif (approach_type == 0)
        pvec = 0;
    elseif (approach_type == 4)
        pvec = 99;
    elseif approach_type == 3
        pvec = 1;
    end

    %from 1981q3 onwards
       
    w_adhoc_vec = [0.75;0.5;0.25;0.0];
    disp(export_sheet);
    counter_p = 0;
    T = rows(forecasts_yoy)-cutoff;    
    t1 = 64+cutoff_start;
    start_quarter = mod(t1+2,4)+1 ;
    
    forecast_target = forecasts_yoy(t1:T,5);
    forecasts_annual_growth_c_q = highfrequency_vector_to_years_times_x_matrix(forecasts_annual_growth(t1:T,1),4,start_quarter);
    forecasts_annual_growth_n_q = highfrequency_vector_to_years_times_x_matrix(forecasts_annual_growth(t1:T,2),4,start_quarter);
    forecast_target_q           = highfrequency_vector_to_years_times_x_matrix(forecast_target                ,4,start_quarter);
                 
    for p = pvec       
        current_year = 1;
        counter_p = counter_p + 1;    
        clearvars w_opt_vec rhovec mse_opt mse_adhoc;
        
        if (approach_type == 0)  
            rhovec = 0;
            for current_quarter = 1:4
                w_opt_vec(current_quarter,1) =   optweights_ARp(4,rhovec,current_quarter+quarters_known,current_quarter+4,4);
            end                
        elseif (approach_type == 2)           
            data_sample = vintage2_qoq(1:T);
            [ b, ~, ~ ,~ ] = estimarp( data_sample , p );
            rhovec = b(2:end);                       
            for current_quarter = 1:4
                w_opt_vec(current_quarter,1) =   optweights_ARp(4,rhovec,current_quarter+quarters_known,current_quarter+4,4);
            end
        elseif (approach_type == 4)  
            for current_quarter = 1:4
               yreg = forecast_target_q(:,current_quarter) - forecasts_annual_growth_n_q(:,current_quarter);
               xreg = forecasts_annual_growth_c_q(:,current_quarter) - forecasts_annual_growth_n_q(:,current_quarter);
               yreg = yreg(~isnan(yreg));
               xreg = xreg(~isnan(xreg));           
               [bb(1,current_quarter) , bb(2,current_quarter)]= lscov(xreg,yreg);    
               u = yreg - xreg * bb(1,current_quarter);
               w_opt_vec(current_quarter,1) = bb(1,current_quarter);
               [hac,k] = nw_new_nossc(u,xreg,0,-1);
               bb(3,current_quarter) = sqrt(hac);
            end                
        elseif (approach_type == 3) 
            rho_vec_ar1 = -0.99:0.01:0.99;
            counter_rho = 0;
            % disp('searching over rho_vec...');
            for rho = rho_vec_ar1                 
                counter_rho = counter_rho + 1;
                for current_quarter = 1:4 
                    w_opt_vec_prel(current_quarter,1) = ( optweights_ARp(4,rho,current_quarter+quarters_known,current_quarter+4,4) )  ; 
                end
                [mse_opt,mse_adhoc,~,~,~,~,~,~] = calc_mses(forecasts_annual_growth(t1:T,:),forecast_target,start_quarter,w_opt_vec_prel,w_adhoc_vec);
                mserat_rho_vec(counter_rho) = mse_opt/mse_adhoc;
            end
            [~,minind] = min(mserat_rho_vec);
            rhovec = rho_vec_ar1(minind);
            for current_quarter = 1:4 
                w_opt_vec(current_quarter,1) = ( optweights_ARp(4,rhovec,current_quarter+quarters_known,current_quarter+4,4) )  ; 
            end

        elseif (approach_type == 1)              
            counter = 0; 
            rhovec_all = [];
            for t = t1:T
                current_quarter = mod(t+2,4)+1;
                counter = counter + 1;         
                data_sample = vintage2_qoq(1:t);                  
                [ b, ~, ~ ,~ ] = estimarp( data_sample , p );
                rhovec = b(2:end); 
                w_opt =   optweights_ARp(4,rhovec,current_quarter+quarters_known,current_quarter+4,4);
                w_opt_vec(counter,1) = w_opt;
                rhovec_all(counter,:) = rhovec';
            end                      
        end
        
                                               
        date_t = date_vec(t1:T);
        %Note: w_opt_vec is allowed to have 2 different dimensions: 4x1 or Nx1, where N is the sample size. The calc_mses function can handle both cases
        [mse_opt,mse_adhoc,mse_opt_q,mse_adhoc_q,approx_opt,approx_adhoc,weights_opt_all,weights_adhoc_all] = calc_mses(forecasts_annual_growth(t1:T,:),forecast_target,start_quarter,w_opt_vec,w_adhoc_vec);
        
        w_opt_all_q                 = highfrequency_vector_to_years_times_x_matrix(weights_opt_all                ,4,start_quarter);
        w_adhoc_all_q               = highfrequency_vector_to_years_times_x_matrix(weights_adhoc_all              ,4,start_quarter);
        
        if sample_choice == 1 % only make plot for full sample
            if approach_type == 0
                year_start = floor(date_t(1));
                year_end = floor(date_t(end));
                month_start = ( (date_t(1)   - year_start) + 0.25 ) * 12 - 1;
                month_end   = ( (date_t(end) - year_end)   + 0.25 ) * 12 - 1;
                tt1 = datetime(year_start  ,  month_start  ,1);
                tt2 = datetime(year_end    ,  month_end  ,1);
                tt = tt1:calmonths(3):tt2;
                figure(vari);
                hold on;
                plot(tt,[forecast_target ],'-k', 'LineWidth',1);
                plot(tt,[ approx_opt  ], '-b','LineWidth',1);
                plot(tt,[  approx_adhoc ], '-r','LineWidth',1);            
                hold off;

                datetick('x','mmm-YY');              
                %xticks(tt1:calmonths(60):tt2);
                xlim([min(tt) max(tt)]);            
                legend('True fixed-horizon forecast','Optimal approximation (k=0)','Ad-hoc approximation','Location','northwest','Box','off');
                if vari == 1
                    title('US SPF - Inflation Mean','FontSize',10,'FontWeight','Normal');
                elseif vari == 2
                    title('US SPF - GDP Growth Mean','FontSize',10,'FontWeight','Normal');
                end

                if expflag == 1
                    saveas(gcf,['./plots/exante_rho0_' export_sheet '_cutoff_start_' num2str(cutoff_start) '_cutoff_' num2str(cutoff)  '.pdf']);
                end           
            end
        end
        
%         disp('AR order');
%         disp(p);
%         disp('mse_opt mse_adhoc ratio(mse_opt/mse_adhoc)');
%         disp([mse_opt mse_adhoc mse_opt/mse_adhoc]);
        % full_sample
       mserat   = mse_opt/mse_adhoc;
       mserat_q = mse_opt_q./mse_adhoc_q;
       if (approach_type == 1)
%            disp('mean and stddev of rhovec_all');
%            disp(nanmean(rhovec_all));
%            disp(nanstd(rhovec_all));
%            disp('mean and stddev of optimal weights');
%            disp(nanmean(w_opt_all_q));
%            disp(nanstd(w_opt_all_q));
           %[sqrt_hac_rowvec] = nanstdhac(y);
           % disp(nanstdhac(w_opt_all_q));

           weights_all = [weights_all; [vari approach_type p 0 nanmean(w_opt_all_q)]  ];
           weights_all = [weights_all; [vari approach_type p 1 nanstd(w_opt_all_q)]  ];
           weights_all = [weights_all; [vari approach_type p 2 NaN*nanmean(w_opt_all_q)]  ];
           %weights_all = [weights_all; [vari approach_type p 2 nanstdhac(w_opt_all_q)]  ];
       elseif (approach_type == 4)
%            disp('mean and stddev [a) simple OLS, b) HAC] of LS weights');
%            disp(bb);       
           weights_all = [weights_all; [vari approach_type p 0 bb(1,:)]  ];
           weights_all = [weights_all; [vari approach_type p 1 bb(2,:)]  ]; % OLS std dev
           weights_all = [weights_all; [vari approach_type p 2 bb(3,:)]  ]; % HAC std dev
       else 
%            disp('rhovec');
%            disp(rhovec');
%            disp('optimal weights');
%            disp(w_opt_vec');    
%            weights_all = [weights_all; [vari approach_type p 0 w_opt_vec']  ];
       end 
       results_all = [results_all ; [vari approach_type p mserat_q mserat]];
       
    end   
end
    
