clc;
clearvars;
close all
% Determine where your m-file's folder is.
folder = fileparts(which(mfilename)); 
% addpath Functions;
addpath(genpath(folder)); % Add that folder plus all subfolders to the path.
data_download_time = '2023_May';
%data_download_time = '2022_October';
make_real_time_data_levels_quarterly;
merge_data_and_forecasts_3;

for sample_choice = 1:3
    disp('-------------------------------------------------------------------');
    disp('-------------------------------------------------------------------');
    if sample_choice == 1      
        disp(' --------------------------- Full sample --------------------------');     
        cutoff = 0;       % quarters to cut off at end
        cutoff_start = 0; % quarters to cut off at start
    elseif sample_choice == 2      
        disp(' ---------------------- pre-Covid sample --------------------------');
        cutoff = 14;       % quarters to cut off at end
        cutoff_start = 0; % quarters to cut off at start
    elseif sample_choice == 3
        disp(' ------------------------ Consensus sample ------------------------');
        cutoff = 16;       % quarters to cut off at end
        cutoff_start = 30; % quarters to cut off at start
    end
    disp('-------------------------------------------------------------------');
    disp('-------------------------------------------------------------------');
    % cutoff = 0;  cutoff_start = 0;  corresponds to the full sample 1981Q3 to 2023Q2
    % cutoff = 14; cutoff_start = 0;  corresponds to the pre-Covid sample 1981Q3 to 2019Q4
    % cutoff = 16; cutoff_start = 30; corresponds to the Consensus sample 1989Q1 to 2019Q2 (The Consensus sample ends in May 2019, ie in 2019Q2, but the fixed-horizon forecasts from 2019Q2 are not part of the sample, since they were published in June 2019)

    quarters_known = -1; % 0 means current quarter known, -1 means only previous quarter known
    expflag = 1;

    results_all = [];
    weights_all = [];
    for vari = 1:2
        apply_weights2;
    end

    disp('--------------------------------------------------------------');
    disp('------------------------- MSE RATIOS -------------------------');
    disp('--------------------------------------------------------------');
    disp(['--- observations cut off at start: ' num2str(cutoff_start) '. observations cut off at end: ' num2str(cutoff) ' ---']);
    disp('--------------------------------------------------------------');
    disp('1st col: variable. 1 = inflation, 2 = growth');
    disp('2nd col: approach. 0 = ex-ante approach with k=p=0');
    disp('                   1 = ex-ante approach with k=p');
    disp('                   2 = ex-post approach with k=p');
    disp('                   3 = infeasible approach with k=p=1');
    disp('                   4 = infeasible approach, unrestricted');
    disp('3rd col: p, i.e. number of autoregressive coefs in DGP. 99 means NaN (for infeasible approach, unrestricted)');
    disp('4th-7th col: MSE ratios of optimal vs ad-hoch weights for quarters 1,2,3,4');
    disp('8th col: MSE ratio of optimal vs ad-hoch weights for all observations');
    disp(results_all);

    disp('--------------------------------------------------------------');
    disp('-------------------- current-year weights --------------------');
    disp('--------------------------------------------------------------');
    disp(['--- observations cut off at start: ' num2str(cutoff_start) '. observations cut off at end: ' num2str(cutoff) ' ---']);
    disp('--------------------------------------------------------------');
    disp('1st col: as for results');
    disp('2nd col: as for results');
    disp('3rd col: as for results');
    disp('4th col: 0: weights (in case of time-variation: average weights) in following columns');
    disp('         1: standard deviations of weights in following columns if approach = 1, OLS standard errors of weights if approach = 4');
    disp('         2:                                                                      HAC standard errors of weights if approach = 4');
    disp('5th-8th col: the values indicated in 4th column for quarters 1,2,3,4');
    disp(weights_all);


    xlswrite(['.\Results\MSEratios.xlsx'],[{'vari'} {'approach_type'} {'AR-order'} {'MSEratQ1'} {'MSEratQ2'} {'MSEratQ3'} {'MSEratQ4'} {'MSEratALL'}],['results_' num2str(cutoff_start) '_'  num2str(cutoff)],'a1');
    xlswrite(['.\Results\MSEratios.xlsx'],results_all,['results_' num2str(cutoff_start) '_'  num2str(cutoff)],'a2');
    xlswrite(['.\Results\weights.xlsx'],[{'vari'} {'approach_type'} {'AR-order'} {'mean/stderr'} {'w_Q1'} {'w_Q2'} {'w_Q3'} {'w_Q4'}],['weights_' num2str(cutoff_start) '_'  num2str(cutoff)],'a1');
    xlswrite(['.\Results\weights.xlsx'],weights_all,['weights_' num2str(cutoff_start) '_'  num2str(cutoff)],'a2');

end